#include "bvpardialog.h"
#include "ui_bvpardialog.h"
#include "floatdelegate.h"

extern "C" void jav_edit_bvpar_(int *nbvec, bond_valence_t *bvec, int *act);

BvParDialog::BvParDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::BvParDialog)
{
    ui->setupUi(this);
    bvParModel = new QStandardItemModel(this);
    QStringList bvParLabels = {"Atom1", "Ox.St.1", "Atom2", "Ox.St.2", "Ro", "B", "dmin", "dmax"};
    bvParModel->setHorizontalHeaderLabels(bvParLabels);
    ui->bvParTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->bvParTableView->setModel(bvParModel);
    ui->bvParTableView->verticalHeader()->hide();
    ui->bvParTableView->setShowGrid(false);
    ui->bvParTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->bvParTableView->setSelectionBehavior(QAbstractItemView::SelectItems);
    ui->bvParTableView->setAlternatingRowColors(true);
    //ui->bvParTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);

    FloatDelegate *bvparDelegate = new FloatDelegate(ui->bvParTableView);
    bvparDelegate->setDecimals(3);
    ui->bvParTableView->setItemDelegateForColumn(4, bvparDelegate);
    ui->bvParTableView->setItemDelegateForColumn(5, bvparDelegate);
    ui->bvParTableView->setItemDelegateForColumn(6, bvparDelegate);
    ui->bvParTableView->setItemDelegateForColumn(7, bvparDelegate);
}

void BvParDialog::on_edit_bvs(const Model *model)
{
    md = model;
    int n = (md->numSpec*(md->numSpec+1))/2, action=1;    
    bvec = new bond_valence_t[n];      

    jav_edit_bvpar_(&n, bvec, &action);    

    FillLista(n,bvec);
}

BvParDialog::~BvParDialog()
{
    delete ui;
}

void BvParDialog::reject()
{
    QDialog::reject();
    delete [] bvec;
}

void BvParDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok)
    {
        ReadLista(bvec);
        int n = bvParModel->rowCount();        
        int action = 2;
        jav_edit_bvpar_(&n,bvec,&action);
        delete [] bvec;

    } else if (stdButton == QDialogButtonBox::RestoreDefaults) {
        int n = bvParModel->rowCount();        
        int action = 3;
        jav_edit_bvpar_(&n,bvec,&action);

        FillLista(n, bvec);
    }
}

void BvParDialog::ReadLista(bond_valence_t *bvec)
{
    for (int i = 0; i < bvParModel->rowCount(); i++) {
        QModelIndex index = bvParModel->index(i,1,QModelIndex());
        bvec[i].val1 = bvParModel->data(index).toInt();
        index = bvParModel->index(i,3,QModelIndex());
        bvec[i].val2 = bvParModel->data(index).toInt();
        index = bvParModel->index(i,4,QModelIndex());
        bvec[i].ro = bvParModel->data(index).toFloat();
        index = bvParModel->index(i,5,QModelIndex());
        bvec[i].b = bvParModel->data(index).toFloat();
        index = bvParModel->index(i,6,QModelIndex());
        bvec[i].rmin = bvParModel->data(index).toFloat();
        index = bvParModel->index(i,7,QModelIndex());
        bvec[i].rmax = bvParModel->data(index).toFloat();
    }
}

void BvParDialog::FillLista(int n, bond_valence_t *bvec)
{
    QList<QStandardItem *> *bvpList;
    bvpList = md->FillBondValenceParameters(n, bvec);
    bvParModel->setRowCount(0);
    for(int row = 0; row < n; row++)
        bvParModel->appendRow(bvpList[row]);
}
