#ifndef CODDIALOG_H
#define CODDIALOG_H

#include "qtdownload.h"
#include "itemdelegate.h"
#include "checkboxdelegate.h"
#include "checkboxheader.h"
#include "networkdownload.h"

#include <QDialog>
#include <QString>
#include <QStandardItemModel>
#include <QAction>
#include <QRect>
#include <QAbstractButton>

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif

namespace Ui {
class CODDialog;
}

class CODDialog : public QDialog
{
    Q_OBJECT

public:
    explicit CODDialog(QWidget *parent = nullptr, int WhichModel = 1, int kadd = 0);
    void showCOD(int WhichModel, int kadd);
    ~CODDialog();

private slots:
    void on_searchButton_clicked();
    void jsonDownloadComplete(bool success, QString errMsg);
    void downloadCompleteCIF(bool success, const QString &errMsgFile);
    void tableView_currentRowChanged(const QModelIndex & current, const QModelIndex & previous);
    void tableView_dataChanged(const QModelIndex & current, const QModelIndex & previous);
    void customMenuRequested(QPoint pos);
    void copy();
    void download();
    void myOpenUrl();
    void editingCompleted();
    void textingLine(QString line);
    void on_downloadButton_clicked();
    void on_buttonBox_clicked(QAbstractButton *button);
    void on_checkBoxClicked();

private:
    Ui::CODDialog *ui;

    QString m_ID;
    QString m_text;
    QString m_formula;
    QString m_el;
    QString m_nel;
    QString m_strictmin;
    QString m_strictmax;
    QString m_amin;
    QString m_amax;
    QString m_bmin;
    QString m_bmax;
    QString m_cmin;
    QString m_cmax;
    QString m_alpmin;
    QString m_alpmax;
    QString m_betmin;
    QString m_betmax;
    QString m_gamin;
    QString m_gamax;
    QString m_vmin;
    QString m_vmax;
    QString m_minZ;
    QString m_maxZ;
    QString m_minZprime;
    QString m_maxZprime;
    QString m_journal;
    QString m_year;
    QString m_volume;
    QString m_issue;
    QString m_doi;
    QtDownload *jsonDownloder;
    QtDownload *CIFdownloder;
    QStandardItemModel *model;
    QAction *m_copyAct;
    QAction *m_downloadAct;
    QAction *m_openUrl;
    QModelIndex indexMenu;
    QString m_fileCIFSelected;
    int m_WhichModel;
    int m_kadd;
    QRect geometry;
    CheckBoxDelegate *delegate;
    CheckBoxHeader *cbHeader;
    NetworkDownload *tableViewDownloder;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif

    enum colPos { COLPOS_CHECK, COLPOS_ID, COLPOS_SG, COLPOS_FORM, COLPOS_VOL,
                  COLPOS_CELL, COLPOS_NAME, COLPOS_BIB, NUM_COLS
    };

    QString getElements(QString s, QString w, int lim);
    void addItemsToQuery(QUrlQuery& q, QString s, QString w, int lim);
    QUrl CODCommand();
    bool form_per_COD(const QString &form1, QString &form2);
    int fillList(QString &errMsg);
    void showCrystalDetails(const QModelIndex &index);
    void clearResults();
    QString getFileName(const QModelIndex &index);
    bool linesEmpty();
};

#endif // CODDIALOG_H
