MODULE datamod

   implicit none

CONTAINS

   subroutine gener(wave,iflagref,kpr)
   USE Riflessi
   USE General, only:ihmx,rhomin,rhomax,dtor,lo,cell,nrefl, &
                     knw,spacegr,alambda,is_anomalous
   USE counts, only: dvalue
   USE variables, only: cryst,ref,dataset
   USE PatternRef, only: thmax,thmin
   USE reflection_type_util
   USE testmodel, only: atmpub
   implicit none
   real, dimension(:), intent(in)                   :: wave
   integer, intent(out)                             :: iflagref
   logical, intent(in)                              :: kpr
   integer                                          :: i,ier
   type(reflection_type), dimension(:), allocatable :: refob
   integer                                          :: nrefob
!
   rhomax = 1. / (2*dvalue(thmax,alambda))**2
   rhomin = 1. / (2*dvalue(thmin,alambda))**2
!
   iflagref = 0
   call create_reflections(thmin,thmax,cell(:6),spacegr,wave,refob,nrefob,ihmx)
!
   call AllocaRiflessi(2)
   nrefl = nrefob
   if (nrefl == 0) return
   call AllocaRiflessi(1)
   if (knw > 0) then
       call manage_filefc(refob,ier)
       if (ier /= 0) knw = 0
   endif
   do i=1,nrefl
      IndHKL(i,1:3) = refob(i)%hkl(:)
      IndHKL(i,4) = 1
!corr      IndHKL(i,5) = refob(i)%phv
      IndHKL(i,5) = refob(i)%jcode + refob(i)%phv*32
      SetFob(i,1) = 0.0
      SetFob(i,2) = 0.0
      SetFob(i,3) = 0.0
      SetFob(i,4) = 0.0001
      TThet (i)   = refob(i)%tthd(1)
      Mult  (i)   = refob(i)%m
      WHM   (i)   = 0.0
   enddo
   if (knw > 0) fxsav(:) = refob(:nrefl)%fv   ! :nrefl because refob is not reallocated
   nrefl = nrefob  ! set global varible nrefl after AllocaRiflessi
!
   if (nrefl > 3000) iflagref = 1             ! segnala che ci sono + di 3000 riflessi
   if (kpr) then
       write(lo, '(16x,"*",7x,"Generated reflections information",10x,"*")')
       write(lo, '(16x,"*",50x,"*")')
       write(lo, '(16x,"*",9x,"rho min         = ",f8.5,15x,"*")') rhomin
       write(lo, '(16x,"*",9x,"rho max         = ",f8.5,15x,"*")') rhomax
       write(lo, '(16x,"*",9x,"d (resolution)  = ",f8.5,15x,"*")') dvalue(thmax,alambda)
       write(lo,'(16x,"*",9x,"reflections nr. = ",i8,15x, "*",/,16x,"*",9x,"h k l (max)     = ",3i4,11x,"*",/,16x,52("*"))') &
       nrefl,(ihmx(i),i=1,3)
   endif
   call cryst(1)%set_reflections(refob)
   if (knw > 0) call cryst(1)%set_as_known(atmpub,dataset(1)%radtype,is_anomalous)
   call copy_ref(ref,refob)
   end subroutine gener

!-----------------------------------------------------------------------

   subroutine manage_filefc(refob,ier)   
   USE General, only:nrefl, alambda, fname, FC_FILE, &
                     dtor, neutro, celem, lo, spacegr, is_anomalous
   USE filereading
   USE fileutil
   USE TestModel
   USE reflection_type_util
   USE atom_type_util
   USE strutil
   USE elements, only:H_at, NEUTRON_SOURCE
   USE connect_mod
   USE errormod
   USE fra_frm
   USE spginfom
   USE variables, only: cryst
   USE gen_frm
   USE unit_cell
   implicit none
   type(reflection_type), dimension(:), intent(inout) :: refob
   integer, intent(out)                               :: ier
   type(reflection_type), allocatable, dimension(:)   :: reffc
   type(error_type)                                   :: error
   integer                                            :: nrfc
   integer                                            :: fase
   integer                                            :: i,j
   type(bond_type), dimension(:), allocatable         :: legpub
   integer, dimension(3)                              :: hkl
   integer                                            :: is
   type(spaceg_type)                                  :: spgfc
   type(cell_type)                                    :: cellpub
   logical                                            :: is_bfac,is_occ,has_symmetry
!
   ier = 0
   if (s_eqi(get_extension(fname(FC_FILE)),'fc')) then    ! read F from .fc file
       refob(:)%fv = 0
       call read_reflection_file(fname(FC_FILE),reffc,nrfc,'fc',error)
       if (error%signal) then
           call error%print()
           ier = 1
       else
           do i=1,nrfc
              loop_sym: do is=1,spacegr%nsymop
                 !hkl(:) = matmul(reffc(i)%hkl,kmat(is,:,:))
                 hkl(:) = matmul(reffc(i)%hkl,spacegr%symop(is)%rot)
                 do j=1,nrefl
                    if (all(hkl == refob(j)%hkl)) then
                        fase=mod(reffc(i)%ph,360)
                        if(fase <= 0)fase=fase+360
                        refob(j)%fv = reffc(i)%fc
                        refob(j)%phv=fase
                        exit loop_sym
                    endif
                 enddo
              enddo loop_sym
           enddo
!
!          read file .fra
           call read_fra_file(file_rem_ext(fname(FC_FILE))//'.fra',atmpub,is_bfac,is_occ,error)
           if (error%signal) then
               write(lo,'(a)')trim(error%message)
               write(lo,'(a)')'Match between Expo model and fragment is not possible!'
               return
           endif
           call specie_from_ptab(atmpub,celem)
       endif
   else                                                   ! try to read structure file
       allocate(reffc(nrefl))
       reffc(:) = refob(:nrefl)
       reffc(:)%slaq = (sin(0.5*refob(:nrefl)%tthd(1)*dtor)/alambda)**2
       has_symmetry = .false.
       call import_crystal(fname(FC_FILE),ALL_FILES,atmpub,legpub,celem,cellpub,spgfc,  &
                           has_symmetry,alambda,neutro,.false.,.false.,error)
       call cryst(1)%set_scattering(celem) ! update elements, published model could have different contents
       if (error%signal) then
           ier = 1
       else           ! now compute fc
           call fcalcang(reffc,atmpub,spacegr,celem,neutro,is_anomalous)
           do i=1,nrefl
              fase=mod(reffc(i)%ph,360)
              if(fase <= 0)fase=fase+360
              refob(i)%fv=reffc(i)%fc
              refob(i)%phv=fase
           enddo
           if (neutro /= NEUTRON_SOURCE) call remove_atoms_from_list(atmpub,atmpub%ptab,H_at)  ! remove H atoms
       endif
   endif
!
   end subroutine manage_filefc

!-----------------------------------------------------------------------

   subroutine viewrefcell(ij) bind(C,name="viewrefcell")
!
!  Display reflections corrisponding to selected cell, unindexed peaks.
!  Shift is applied to peaks and pattern
!
   USE Symmetry, only: ifRhombAx
   USE Conteggi, only: theta_int,npunti
   USE indexing
   USE General, only: spacegr,cell,ifAutomatic,alambda
   USE VIEW
   USE strutil
   USE messagemod
   USE arrayutil
   USE molcom, only:jscreen
   USE spginfom, only: init_spaceg_type, rhomb_axes
   USE variables, only: cryst,dataset,ref
   USE unit_cell
   USE peak_mod, only: pkind
   USE plotstyle
   use peak_util
   use iso_c_binding 
   implicit none
   integer(c_int), intent(in), value :: ij
   common /grafica_ntreor/ibest,cellsav(6,660),ameff(660),        &
          m20sav(660),shiftsav(660),Noindsav(660),nobssav(660),   &
          krefinc(660)
   integer :: ibest,m20sav,Noindsav,nobssav,krefinc
   real    :: cellsav,ameff,shiftsav
   real, dimension(6) :: celtemp
   character(len=80) :: line
   integer :: ishif,ix,iflagref
   integer, dimension(3) ::  kshif 
   integer :: ier
   integer :: i,npund
!
   celtemp(:) = cell_list(ij)%par(:)
   cell(:6) = celtemp(:)
   call cryst(1)%set_unit_cell(set_cell_type(cell))
   call frm01( (/(celtemp(i),i=1,6),(0.0,i=7,13)/) )
!
   select case (cell_list(ij)%sys)
      case ('TR')
          line= 'P -1'
      case ('MO')
          line= 'P 2/m'
      case ('HE')
          line= 'P 6/m m m'
      case ('OR')
          line= 'P m m m'
      case ('TE')
          line= 'P 4/m m m'
      case ('CU')
          line= 'P m -3 m'
   end select
   call lcase(line)
   spacegr = init_spaceg_type(line)
   call cryst(1)%set_spg(spacegr)
   call StandardSpg(line,ier)
   ishif = 0
   kshif(:) = 0
   ifRhombAx = 0
   if (rhomb_axes(spacegr%symbol_xhm,celtemp)) ifRhombAx=1
   call insym(ishif,kshif,ier)
   call yfrm02
   call sc014(ix)
   call xspinb
   call gener(dataset(1)%wave,iflagref,.false.)
   if (jscreen > 0 .and. ifAutomatic == 0) then
!
       call end_viewrefcell()  ! undo previous shift
!
!      Apply shift to pattern and peaks; save shift value
       shift_ntreor = shiftsav(ij)
       shift_applied = .true.
       do i=1,npunti
          theta_int(i,7)=theta_int(i,7) - shiftsav(ij)
       enddo
       do i=1,numpeaks(pkind)
          call pkind(i)%setxd(pkind(i)%getx()-shiftsav(ij),alambda)
       enddo
!           
!      Identify unindexed lines
       select case(cell_list(ij)%pcode)
         case (NTREOR09_PROG)
           npund = min(20,numpeaks(pkind))
         case (DICVOL_PROG)
           !npund = min(20,numpeaks(pkind))
           !npund = min(npund,indx_set(cell_list(ij)%pcode)%nlines) ! check with number of peaks from GUI
!
!          Dicvol output contains the number of indexed peaks calculated on the total number of peaks
           npund = numpeaks(pkind)
         case (MCMAILLE_PROG)
           npund = min(indx_set(MCMAILLE_PROG)%nlines,numpeaks(pkind))
       end select
       call find_unindexed(pkind,npund,ref,cell_list(ij)%nund)
       !call get_unindexed_peaks(xund,yund,nund)
       !if (nund > 0) call add_plot(xund,yund,nund,Unindexed_Peaks,1,dataset(1)%wave(1),char(0))
       !call display_unindexed_peaks()
!
       call vedinew(8,lsetstyle=.false.,rescale=0)
       call write_message('The reflections generated from cell #'//trim(i_to_s(ij))//' have been visualized.',pos=2)
       call write_message('Please check their agreement with the '//'observed pattern.',pos=3)
   endif
   end subroutine viewrefcell

!-----------------------------------------------------------------------

   subroutine update_content(new_content,ier)
   use elements
   use General, only: neutro, lo, celem
   use variables, only: cryst, dataset
   character(len=*), intent(in)                  :: new_content
   integer, intent(out)                          :: ier
   type(element_type), dimension(:), allocatable :: elem
!
   ier = 0
!
!  Convert string in new cell content
   call read_chemformula(new_content,ier,elem=elem)
   if (ier > 0 .or. numelem(elem) == 0) return

   call elem_set_radiation(elem,neutro)
   call ordina_elements(elem,neutro)
!
!  update content if it has changed in type or number
   if (.not.equal_elements(celem,elem)) then
       call copy_elem(celem,elem)
       call cryst(1)%set_scattering(celem)
       call cryst(1)%set_strelem(new_content)
       call print_elements(celem,lo,neutro,dataset(1)%wave(1))
   endif

   end subroutine update_content

!-----------------------------------------------------------------------

   subroutine update_cell(new_cell, updated)
   use variables, only: cryst
   use general, only: cell
   use unit_cell
   real, dimension(6), intent(in) :: new_cell
   logical, intent(out)           :: updated
   integer                        :: i
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   real :: tol = 1.0e-4

   updated = .false.
   if (.not.cryst(1)%is_cell()) then
!      cell may be undef. if Import xpd > Pattern > Space Group Determ.
       cell(1:6) = new_cell(:)
       call frm01( (/(new_cell(i),i=1,6),(0.0,i=7,13)/) )
       call cryst(1)%set_unit_cell(set_cell_type(new_cell))
       iesdircell = 1
       updated = .true.
   else
       do i=1,6
          if (abs(new_cell(i) - cell(i)) > tol) then
              cell(i) = new_cell(i)
              updated = .true.
          endif
       enddo

       if (updated) then
           call frm01( (/(new_cell(i),i=1,6),(0.0,i=7,13)/) )
           call cryst(1)%set_unit_cell(set_cell_type(new_cell))
           cryst(1)%is_symm_user = .true.
           iesdircell = 1
       endif
   endif

   end subroutine update_cell

!---------------------------------------------------------------------------

   subroutine open_pattern(input_file,output_file,add_data,ier)
   USE fileutil
   USE General, only: StructureName, fname, ext, INP_FILE, OUT_FILE
   USE datasetmod
   USE datautil
   USE molcom, only:ifProject
   USE strutil
   USE General, only:lo
   USE variables, only: dataset,cryst
   implicit none
   character(len=*), intent(in) :: input_file,output_file
   integer, intent(in), value   :: add_data
   integer, intent(out)         :: ier
   integer                      :: iflag
   integer                      :: itype, iform, nlenn, jop, jopen, ier2
   integer                      :: i,ierc
   type(dataset_type)           :: datas
!
   ier = 0
   call load_datafile(datas,input_file,.true.,ier) 
   if (ier == 0) then
       if (add_data == 0) then
           if (ifProject.ne.0) then   ! importante se esegui run successivi
               iflag = 2
!              comment the next line to add pattern
               call chiudi_bin(iflag) ! call InitExpo
           endif
           StructureName = file_rem_ext(file_get_name(input_file))
           call cryst(1)%set_name(StructureName)
!
           do i=1,size(fname)
              if (i /= INP_FILE .and. i /= OUT_FILE) then
                  fname(i) = s_blank_delete(trim(StructureName)//ext(i))
              endif
           enddo
           fname(INP_FILE) = input_file
           fname(OUT_FILE) = output_file
!
           close(lo,iostat=ierc)
           itype = 0
           iform = 1
           nlenn = 80
           jop = jopen(lo,fname(OUT_FILE),itype,iform,ier2)
!
           call clear_dataset(dataset)
       endif
       call push_back_dataset(dataset,datas)
       call work_on_pattern(dataset(1),add_data)
   endif
!
   end subroutine open_pattern

!---------------------------------------------------------------------------------

   subroutine abilita_tasti2()
   use variables
   use enable_amb
   use datasetmod
   logical :: powder, structure

   powder = .false.
   structure = .false.
   if (ndataset(dataset) > 0) then
       powder = dataset(1)%npoints() > 0 .and. dataset(1)%datatype == POW_DATA
   endif
   structure = cryst(1)%natoms() > 0
    
   if (structure .and. powder) then
       call abilita_tasti(PatternAndStructureAction)
       return
   endif
   if (structure) then
       call abilita_tasti(StructureAction)
       return
   endif
   if (powder) call abilita_tasti(PatternAction)

   end subroutine abilita_tasti2

!---------------------------------------------------------------------------------

!corr
!corr   subroutine get_unindexed_peaks(xund,yund,nund)
!corr   USE peak_mod
!corr   USE variables, only: dataset
!corr   real, dimension(:), allocatable, intent(out) :: xund,yund
!corr   integer, intent(out)                         :: nund
!corr   integer                                      :: i
!corr   real                                         :: intensity
!corr!
!corr   nund = 0
!corr   do i=1,numpeaks(pkind)
!corr      if (pkind(i)%info > 0) then
!corr          if (nund == 0) allocate(xund(pkind(i)%info),yund(pkind(i)%info))
!corr          nund = nund + 1
!corr          xund(nund) = pkind(i)%getx()
!corr          yund(nund) = peak_intensity(pkind(i)%getx(),dataset(1)%x0,dataset(1)%y)
!corr      endif
!corr   enddo
!corr!
!corr   end subroutine get_unindexed_peaks

END MODULE datamod

   subroutine init_data_global()
   USE General, only: nsyn
   USE Symmetry, only: ifRhombAx
   USE filereading, only: FTYPE_POW
   USE PatternRef, only: filetypepow, modify_range, kalpha
   implicit none
   common /step_data/istog
   integer :: istog
   common /ksearch/ ksearch
   integer :: ksearch
   common /cpseud_new/ ivet_new(3,4),percme
   integer :: ivet_new
   real :: percme   !,esqv
   common/errer/erreprof,erreprofw,erre_old,erref,errerif
   real :: erreprof,erreprofw,erre_old,erref,errerif
   common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),itry,pfwhm,kequirip
   integer :: ncycl,nroset,nundset,nsec,jback,itry,kequirip
   real :: uvwt,pfwhm
   common /peak_noghost/noghost
   integer :: noghost
   common/iesext/iesext
   integer :: iesext
   common/corr2th/thstepcor
   real :: thstepcor
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   common /direx/ncycle,icycle,nosing,izero ,zeroprof,asym,scal,frac   &
                 ,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real :: zeroprof,asym,scal,frac,sigma
   common/itypepeak/itypep
   integer :: itypep
   common /profp/dumpp,nprof,ipol
   real :: dumpp
   integer :: nprof,ipol
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common/spaceforplot/numsymb,spacesymb(400),figsymb(400),isymb(400),numerosymb(400)
   integer :: numsymb,isymb,numerosymb
   real :: figsymb
   character*20 spacesymb     
   common/exterfile/fname3ex
   character(len=80) :: fname3ex
!
! nprop=1 pearson
! nprop=2 gussian
! nprop=3 lorentzian
! nprop=4 pseudo-voigt
!
!
!  Global variables
   filetypepow = FTYPE_POW
   modify_range = .false.
   ifRhombAx = 0
   numsymb=0
   erreprof=0.0
   itypep = 1
   asym=0.0
   zeroprof=0.0
   thstepcor=0.0  
   percme=0.0
   scal=1.0
   ncycle=0
   icycle=0
   nosing=0
!
   iesdircell = 0
   iesspace = -1
   iesspacesav = 0
   itrial = 1
   irunextra = 0
   frac = 10.
   iesext = 0
   shiftzero = -1.E-9
!
   fname3ex=' '
   nprof=1
   nsyn=0
   kalpha= 0
   noghost=0
   istog=3
   itry=0
   ksearch = 0
   end subroutine init_data_global

!---------------------------------------------------------------------

   subroutine data1new(active_file,ier)
   USE General
   USE Symmetry
   USE filereading
   USE commandsmod
   USE PatternRef
   USE indexing
   USE elements
   USE strutil, only: string_locate, getnum, err_string
   USE spginfom
   USE peak_mod, only: pkcond, peak_set, pkind, check_pkcond
   USE comandiexpo
   USE unit_cell
   USE sinterface
   USE variables, only: cryst, dataset
   USE crystal_phase
   USE datasetmod
   USE background, only: back_condition_type, set_background_from_string
   USE mpi_prog
   USE datautil
   USE Molcom, only:jscreen
   USE Conteggi, only: theta_int, npunti
   USE ssmoothing
   USE messagemod
   USE VIEW
   USE datamod

   implicit none
   integer, intent(in)  :: active_file 
   integer :: ier  ! FIX this with init
   integer                  :: iesdirsear,ismoo,nsmoo,npsmoo
   logical                  :: modify_step,okrad,scaledata
   integer                  :: nwave
   real                     :: scaleval
   real, dimension(3)       :: wave,ratio
   type(back_condition_type):: backc
   common /step_data/istog
   integer :: istog
   common /ksearch/ ksearch
   integer :: ksearch
   common /cpseud_new/ ivet_new(3,4),percme
   integer :: ivet_new
   real :: percme   !,esqv
   character dire*80,diret*80     !!!!!,formt*80
   character(len=len(card)) :: line
   common/errer/erreprof,erreprofw,erre_old,erref,errerif
   real :: erreprof,erreprofw,erre_old,erref,errerif
   common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),itry,pfwhm,kequirip
   integer :: ncycl,nroset,nundset,nsec,jback,itry,kequirip
   real :: uvwt,pfwhm
   integer :: ivet(40),kshif(3),kistp(8)
   real :: vistp(8), vet(40)
   character(len=4), dimension(8) :: isotope
   common /peak_noghost/noghost
   integer :: noghost
   common/iesext/iesext
   integer :: iesext
   common/corr2th/thstepcor
   real :: thstepcor
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   common /direx/ncycle,icycle,nosing,izero ,zeroprof,asym,scal,frac   &
                 ,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real :: zeroprof,asym,scal,frac,sigma
   common/itypepeak/itypep
   integer :: itypep
   common /profp/dumpp,nprof,ipol
   real :: dumpp
   integer :: nprof,ipol
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /misurato/ misu
   integer :: misu
   common/spaceforplot/numsymb,spacesymb(400),figsymb(400),isymb(400),numerosymb(400)
   integer :: numsymb,isymb,numerosymb
   real :: figsymb
   common/spaceindexing /ispaceindexing,fspaceindex(660),ntotmult1,ntotmult2,      &
                         percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real :: fspaceindex,percind,errespac
   character*20 lauecell
   character*20 spacesymb     
   common/exterfile/fname3ex
   character(len=80) :: fname3ex
   logical :: is_indexing, make_standard
   integer :: i,j,iv,lend,kistog,ic,icmax
!corr   integer :: Lung, nExitCode,kpos,ksh,lix1,lix2,lix3,lix7
   integer :: Lung, nExitCode,kpos,ksh,lix1,lix2,lix7
   integer :: iesnorefine,nistp,ishif,lenp
   logical :: iesdircont
   integer :: ier_spg, ier_std
   logical :: symm_required,prncell
   integer :: nrt
   type(cell_type) :: cellt
   character(len=:), allocatable :: strcontent
!  ---- data1
   type(dataset_type)        :: datas
   integer                   :: ndat
   integer                   :: exitcode
   integer                   :: iflagref
!
!
   call write_inte_routine('data')
!
   nsym=0
   nrefl=0
   rhomax=0.0
   lpseuv(:)=0
   iflag=0
!
! nprop=1 pearson
! nprop=2 gussian
! nprop=3 lorentzian
! nprop=4 pseudo-voigt
!
!pgMAD280907
   kmad = 0
!
!  Output variables
   modify_step = .false.
   ismoo=0
   nsmoo=5
   npsmoo=3
!
!  Local variables
   iesnorefine = 0
!corr   celltemp(:) = 0.0
   iesdircont = .false.
   kistog=0
   nistp=0
   ishif=0
   nwave = 0
   wave = [CU_WAVE,0.,0.]
   ratio = [1.0,0.5,0.2]
   make_standard = .false.
   iesdirsear = 0
!
!  Global variables
   call init_data_global()
   !filetypepow = FTYPE_POW
   !modify_range = .false.
   !ifRhombAx = 0
   !numsymb=0
   !erreprof=0.0
   !itypep = 1
   !asym=0.0
   !zeroprof=0.0
   !thstepcor=0.0  
   !percme=0.0
   !scal=1.0
   !frac=0.0
   !ncycle=0
   !icycle=0
   !nosing=0
!
   !iesdircell = 0
   !iesspace = -1
   !iesspacesav = 0
   !itrial = 1
   !irunextra = 0
   !frac = 10.
   !iesext = 0
   !shiftzero = -1.E-9
!
   !fname3ex=' '
   !nprof=1
   !nsyn=0
   !kalpha= 0
   !noghost=0
   !istog=3
   !itry=0
   !ksearch = 0
   thmin=-0.01
   thmax=-0.01
   is_indexing = command_is_after(commands,NTREOR_CMD,icomat)>0 .or.         &
                 command_is_after(commands,DICVOL_CMD,icomat)>0 .or.         &
                 command_is_after(commands,MCMAILLE_CMD,icomat)>0
   if (is_indexing) then
       ksearch = 1
   endif
!--
   filehpow=trim(StructureName)//'.pow'
   neutro=RX_SOURCE
   okrad = .false.
   is_anomalous = .false.
   kequirip=0
   neuwei=0
   iprin=0
   scaledata = .false.
   scaleval = 1000.
   pfwhm=0.1
   kshif(:)=0
   bt=0.0
   sc=-99.9
   knw=0
   nstr=0
   nzro=166
   th=2.0
   rhomin=0.0
   en=1.0
   er=0.3
   npseud=0
   llpse=0
   lix1=0
   lix2=0
   !lix3=0
   lix7=0
!
   icmax=icomq(icomat,2)
   ipdir = get_ipdir(commands,icomat)  ! get pointer to first dir in card
!
   loop_datadir: do ic=1,icmax
      ipdir=ipdir+1
      line=card(ipdir)
      diret=line
      call cutst(line,lenp,dire,lend)
      call lcase(dire)
!-- directives with alphanumeric parameters
!-- global directives (extra,sirpow)
      select case (dire(1:4))
        case ('cont')
          !call cryst(1)%set_scattering(line,wave(1))
          !if (cryst(1)%numelem() == 0) go to 8000
          !call cryst(1)%get_scattering(celem)
          call read_chemformula(line,ier) ! only check validity
          if (ier /= 0) go to 8000
          strcontent = line
          iesdircont = .true.
          !lix3=2

        case ('isot')
          nistp=nistp+1
          call cutst(line,lenp,dire,lend)
          call lcase(dire)
          isotope(nistp) = trim(dire)
          read(dire(1:2),'(a2)') kistp(nistp)
          !iopt=0
          !call getnum(line,vet,ivet,iv,iopt)
          !if (iopt.eq.-1) go to 8000
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          vistp(nistp)=vet(1)
          !lix3=2

        case ('equi')
          kequirip=1

        case ('know')
          knw=1
          if (lenp.gt.0) read(line,'(a)') fname(FC_FILE)

        case ('neut')
          neutro=NEUTRON_SOURCE
          okrad = .true.

        case ('elec')
          neutro=ELECTRON_SOURCE
          okrad = .true.

        case ('anom')
          !Ignore 'anomalous' directive for neutrons and electrons
          if (.not.(is_directive(commands(icomat),'electrons') > 0 .or. &
              is_directive(commands(icomat),'neutron') > 0)) then
              is_anomalous = .true.
          endif

        case ('spac')
          if (lenp.le.1) go to 8000
          spacegr = init_spaceg_type(line)
          call cryst(1)%set_spg(spacegr)
          call lcase(line)
          call StandardSpg(line,ier_spg)
          if (ier_spg /= 0 .and. spacegr%undef()) then
              ier = ier_spg
              go to 8000
          endif
          lix2=1
          iesspace = 0

        case ('form')   ! obsolete directive

!--  sirpow directives
        case ('refl','ref2') ! lettura file h k l f^2 sigma
          if (active_file == REFL_FILE) then
              read(line,'(a)') filehpow
          endif

        case ('fobs')        ! file di rifllessi con f e non f^2
          if (active_file == REFL_FILE) then
          endif
!
!--  extra directives
!
        case('file')
          if (active_file == POW_FILE) then
              filetypepow = filetype_from_string(line)
              if (filetypepow == 0) filetypepow = FTYPE_POW
          endif

        case ('patt')
          if (active_file == POW_FILE) then
              if (lenp.eq.0) go to 8000
              read(line,'(a)') filehpow
              filetypepow = filetype_from_filename(filehpow) 
              if (filetypepow == 0) filetypepow = FTYPE_POW
          endif

        case ('broo')
          if (active_file == POW_FILE) then
              nsyn=1
          endif

        case ('alph') 
          if (active_file == POW_FILE) then
              kalpha=1
          endif

        case ('nogh')
          if (active_file == POW_FILE) then
              noghost=1
          endif

        case ('sync') 
          if (active_file == POW_FILE) then
              nsyn=1
          endif

        case ('peak') ! direttiva obsoleta
          if (active_file == POW_FILE) then
              ksearch = 1
          endif

        case ('exte') 
          if (active_file == POW_FILE) then
              if (lenp.eq.0) go to 8000
              read(line,'(a)') fname3ex
              iesext = 1
!             azzera peak search
              ksearch=0
          endif

        case ('pear')
          if (active_file == POW_FILE) then
              nprof=1
          endif

        case ('gaus')
          if (active_file == POW_FILE) then
              nprof=2
          endif

        case ('2-th')
          if (active_file == POW_FILE) then
              itypep=2
          endif

        case ('nore')
          if (active_file == POW_FILE) then
              iesnorefine = 1
          endif

        case ('find') 
          if (active_file == POW_FILE .and. .not.is_indexing) then ! next command is not indexing
          !if (active_file == POW_FILE) then ! next command is not indexing
              iesspace=1
              iesspacesav = 1
          endif
!
!-- directives with numeric parameters

        case ('sear')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              ksearch=1
              iesdirsear = 1
              if (iv == 0) go to 8000
              if (vet(1) > 0) pkcond%minp = vet(1)
              if (vet(2) > 0) pkcond%maxp = vet(2)
              if (ivet(3) > 0) pkcond%maxpk = ivet(3)
              if (vet(4) > 0) pkcond%amp = vet(4)
              if (ivet(3) < 0 .and. vet(4) > 0) pkcond%maxpk = -1 ! no limit for number of peaks
          endif

        case ('rang')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              select case (iv) 
                 case (2,3)  ! 2)thmin thmax 3) thmin thmax thstep
                   thmin=vet(1)
                   thmax=vet(2)
                   minrange = thmin
                   maxrange = thmax
                 case default
                   go to 8000
              end select
              modify_range = .true.
          endif

        case ('step')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv /= 1) go to 8000
              if (vet(1) > 0.0) then
                  thstep = vet(1)
                  modify_step = .true.
              else
                  ier = -1
                  go to 8000
              endif
          endif

        case ('stan')
          make_standard = .true.

        case ('hist')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              kistog=1
              if (iv.ne.1) go to 8000
              istog=ivet(1)
              if(istog.ne.3.and.istog.ne.5.and.istog.ne.7)then
                 ier=-1
                 write(Msg,8100) diret(1:Lung(diret))
        8100        format('Error in DATA directive: (histogram bar number can be 3 or 5 or 7)',a)
                 call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
                 return
              endif
          endif

        case ('smoo')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              ismoo=1
              select case (iv)
                 case (1)
                   nsmoo = ivet(1)
                 case (2)
                   nsmoo = ivet(1)
                   npsmoo = ivet(2)
              end select
              if (npsmoo > 2*nsmoo) then
                  ier=-1
                  Msg = "Error in DATA directive "//trim(diret)//': '//  &
              "poly. degree must be <- 2*inter. smoothing"
                  call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
                  return
              endif
          endif

        case ('zero')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv.ne.1) go to 8000
              shiftzero=vet(1)
          endif

        case ('thco')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv.ne.1) go to 8000
              thstepcor=vet(1)
          endif
!
!-- global directives (extra,sirpow)
        case ('fwhm')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          pfwhm=vet(1)
          if(pfwhm.lt.0.0.or.pfwhm.gt.1.0) go to 8000

        case ('rhom')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          th=vet(1)
          if (th.lt.0.0.or.th.gt.1.0) go to 8000

        case ('wave')     ! wave1 wave2 [ratio2]  or wave1 wave2 ratio2 wave3 [ratio3]  or ....
          call getnum(line,vet,ivet,iv)
          if (iv == 0) go to 8000
          nwave = iv / 2 + 1   ! number of values for wavelengths
          wave(1) = vet(1)
          wave(2:nwave) = vet(2:iv:2)
          nrt = (iv-1) / 2     ! number of ratios, last might be omitted
          ratio(2:nrt+1) = vet(3:iv:2)

        case ('cell')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.6) go to 8000
          do i=1,6
             cell(i)=vet(i)
          enddo
          iesdircell = 1
          lix1=1
          call cryst(1)%set_unit_cell(set_cell_type(cell))
          cryst(1)%is_symm_user = .true.

        case ('prin')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          iprin=ivet(1)

        case ('shif')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.3) go to 8000
          ishif=1
          do i=1,3
             ksh=ivet(i)
             if (ksh.ne.0) then
                 if (mod(24,ksh).ne.0) then
                     write(lo,230)
  230                format(///' origin shift components must be integral multiples of 1/24 ')
                     go to 8000
                 endif
             endif
             kshif(i)=ksh
          enddo

        case ('scal')  
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv > 0) then
              scaleval = vet(1)
          endif
          scaledata = .true.

        case ('back') 
          call set_background_from_string(backc,line,ier)
          if (ier /= 0) go to 8000

        case default
          write(Msg,6000) diret(1:Lung(diret))
 6000     format('Wrong directive in DATA modulus: ',a)
          ier=-1
          call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
          return

      end select
   enddo loop_datadir

   if (neutro /= RX_SOURCE .and. nsyn == 1) then
       write(Msg,8216)
 8216 format('Ambiguous radiation (neutron/syncrothron)')
       ier=-1
       call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
       return
   endif
   if (kalpha.eq.0.and.kistog.eq.1.and.active_file == POW_FILE)then
       ier=-1
       call MsgWinErr('ERROR',                                           &
       'Alpha2 stripping request is not in input while the number '//    &
       'of bars of the histogram for the alpha2 correction is given',ERR_WINDOW,nExitCode)        
       return
   endif
!
!  Symm. info are not required with  %crystal and dataset after first
   symm_required = is_symm_required(commands) .and. ndataset(dataset) == 0
!
!  Cell 
   if (lix1 <= 0) then
!
!     error message if search or filepeak or peak directives are not used
      prncell = .false.
      if (ksearch == 0 .and. iesext == 0 .and. symm_required) then
          ier=-1
          call MsgWinErr('ERROR','No cell parameters supplied',ERR_WINDOW,nExitCode)        
          return
      endif
   else
      prncell = .false.
      if (make_standard) then
          cellt = set_cell_type(cell)
          if (mpi_prog_rank == mpi_prog_master) then
              call standard_symmetry(spacegr,cellt,lo,ier_std)
          else
              call standard_symmetry(spacegr,cellt,-1,ier_std)
          endif
          if (ier_std == 0) then
              call cryst(1)%set_spg(spacegr)
              cell(:6) = cellt%get_par()
              call cryst(1)%set_unit_cell(cellt)
              line = trim(spacegr%symbol_xhm)
              call StandardSpg(line,ier_spg) ! this set nSpaceG variable
              prncell = .false.
          endif
      endif
      call CheckRhombAx(nSpaceG, cell, ifRhombAx)
      call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
!removed: problem with mpi version   call xfl01
   endif
!
!  Spacegroup
   if (lix2 > 0.and.iesspace > 0) then
       call MsgWinErr('WARNING',                                        &
       'Directives regarding space group are in clash'//char(10)//      &
       'Do you want to perform the space group determination?',         &
       QUEST_WINDOW_YES_NO,nExitCode)        
       if (nExitCode == 2) then
           iesspace = 0
           iesspacesav = 0
       endif
   endif
   if (lix2 == 0) then
       if (symm_required) then
!--- error message if search or filepeak or peak directives are not used
           if (ksearch.eq.0.and.iesext.eq.0.and.iesspace.eq.-1) then
               call MsgWinErr('WARNING',                                &
               'No space group symbol supplied.'//char(10)//            &
               'Space group determination will be performed.',          &
               WARN_WINDOW,nExitCode)        
                iesspace=1
                iesspacesav = 1
           endif
      endif
   else
      if (ier_spg == 0) then
          call insym(ishif,kshif,ier)
          if (ier < 0) then
              return
          endif
      else
          call set_spaceg_new(spacegr)
          call cryst(1)%set_spg(spacegr)
      endif
      call xspinb
      call yfrm02
   endif
! 
!   if (lix3 <= 0) then
   if (.not.iesdircont) then
!--- error message if search or filepeak or peak directives are not used
       if (ksearch.eq.0.and.iesext.eq.0) then
!
!          Controlla se il contenuto e' necessario
           if (.not.next_command_is(commands,icomat,CRYSTAL_CMD)) then  ! jump check with %crystal
               if (is_content_required(commands)) then
                   ier=-1
                   call MsgWinErr('ERROR','No cell content supplied',ERR_WINDOW,nExitCode) 
                   return
               endif
           endif
       endif
!corr   elseif (lix3 == 2) then
!corr       call ordina_elements(celem,neutro)
!corr       call cryst(1)%set_scattering(celem)
   endif
   call xdump
   if (neutro /= NEUTRON_SOURCE .and. nistp > 0) then
       ier=-1
       call MsgWinErr('ERROR','The ISOTOPE directive must be used with the NEUTRON directive',ERR_WINDOW,nExitCode)
       return
   endif
!
   if (iesnorefine.eq.1.and.active_file == POW_FILE) pkcond%refine = .false.
!
!---- ex data 1 -------
   if (active_file == REFL_FILE) then
       if (nwave > 0) then
           alambda = wave(1)
           wavel = wave(1)
       endif
       call manage_scatters()
       !return
   else
!
!      Leggi file di dati in countp
       call datas%set_wave(nwave,wave,ratio)
       call load_datafile(datas,filehpow,jscreen>0,ier,filetypepow)
       if (ier /= 0) go to 90
       call push_back_dataset(dataset,datas)
       ndat = ndataset(dataset)
       if (modify_range) call dataset(ndat)%resize(thmin,thmax)
       if (okrad) dataset(ndat)%radtype = neutro
       if (scaledata) call dataset(ndat)%scale_data(scaleval)
       if (dataset(ndat)%nwave > 0) then
           alambda = dataset(ndat)%wave(1)
           wavel = dataset(ndat)%wave(1)
           call init_indexation(dataset(ndat)%wave(1))
           if (dataset(ndat)%nwave > 1) then
               call replace_command(commands,EXTRA_CMD,LEBAIL_CMD)
           endif         
       else
           call MsgWinErr('ERROR','No wavelength supplied',ERR_WINDOW,exitcode)
           ier = -2
           go to 90
       endif
       call dataset(ndat)%set_cond(backc)
!
!      scarica in theta_int
       call dataset_to_thetaint(dataset(ndat))
!
!      Check peak search conditions provided by user by directive search
       if (iesdirsear == 1) call check_pkcond(pkcond,thmin,thmax)
!
       if(ismoo.gt.0)theta_int(:,2)=savgol_smooth(theta_int(:,2),nsmoo,nsmoo,0,npsmoo)
!
       if (mpi_prog_rank == mpi_prog_master) then
           call dataset(ndat)%prn(lo,iesdircell==0)
       endif
       call write_message_pattern(filehpow)
!
       call manage_scatters() ! set scatters after wave and before manage_filefc (=> gener)
!
       if (iesdircell == 1) then  ! cell defined
           if(.not.cryst(1)%is_spg()) then
!
!             cell defined and procedure find space required
              call gener_findspace(ier) ! generate in laue class
              if (ier.lt.0) go to 90
           endif
           call gener(dataset(ndat)%wave,iflagref,mpi_prog_rank==mpi_prog_master)
!qt           call write_message('Reflections nr.: ',inum=nrefl,pos=6)
       endif
       call ysfr37(npunti)
       if(jscreen > 0) then
          call vedinew(8,1,npunti)
!qt          call write_message('Press " Pattern " to work on the pattern or '   &
!qt                             //'"Next" to go on',pos=7)
       endif
       if (iesext > 0) then
!
!          Load peak from external file specified by directive 'extefile'
           if (itypep == 1) then
               call LoadPeaks(fname3ex,2,ier)   ! d values (default)
           else
               call LoadPeaks(fname3ex,1,ier)   ! 2theta values
           endif
           if (ier == 0) call peak_set(pkind,code=0) ! this disable the refinement of peaks
       endif

       !call combine_dataset(dataset,"tmp.xy",err)
   endif
!
!  Print symmetry and content
   if (mpi_prog_rank == mpi_prog_master) then
       if (prncell) call print_cell(cell,kpr=lo)
       if (lix2 > 0) call WriteSymmInfo(lo)
       if (numelem(celem) > 0) call print_elements(celem,lo,neutro,wave(1))
   endif

   return

 8000 continue
   ier=-1
   call MsgWinErr('ERROR','Error in DATA directive : '//trim(diret),ERR_WINDOW,nExitCode)
   return
 
 90 continue
   return

   contains

   subroutine manage_scatters()
   if (iesdircont) then

       call cryst(1)%set_scattering(strcontent,wave(1),neutro,ier)
       call cryst(1)%get_scattering(celem)

       if (neutro == NEUTRON_SOURCE) then
          if (nistp > 0) then
              do j=1,nistp
                 kpos = string_locate(isotope(j),celem%lab)
                 if (kpos > 0) then
                     !celem(kpos)%fact = vistp(j)
                     call set_neutron_factor(celem(kpos),vistp(j))
                     cycle
                 endif
                 write(Msg,9007) isotope(j)
 9007            format('Atom type ',a,' not expected. Check isotope directive')
                 ier=-1
                 call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
                 return
             enddo
             call ordina_elements(celem,neutro)
             call cryst(1)%set_scattering(celem)
          endif
       endif
       s3s2 = s3s2_value(celem,neutro)
       if (abs(s3s2) < 0.0001) then
           ier=-1
           write(Msg,'("Sigma(3) / sigma(2)**1.5 = ",f10.3)') s3s2
           call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
           return
       endif
   endif

   end subroutine manage_scatters

   end subroutine data1new

!------------------------------------------------------------------

   subroutine datanew(active_file,ljump,lstop,exit_code,ier)
   USE Molcom, only:jscreen 
   USE General, only: lo,POW_FILE
   USE PatternRef
   USE messagemod
   USE enable_amb
   USE proginterface
   USE peak_mod
   USE indexing
   USE errormod
   USE spginfom
   USE Conteggi, only: npunti
   USE view
   implicit none
   interface
     subroutine run_peaksearch(gui)
     logical, intent(in), optional :: gui
     end subroutine run_peaksearch
   end interface
   integer, intent(in) :: active_file
   logical, intent(in) :: ljump
   logical, intent(in) :: lstop
   integer, intent(out) :: exit_code, ier
   common /ksearch/ ksearch
   integer :: ksearch
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero 
   common /centering/CellType,CellAxis,nruntre,iesdircell
   character CellType,CellAxis*3
   integer :: nruntre,iesdircell 
   common/ylimi/nfl,lfl,idummy(4),nfls
   integer :: nfl,lfl,idummy,nfls
   type(error_type) :: err
   common/spaceindexing /ispaceindexing,fspaceindex(660), &
   ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   character(len=20) :: lauecell
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real :: fspaceindex,percind,errespac
   common/iesext/iesext
   integer :: iesext
   logical :: stop_enabled
!
   ier = 0
   exit_code = 0
   if (jscreen > 0) then
       !call toolbar_set_state('continue'//char(0),1)
       !call stop_fortran_visible(1)
       call clear_messages()
       call write_message('Data command',pos=1)
   endif
!
   nfl=1
   if (.not.ljump) then
       call data1new(active_file,ier)
       if (ier /= 0) go to 10 
   endif
!
   stop_enabled = (ispaceindexing == 0) .and. lstop
   if (active_file == POW_FILE) then
       if (stop_enabled .and. jscreen > 0) then
           call stop_fortran_visible(1)
!qt           call abilita_tasti('pattern')
           call abilita_tasti(PatternAction)
           call abilita_tasti(RunAction,1)
           call write_message('Press " Pattern " to work on the pattern or '   &
                      //'"Next" to go on',pos=1)
           call wmenu_set_state('Range'//char(0),1)
           call stop_fortran()           
           call wmenu_set_state('Range'//char(0),0)
!qt           if (kexitrun == 1) go to 10     ! ho premuto il tasto 'exit' 
           if (event_from_gui() == STOP_EVENT) then
               kexitrun = 1
               go to 10
           endif
       endif
!
       if (kalpha.gt.0) then
          call dataalpha
          if (stop_enabled .and. jscreen > 0) then
!qt              call abilita_tasti('pattern')
              call abilita_tasti(PatternAction)
              call abilita_tasti(RunAction,1)
              call stop_fortran()
              if (event_from_gui() == STOP_EVENT) then
                  kexitrun = 1
                  go to 10
              endif
          endif
          if (kexitrun == 1) go to 10  ! ho premuto il tasto 'exit'
       endif
!
!      Peak search for indexing
!qt       ier = 0
       if (iesext == 0) then
           if (ksearch > 0) then
               if (.not.allocated(pkind)) then
                   call run_peaksearch()    ! new peak search for indexing
               else
                   call write_message('Number of peaks:',inum=numpeaks(pkind),pos=2)
                   if (jscreen > 0) then
                       call vedinew(5,1,npunti)
                       call clear_messages(ipos=3)
                   endif
               endif
           endif
       endif

!qt       if (ier /= 0) go to 10 
   endif

   if (ksearch > 0) then !!! .or. iesext > 0) then  ! Change peaks manually
!qt       call abilita_tasti('pattern')
       call abilita_tasti(PatternAction)
       call abilita_tasti(RunAction,1)
       !call abilita_tasti(PeaksAction,1)
!qt       call abilita_tasti('peaks',state=1)
       call stop_fortran()
!qt       call abilita_tasti('peaks',state=0)
       !call abilita_tasti(PeaksAction,0)
!qt       if (kexitrun == 1) go to 10      ! ho premuto il tasto 'exit'
       if (event_from_gui() == STOP_EVENT) then
           kexitrun = 1
           go to 10
       endif
       call peak3(err)
       if (err%signal) return
       call peak_print(pkind,lo)        ! print peaks on output file
   endif
!
   if (iesdircell.eq.1.and.iesspace.eq.1)then
      call definespace(active_file,exit_code,ier)
      if (exit_code == STOP_EVENT) go to 10
      if (ier /= 0) go to 10 
      if (kexitrun > 0) go to 10       ! ho premuto il tasto 'Cancel' o 'Exit'
      call export_file_exp()           ! genera file nome1.exp
   endif
!  
   call data3(active_file,ier)
!
10 if (jscreen > 0) then
       call stop_fortran_visible(0)
       !call toolbar_set_state('continue'//char(0),2)
       !call abilita_tasti('pattern')
       call abilita_tasti(PatternAction)
   endif
!
   return
   end

!---------------------------------------------------------------------

   subroutine open_diffraction_patt(fileIn,lengthIn,fileOut,lengthOut,add_data,ier) bind(C,name="open_diffraction_patt")
   USE iso_c_binding, only: c_char, c_int
   USE fileutil
   USE General, only: ext, INP_FILE, OUT_FILE
   USE datasetmod
   USE datautil
!qt   USE molcom, only:ifProject
   USE strutil
!qt   USE variables, only: dataset, cryst
!qt   USE plotstyle, only:styledata
   USE datamod
   implicit none
   character(kind=c_char), intent(in) :: fileIn(*)
   integer(c_int), intent(in), value  :: lengthIn
   character(kind=c_char), intent(in) :: fileOut(*)
   integer(c_int), intent(in), value  :: lengthOut
   integer(c_int), intent(in), value  :: add_data
   integer(c_int), intent(out)        :: ier
!qt   integer                            :: iflag
!qt   integer                            :: nlenn
!qt   integer                            :: ierc
!qt   type(dataset_type)                 :: datas
   character(len=:), allocatable      :: filnam,filout
!
   ier = 0
   filnam = toFortranString(fileIn,lengthIn)
   filout = toFortranString(fileOut,lengthOut)
   call open_pattern(filnam,filout,add_data,ier)
!corr   call load_datafile(datas,filnam,.true.,ier) 
!corr   if (ier == 0) then
!corr       if (add_data == 0) then
!corr           if (ifProject.ne.0) then   ! importante se esegui run successivi
!corr               iflag = 2
!corr!              comment the next line to add pattern
!corr               call chiudi_bin(iflag) ! call InitExpo
!corr           endif
!corr           StructureName = file_rem_ext(file_get_name(filnam))
!corr           call cryst(1)%set_name(StructureName)
!corr           do i=1,size(fname)
!corr              if (i /= INP_FILE .and. i /= OUT_FILE) then
!corr                  fname(i) = s_blank_delete(trim(StructureName)//ext(i))
!corr              endif
!corr           enddo
!corr           fname(INP_FILE) = filnam
!corr           fname(OUT_FILE) = filout
!corr!
!corr           close(lo,iostat=ierc)
!corr           itype = 0
!corr           iform = 1
!corr           nlenn = 80
!corr           jop = jopen(lo,fname(OUT_FILE),itype,iform,ier2)
!corr!
!corr           call clear_dataset(dataset)
!corr       endif
!corr       call push_back_dataset(dataset,datas)
!corr       call work_on_pattern(dataset(1),filnam,add_data)
!corr   endif
!
   end subroutine open_diffraction_patt

!---------------------------------------------------------------------------
!qt
!qt   subroutine open_pattern(filnam,wavel,ier)
!qt   USE datasetmod
!qt   USE datautil
!qt   USE General, only: StructureName
!qt   USE fileutil, only: file_get_name, file_rem_ext
!qt   USE variables, only: dataset,cryst
!qt   USE molcom, only: jscreen
!qt   USE plotstyle, only:styledata
!qt   USE enable_amb
!qt   implicit none
!qt   character(len=*), intent(in) :: filnam
!qt   real, intent(in)             :: wavel
!qt   integer, intent(out)         :: ier
!qt   type(dataset_type)           :: datas
!qt!
!qt   if (wavel > 0.0) call datas%set_wave(1,[wavel])
!qt   call load_datafile(datas,filnam,jscreen>0,ier) 
!qt   if (ier == 0) then
!qt       StructureName = file_rem_ext(file_get_name(filnam))
!qt       call cryst(1)%set_name(StructureName)
!qt       datas%style = styledata%get()
!qt       call clear_dataset(dataset)
!qt       call push_back_dataset(dataset,datas)
!qt       call work_on_pattern(dataset(1),filnam,0)
!qt       call abilita_tasti("pattern")
!qt   endif
!qt!
!qt   end subroutine open_pattern
!qt
!---------------------------------------------------------------------------

   subroutine work_on_pattern(datas,add_data)
   USE Conteggi, only: npunti
   USE view, only: vedinew
   USE messagemod
   USE enable_amb
   USE datasetmod, only: dataset_type
   USE molcom, only: jscreen
   USE datamod, only: abilita_tasti2
   implicit none
   type(dataset_type), intent(in) :: datas
!   character(len=*), intent(in)   :: filnam
   integer, intent(in)            :: add_data
!
   if(add_data == 0) call dataset_to_expo(datas)

   if (jscreen > 0) then
       call init_messages()
       call write_message_pattern(datas%fname)
       call vedinew(8,1,npunti)
       !call abilita_tasti('pattern')
       call abilita_tasti2()
       call wmenu_set_state('Range'//char(0),1)
!corr       call SetWindowTitle(filnam)
       call toolbar_set_state('continue'//char(0),2)
   endif

   if (add_data == 0) call init_dati()
!
!  create background FIXME-non serve se fai una open_background
!   call fillcounts()
!   call back_for_peaksearch()
!!!!!test peak search
   !     call test_peaksearch()
!!!!!test peak search

!!!!!!!!!!!!!!!prova index
      !call run_indexing(filehpow, alambda, 'mcmaille')
      !call run_indexation(filehpow, alambda, 'dicvol')
      !call run_indexation(filehpow, alambda, 'ntreor')
!!!!!!!!!!!!!!!prova index

   end subroutine work_on_pattern
!---------------------------------------------------------------------------

!   subroutine test_peaksearch()
!   USE peak_mod
!   USE messagemod
!   USE Conteggi
!   USE view
!   USE proginterface, only:fillcounts
!!corr   USE indexing
!   integer :: ier
!   common /ksearch/ ksearch
!   integer :: ksearch
!   call fillcounts()
!   call back_for_peaksearch()
!   !call peaksearch(shiftzero,ier)
!   ksearch = 1
!   call peak1(ier)
!   call vedinew(5,1,npunti)
!   call write_message('Number of peaks: ',inum=npk,pos=2)
!   end subroutine test_peaksearch

!---------------------------------------------------------------------------
   
   subroutine dataset_to_expo(datas)
   USE datasetmod, only: dataset_type
   USE General, only:alambda, neutro, nsyn, wavel
   USE patternref, only:filehpow
   USE arrayutil
   USE indexing
   implicit none
   type(dataset_type), intent(in) :: datas
!
   if (size_array(datas%x) > 0) then
       call dataset_to_thetaint(datas)
       alambda = datas%wave(1)
       wavel = datas%wave(1)
       call init_indexation(alambda)
       filehpow = trim(datas%fname)
       neutro = datas%radtype
       if (datas%sync) then
           nsyn = 1
       else
           nsyn = 0
       endif
   endif
   end subroutine dataset_to_expo

!---------------------------------------------------------------------------

   subroutine dataset_to_thetaint(dataset)
   USE Conteggi
   USE PatternRef
   USE arrayutil
   USE datasetmod, only: dataset_type
   implicit none
   type(dataset_type), intent(in) :: dataset
!
   call AllocaConteggi(2)
   npunti = size_array(dataset%x)
   call AllocaConteggi(1)
   theta_int(:,1) = dataset%x
   theta_int(:,2) = dataset%y
   theta_int(:,7) = theta_int(:,1)
   theta_int(:,3:6) = 0.0
   thstep = dataset%tstep
   thmin = dataset%tmin
   thmax = dataset%tmax
!
   end subroutine dataset_to_thetaint


!---------------------------------------------------------------------------

   subroutine GenInfoV2(funit)
   USE General, only: slopeWils,bt,neutro,is_anomalous
   USE PhaseModule, only: numset,nset,erre
   USE atom_type_util
   USE stat
   USE variables, only: cryst,dataset
   USE crystal_phase
   USE datasetmod
   use prog_constants, only: DEF_WAVE
!
   implicit none
   integer, intent(in)                           :: funit
   real                                          :: fom,cfom,ep,dens,wave
   integer                                       :: lsn,lfw,llw,lpb,ll16,ipos,iphb
   integer                                       :: i,j,nph,z_val,radtype
   character(len=:), allocatable                 :: strf
   type(element_type), dimension(:), allocatable :: chform
   dimension ipos(302),cfom(302),fom(11),ep(2000),iphb(2000)
!
   if (ndataset(dataset) > 0) then
       call dataset(1)%prn(funit,.true.)
       radtype = dataset(1)%radtype
       wave = dataset(1)%wave(1)
   else
       wave = DEF_WAVE
       write(funit,'(" Wavelength =",f10.6/)') wave
       radtype = RX_SOURCE
   endif
!
   do nph=1,numphase(cryst)
      if (numphase(cryst) > 1) write(funit,'(/,100("=")/)')
      call cryst(nph)%print(funit,'NCSE',radtype=radtype,wave=wave)
!
      if (numphase(cryst) == 1) then
          if(abs(slopeWils).gt.0.0) then
              write(funit,'(" Wilson Thermal Factor B(iso) = ",f7.3)') bt
              write(funit,*)
          endif
!
          if (numset > 0) then
              call srlin(16,lsn,lfw,llw,lpb,ll16)
              if (lfw.gt.0) then
                  write(funit,'("                        Figures of Merit")')
                  write(funit,'("   Set  (trial)",1x,"  mabs  alcomb R(x,y)"," cphase  cfom",3x,"nund")')
                  call ysfl16(cfom,ipos,nset)
                  do j=1,nset
                     i = ipos(j)
                     call snr16(i,fom,ep,iphb)
                     write(funit,'(i6,"  (",i5,")",5f7.3,i6)') j,nint(fom(10)),(fom(i),i=1,5),nint(fom(6))
                  enddo
              endif
              write(funit,*)
          endif
      endif
!
      if (cryst(nph)%natoms() > 0) then  
          write(funit,'(" Nr. of atoms in the asymmetric unit: ",i0)')cryst(nph)%natoms()
          if (cryst(nph)%numref() > 0) then
              call cryst(nph)%sfactor(neutro,is_anomalous)
              if (cryst(nph)%is_known()) then
                  erre = Rvalue(cryst(nph)%ref%fv,cryst(nph)%ref%fc)
              else
                  erre = Rvalue(cryst(nph)%ref%fo,cryst(nph)%ref%fc)
              endif
              write(funit,'(" R-f Fit: ",f0.3)') erre
          endif
          call print_atoms(cryst(nph)%at,height=.true.,kpr=funit)
          z_val = Z_value(cryst(nph)%at,cryst(nph)%spg%nsymop,chform)
          call chemical_formula(chform,strf,fform=2,ord=.true.,hide1=.true.,hidecharge=.true.)
          write(funit,'(" Formula:         ",a          )')strf
          write(funit,'(" Z:               ",i0         )')z_val
          write(funit,'(" Volume per atom: ",f0.3," °A3")')  &
                volume_per_atom(cryst(nph)%at,cryst(nph)%spg%nsymop,cryst(nph)%cell%volume())
          dens = cryst(nph)%density()
          write(funit,'(" Density:         ",f0.3," g/cm3")')dens
          write(funit,'(" Mu:              ",f0.3," cm-1")')linear_abs_coeff(cryst(nph)%at,dens,wave)
      endif
   enddo
!
   end subroutine GenInfoV2

! -----------------------------------------------------------------

   subroutine run_indexing(prog_index)
   USE indexing
   USE timer
   USE general, only: lo
   USE progtype, only: DISTRIBUTED_VER
   USE messagemod
   implicit none
!qt   character(len=*), intent(in)                    :: sprog
   integer, intent(in)                             :: prog_index
   type(cell_info_type), dimension(:), allocatable :: pcell
   integer                                         :: ier
   logical                                         :: test = .true.
   type(timer_type)                                :: timeind
!   
   call timeind%set_as_elapsed()
   call timeind%tic()
   if (prog_index /= NTREOR09_PROG) then
       call clear_messages()
       call SetRowCountMesg(2,1)
       call write_message('Indexing by '//PINDEX_NAME(prog_index),pos=1)
       call write_message('Please wait ...',pos=2)
   endif
   call timeind%set_name(PINDEX_NAME(prog_index))
   select case (prog_index)
      case (DICVOL_PROG)
        call run_dicvol(indx_set(DICVOL_PROG),pcell)
        call add_to_cell_list(cell_list,pcell)

      case (MCMAILLE_PROG)
        call run_mcmaille(indx_set(MCMAILLE_PROG),pcell)
        call add_to_cell_list(cell_list,pcell)

      case (NTREOR09_PROG)
        call run_ntreor(.false.,ier)

   end select
   call timeind%toc()
   call timeind%tprint(lo)
   call timeind%tprint(0)
!
   if (test .and. .not.DISTRIBUTED_VER) call test_indexing(cell_list)
!    
   end subroutine run_indexing

! -----------------------------------------------------------------

   subroutine end_dicvol() bind(C,name="end_dicvol")
   USE peak_mod
   USE indexing
   use fileutil
   implicit none
   character(len=*), parameter :: cell_file = "dicvol_out.ord"
   type(cell_info_type), dimension(:), allocatable :: dcell
   integer                                                      :: ncell
   integer :: ier
   
   ier = 0  !!!!TOFIX
   if (ier == 0) then
      if (file_exist(cell_file)) then
!
!         Read output file with cell dimensions
          call read_dicvol(cell_file,numpeaks(pkind),dcell,indx_set(DICVOL_PROG)%zero_ref,ncell)
          call cell_sort(dcell,'m20')

          call add_to_cell_list(cell_list,dcell)
      endif
   endif

   end subroutine end_dicvol

! -----------------------------------------------------------------

   subroutine save_cell_list(filename, length) bind(C,name="save_cell_list")
!
!  Save cell list in a exp or txt file
!
   USE indexing
   USE fileutil
   USE strutil
   USE programinput
   use iso_c_binding
   implicit none
   character(c_char), intent(in)     :: filename(*)
   integer(c_int), intent(in), value :: length
   character (len=:), allocatable    :: filnam
   type(file_handle)                 :: cf
!
   !corrfilnam = remc0(filnam)
   filnam = toFortranString(filename,length)
   if (s_eqi(get_extension(filnam),'exp')) then
       call write_input_file(input_file_create(filnam),'space',cell_list) 
   else
       call cf%fopen(filnam,'w')
       call print_cell_list(cell_list,cf%handle())
       call cf%fclose()
   endif
!
   end subroutine save_cell_list

! ---------------------------------------------------------------------

   subroutine run_peaksearch(gui)
   USE Conteggi, only: npunti, theta_int
   !USE counts, only:deltatval,deltadval,delta_from_lambda
   USE peak_mod
   USE proginterface, only:fillcounts
   USE General, only:alambda
   USE variables, only: dataset
   USE arrayutil
   !use filereading
   implicit none
   logical, intent(in), optional   :: gui
   real, dimension(:), allocatable :: yc
   real, parameter                 :: SRANGE = 0.15 ! 2theta-range for peak_search at 1.54056
   logical                         :: guivar
!
   if (present(gui)) then
       guivar = gui
   else
       guivar = .true.
   endif
!
!  Subtract background
   if (size_array(dataset(1)%yb) /= npunti) then
       !call dataset_to_thetaint(dataset(1))
       call back_for_peaksearch(.true.)
   endif
   allocate(yc(npunti))
   !where (backgr(:) > theta_int(:,2)) 
   !       yc(:) = 0
   !elsewhere
           yc(:) = theta_int(:,2) - dataset(1)%yb(:)
   !endwhere
!
!  compute srange
   pkcond%srange = init_peak_range(theta_int(:,1),yc,alambda)
   !write(0,*)'SRANGE =',pkcond%srange
!
   call peak_create(theta_int(:,1),yc,pkind,pkcond,alambda,pkindtot)
   pkcond%maxpk = -1 ! unset max number of peaks for GUI
   pkcond%minpk = -1 ! unset min number of peaks for GUI
!
   if (guivar) call update_peak_graph()
   !call write_column('picchi.txt',xcol1=pkind%getx(),metad='#2theta values')
!
   end subroutine run_peaksearch

! ---------------------------------------------------------------------

   subroutine update_peak_graph()
   USE molcom, only: jscreen
   USE enable_amb
   USE messagemod
   USE peak_mod
   USE view
   interface
      subroutine update_peak_list() bind(C,name="update_peak_list")
      end subroutine update_peak_list
   end interface
!
   if (jscreen > 0) then
       !call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
       call write_message_peak()
       call update_peak_list()
       call abilita_tasti('peaks',state=1)
       call vedinew(5, rescale=0)
   endif
!
   end subroutine update_peak_graph

! ---------------------------------------------------------------------

   subroutine update_peak_graph1(pos_message)
   USE molcom, only: jscreen
   USE enable_amb
   USE messagemod
   USE peak_mod
   USE view
!
   integer, intent(in) :: pos_message
!
!!!TOFIX: move all functions in GUI, except for vedinew
   if (jscreen > 0) then
       call write_message('Number of peaks:',inum=numpeaks(pkind),pos=pos_message)
!qt-moved in GUI       call update_peak_list()
       call abilita_tasti('peaks',state=1)
       call vedinew(5,rescale=0)   !!!!,1,npunti)
   endif
!
   end subroutine update_peak_graph1

! ---------------------------------------------------------------------

   subroutine cell_dialog(ier,exit_code)
   USE PatternRef
   USE General, only: POW_FILE,ifAutomatic,lo,cell
   USE enable_amb
   USE commandsmod
   USE comandiexpo, only:icomat
   USE strutil, only:cutst,remc0,addc0
   USE sinterface
   USE molcom, only: jscreen
   USE indexing
   USE spginfom
   USE unit_cell
   use indexing_interop
   use iso_c_binding
   use datamod, only: viewrefcell
   use view
!
   implicit none
   interface
      subroutine open_cell_dialog(ncell, c_cell_list, icell) bind(C,name="open_cell_dialog")
      use indexing_interop
      use iso_c_binding
      integer(c_int), intent(in), value    :: ncell
      type(c_cell_info_type), dimension(*) :: c_cell_list
      integer(c_int)                       :: icell
      end subroutine open_cell_dialog 
   end interface
   integer, intent(out) :: ier, exit_code
   common/spaceindexing /ispaceindexing,fspaceindex(660),    &
                      ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real    :: percind,errespac,fspaceindex
   character(len=20) :: lauecell
   common/iesext/iesext
   integer :: iesext
   common /ksearch/ ksearch
   integer :: ksearch
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   common /prova_kalpha/ kalphasav
   integer :: kalphasav
   integer :: kfindspace
   character(len=:), allocatable  :: line
   integer :: ncell, icell
   type(spaceg_type) :: spgtemp
   type(c_cell_info_type), dimension(:), allocatable :: c_cell_list
   integer :: i
!
   ier = 0
   exit_code = 0
   call ClearMessage()
!
   call print_cell_list(cell_list,lo,'List of plausible cell parameters:')
   icell = 1     ! select the first cell
   if (ifAutomatic == 1 .or. jscreen == 0) then
       call viewrefcell(int(icell,c_int))  
   else
       call toolbar_set_state('continue'//char(0),0)
       call abilita_tasti('indexwin',0)
       call abilita_tasti('peaks',0)
       ncell = number_of_cells(cell_list)
       kexitrun = 0
       style(STYLE_UND_PEAKS)%vis = 1
       call init_viewrefcell()
       allocate(c_cell_list(ncell))
       do i=1,ncell
          c_cell_list(i) = cast_cell_to_c(cell_list(i))
       enddo
       call open_cell_dialog(ncell, c_cell_list, icell)
       call end_viewrefcell()
       style(STYLE_UND_PEAKS)%vis = 0
       call vedinew(8,lsetstyle=.false.,rescale=0) ! redraw to remove unindexed peaks
       call abilita_tasti('indexwin',1)
       call abilita_tasti('peaks',1)
!corr-not necessary       if (kexitrun /= 0) return  ! exit from File > Exit !!!!FIXME - non funziona se fai Pattern > Indexing
   endif
!
!  Exit for 'cancel' (icell=0); %end with -nogui or -auto; index is last command with -nogui or -auto
   if (icell == 0 .or. ( (jscreen == 0 .or. ifAutomatic == 1) .and.   &
      (next_command_is(commands,icomat,'end') .or. (icomat == ncommands(commands))))) then
       !call abilita_tasti('pattern')
       !call abilita_tasti('peaks',1)   !!!add abilita index
       if (jscreen > 0) call toolbar_set_state('continue'//char(0),2)
!!! reset this variables to avoid problems in data1 for new run with ntreor
!!!FIXME - check is necessary also for ntreor after structure solution
       irunextra = 0
       iesspacesav = 1
       itrial = 1
       exit_code = STOP_EVENT
       kexitrun = 2  ! force exit from loop in gescom
       return
   endif
   if (jscreen > 0) call toolbar_set_state('continue'//char(0),1)
   write(lo,'(//a)')'       The following cell has been selected: '
   call print_cell(cell_list(icell)%par,cell_list(icell)%sd,cell_list(icell)%sys,kpr=lo)
!
   ispaceindexing = -1
   ksearch = 0
   iesdircell = 1
   iesext = 0
   iesspace = 1
   iesspacesav = 1
!
!  Recupera il gruppo spaziale se è presente la direttiva space
   line = get_string_directive(commands,'space')
   kfindspace = 1   ! forza check sul gruppo spaziale
   if (len_trim(line) > 0) then
!
!      Check compatibility between spg in input file and new cell
       spgtemp = init_spaceg_type(line)
       if (csys_from_cell(cell) == spgtemp%csys_code) then
           call set_spaceg(line)  ! spacegr is now defined
           call yfrm02
           kfindspace = 0 
       endif
   endif
!
   call cell_spg_dialog(kfindspace,exit_code)
   if (exit_code == STOP_EVENT) return
!
   if (kfindspace == 1) then
!      findspace+export del file nome1.exp
!      nota che findspace azzera isspace
!      se ho dato peaksearch + findspace non rifaccio correzione kalpha
       kalphasav = 0
       if (kalpha.gt.0) then
           kalpha = 0
           kalphasav = 1
       endif
       call datanew(POW_FILE,.true.,.true.,exit_code,ier)
   endif
!
   return
   end subroutine cell_dialog

!-----------------------------------------------------------------------

   subroutine cell_spg_dialog(kfindspace,exit_code)
   USE General, only: cell,celem,ifAutomatic,neutro,POW_FILE,spacegr
   USE elements
   USE sinterface, only: spaceg_string
   USE commandsmod
   USE molcom, only: jscreen
   USE strutil
   USE variables, only: cryst,dataset
   USE crystal_phase, only: get_strelem
   USE errormod
   USE unit_cell
   USE datamod
   USE enable_amb, only: STOP_EVENT
   use iso_c_binding
   implicit none
   interface 
      subroutine cell_parameters_dlg(cell,id,space,lenspg,content,lencont,kfindspace,kreturn) bind(C,name="cell_parameters_dlg")
      use iso_c_binding
      real(c_double), dimension(6)    :: cell
      integer(c_int)                  :: id
      character(c_char), dimension(*) :: space
      integer(c_int), intent(out)     :: lenspg
      character(c_char), dimension(*) :: content
      integer(c_int), intent(out)     :: lencont
      integer(c_int)                  :: kfindspace
      integer(c_int)                  :: kreturn
      end subroutine cell_parameters_dlg
   end interface
   integer, intent(inout)        :: kfindspace
   integer, intent(out)          :: exit_code
   character(len=18)             :: spacechar
   character(len=:), allocatable :: cell_cont
   integer                       :: iflagref,kreturn,ier
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /centering/CellType,CellAxis,nruntre,iesdircell
   character CellType,CellAxis*3
   common/spaceindexing /ispaceindexing,fspaceindex(660),    &
          ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real    :: percind,errespac,fspaceindex
   character(len=20) :: lauecell
   integer :: nruntre,iesdircell
   logical :: kgener,updated
   type(error_type) :: err
   real(c_double), dimension(6)      :: c_cell
   character(c_char), dimension(18)  :: c_space
   character(c_char), dimension(200) :: c_content
   integer(c_int)                    :: lencont,lenspg
   logical :: dlg_active
!
   exit_code = 0
!
!  GUI is active only if auto or gui are active
   dlg_active = (ifAutomatic == 0 .and. jscreen /= 0)
!
   spacechar = spaceg_string()
!
!  Apri la finestra per modificare cella,gruppo spaziale, contenuto
   !if (ifAutomatic == 0 .and. jscreen /= 0) then  !if auto or nogui jump this part
   if (dlg_active) then
       cell_cont = cryst(1)%get_strelem()
       if (len_trim(cell_cont) == 0) cell_cont = trim(get_string_directive(commands,'cont'))    !find content in input file
       if (len_trim(cell_cont) == 0) call chemical_formula(celem,cell_cont) !extract from celem if content is not in input file
       c_cell = real(cell(1:6),c_double)
       c_space(:len_trim(spacechar)+1) = toCString(spacechar)
       !allocate(c_content(len_trim(cell_cont)+1))
       c_content(:) = toCString(cell_cont)
       call cell_parameters_dlg(c_cell,spacegr%get_id(),c_space,lenspg,c_content,lencont,kfindspace,kreturn)
       !cell(1:6) = real(c_cell)
       cell_cont = toFortranString(c_content,lencont)
       spacechar = toFortranString(c_space,lenspg)
       if (kreturn == 0) then
           call toolbar_set_state('continue'//char(0),0)
           exit_code = STOP_EVENT
           return  ! exit required by user
       endif
!
       call update_content(cell_cont, ier)
       if (ier /= 0) then
           exit_code = STOP_EVENT
           return
       endif
!
!      Check if cell has been changed ...
       call update_cell(real(c_cell),kgener)
   endif
   iesspace = kfindspace
   iesspacesav = iesspace
!
   if (kfindspace == 0) then
!
!      Prendi il gruppo spaziale dell'utente
       call update_spaceg(spacechar,updated,err)
       if (err%signal) then
           call err%print()
           exit_code = STOP_EVENT
           return
       endif
       call yfl02
!
       ispaceindexing = 0
!
       call dataset_to_thetaint(dataset(1))   ! extra change data: restore from dataset
       call gener(dataset(1)%wave,iflagref,.true.)
       call datanew(POW_FILE,.true.,.true.,exit_code,ier) ! rigenera i riflessi
       call export_file_exp()

   else
!
!      spg may be undef. if Import xpd > Pattern > Space Group Determ.
       if (.not.cryst(1)%is_spg()) then
           call gener_findspace(ier)
           kgener = .true.
       endif
   endif
   if (kgener) call gener(dataset(1)%wave,iflagref,.true.)
!
   end subroutine cell_spg_dialog

!-----------------------------------------------------------------------

   real function ctheta(rho)
!
!-- compute 2-Theta value from ( sin(theta)/lambda ) ** 2
   USE General, only: alambda
   USE trig_constants
   implicit none
   real, intent(in) :: rho
   real :: theta
   theta=sqrt(rho)*alambda
   if (theta.gt.1.) theta=1.
   theta=asin(theta)*rtod
   ctheta=2.0*theta
   return
   end

!-----------------------------------------------------------------------
   subroutine obsin(kv,ibrav,klatt,ksym,vsym)
!
!  Fill irefvet before call this sub. (call checkeq)
!
   USE Riflessi
   USE General
   USE Symmetry
   USE arrayutil
   USE reflection_type_util, only: extin
   USE mpi_prog
   implicit none
   integer, intent(in) :: klatt, ksym
   integer, intent(in) :: ibrav(6,13),vsym(48)
   integer, intent(in) :: kv(3)
   integer :: indh,indk,indl,inbl,id1,jcode,jex
   integer :: indh1,indk1,indl1,idelt,isgnn
   real    :: rho, ctheta, rhomax1
   integer :: hmin,hmax,kmin,kmax,lmin,lmax                  
   logical :: kpr = .false.
   integer :: nhmax,nkmax,nlmax,nru,nr
!
   nhmax=0
   nkmax=0
   nlmax=0
   rhomax1=rhomax
!
!  at this point the allowed reflections are in memory
!
   nru=0
   nr=nrefl
!--
   call indmm_old(hmin,hmax,kmin,kmax,lmin,lmax)
!--
   nhmax=hmin
   nkmax=kmin
   nlmax=lmin
!--
   do indh=hmin,hmax
      do 700 indk=kmin,kmax
         do 600 indl=lmin,lmax
!---
!---        vedere se il nuovo riflesso generato e' equivalente
!---        ad uno gia' considerato, in questo caso :
!---
            if (indh.eq.0.and.indk.eq.0.and.indl.eq.0) go to 600
!   
!           TRIGONALE : va' considerata la condizione aggiuntiva |h|>|k|
!                      nel caso (+-+)
!                     
            if (jsys.eq.5.and.indk.lt.0.and.iabs(indh).lt.iabs(indk)) goto 600
!   
!           CUBICO :  va' considerata la condizione aggiuntiva h<=k<=l
!    
            if(jsys.eq.6.and.(indk.lt.indh.and.indk.gt.indl)) goto 600
!
            indh1=indh
            indk1=indk
            indl1=indl
            call extin(jex,indh1,indk1,indl1,ibrav,klatt,ksym,vsym,nsym,kmat,tmat,icent)
            if (jex.eq.1) goto 600
            call geneq_old(1,indh1,indk1,indl1,idelt,jcode,isgnn,id1,rho)
            inbl=indh1*kv(1) + indk1*kv(2) + indl1 + kv(3) 
!--         in data il superindice e' calcolato come:
!           inbl= lh*kv(1) + lk*kv(2) + ll + kv(3)
            inbl=iabs(inbl)
            if (inbl > size(irefvet)) go to 600 ! outside experimental range
            if (irefvet(inbl).ne.0) go to 600
            rhomax=rhomax1
            if (rho.gt.rhomax1.or.rho.lt.rhomin) go to 600
! --
! --        ibig e' riempito con il riflesso non osservato
! --
            irefvet(inbl)=1
            nru=nru+1
            if (kpr) then
                write(lo,7040) indh1,indk1,indl1,ctheta(rho)
 7040           format(' not measured reflection :',3i4,'  2-Theta = ',f10.4)
            endif
600    continue
700 continue
   enddo
!
   if (mpi_prog_rank == 0) then
       if (nru.gt.0) write(lo,910) nru
!corr      900 format(10x,i5, '  generated reflections ')
  910  format(/,10x,i5, '  not measured reflections in data')
   endif
   call delete_array(irefvet)
!
   return
   end

!-----------------------------------------------------------------------

   subroutine open_cell_list() bind(C,name="open_cell_list")
   USE indexing
   USE Conteggi, only: npunti
   USE variables, only: dataset
   USE enable_amb, only: STOP_EVENT
   implicit none
   integer :: ier,exit_code
!
   if (number_of_cells(cell_list) == 0) return
!
!  scarica in theta_int
   call dataset_to_thetaint(dataset(1))
   call ysfr37(npunti)  ! for extra
!
   call init_recycle()  ! init for extra in cell_dialog (find space g.) and gescom
   call cell_dialog(ier,exit_code)
   if (exit_code == STOP_EVENT) return
   call restart_command()
!
   end subroutine open_cell_list

!---------------------------------------------------------------------

   subroutine restart_command()
   USE General, only: POW_FILE, card
   USE commandsmod
   USE comandiexpo
   USE molcom, only: jscreen
   implicit none
   integer :: iEnd, ier
   integer :: ip
!
   ip = command_position(commands,'data',back=.true.)
   call add_continue(commands,ip)
!
   !call print_commands(commands,0)
   Messages(:) = ' ' ! reset all messagges for sub. scr2
   call create_expo_commands(commands,ipcom,icomq,card)
   icomat = ip
   ipdir = sum(commands(:ip)%ndir)
   if (jscreen > 0) call WaitCursorOn()
   call gescom(POW_FILE,iEnd,ier)
   if (jscreen > 0) call WaitCursorOff()
!
   end subroutine restart_command

!------------------------------------------------------------------

   subroutine data3(active_file,ier)
   USE Riflessi
   USE xdataMod
   USE Conteggi
   USE General
   USE RandomStaff, only: iprandomax,icomatsav,ipdirsav
   USE PatternRef
   USE unit_cell
   USE elements
   USE atom_type_util
   USE mpi_prog
   implicit none
   common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),itry,pfwhm,kequirip
   integer :: ncycl,nroset,nundset,nsec,jback,itry,kequirip
   real :: uvwt,pfwhm
   common/sinte/ abc(3),ang(3),astar,bstar,cstar,cosas,cosbs,coscs
   real :: abc,ang,astar,bstar,cstar,cosas,cosbs,coscs
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: iesdircell,nruntre
   character CellType,CellAxis*3
   common /celsav/cellas(6),porig(6)
   real :: cellas,porig
   common /direx/ncycle,icycle,nosing,izero ,zeroprof,asym,scal,frac,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real :: zeroprof,asym,scal,frac,sigma
   common/spaceindexing /ispaceindexing,fspaceindex(660),ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real :: fspaceindex,percind,errespac
   character*20 lauecell
   integer, intent(in) :: active_file
   integer :: is3s2,natc
   integer :: i,mn
   real :: s3s2p,vol
   integer :: istop,ier,iy
!

   if (active_file == POW_FILE) then
      do i=1,3
         abc(i)=cell(i)
         ang(i)=cell(i+3)
      enddo
      cellas(1)=abc(1)
      cellas(2)=abc(2)
      cellas(3)=abc(3)
      cellas(4)=ang(1)
      cellas(5)=ang(2)
      cellas(6)=ang(3)
      porig(1)=pp(1)
      porig(2)=pp(2)
      porig(3)=pp(3)
      porig(4)=pp(4)
      porig(5)=pp(5)
      porig(6)=pp(6)
      do i=1,npunti
         theta_int(i,4)=theta_int(i,1)
         theta_int(i,5)=theta_int(i,2)
      enddo
!#ifdef MPI
      if (mpi_prog_rank == 0) then
          call ysfr37(npunti)
          call ysfl37(npunti)
      endif
!#endif
   endif
   anat = 0.0
   natc = nint(natom_cell(celem,neutro==0))
   if(natc > 0 .and. spacegr%nsymop > 0) then
      anat = nasym_unit(celem,spacegr%nsymop,neutro==0)
      s3s2p = s3s2_value(celem,neutro,spacegr%ncoper)
      s3s2=s3s2p
      is3s2=int(1./(s3s2p*s3s2p)+0.5)
   endif
!
!  Print info about cell and chemical content
   if (mpi_prog_rank == 0) then
       if (ispaceindexing==0) then
           if (numelem(celem) > 0 .or. iesdircell == 1) write(lo,*)
           if (numelem(celem) > 0) then
               write(lo,'(19x,a,f0.2)') 'Number of atoms in asymmetric unit = ',anat
               write(lo,'(11x,a,i0)') 'Equivalent number of equal atoms in primitive cell = ',is3s2
           endif
           if (iesdircell == 1) then
               vol = cell_volume(cell)
               if (numelem(celem) > 0) write(lo,'(26x,a,f0.2)') 'Volume per atom = ',volume_per_atom(celem,vol)
               write(lo,'(26x,a,f0.2)') 'Volume of the cell = ',vol
           endif
           if (numelem(celem) > 0 .or. iesdircell == 1) write(lo,*)
       endif
   endif
!  calculate number of reflexions for (sem)invariant searching
   if (nstr.gt.0) goto 400
   mn = int(4.0*anat+100.5)
   !if (icent.eq.1) mn = mn + 50
   !if (nsym.eq.1) mn = mn + 50
   if (spacegr%symcent == 1) mn = mn + 50
   if (spacegr%nsymop  == 1) mn = mn + 50
   mn = min0(mn,499)
   nstr=mn+min0(499-mn,100)/2
400 continue
   if (active_file == REFL_FILE) then
!      read reflexion data from cards
!corr       kuser = 3000
       call machkls(filehpow,istop,ier)
       if (ier.lt.0) go to 900
       call defovl
   endif
   if (nrefl > 0) then  
       we(:nrefl) = 1.0
       iprandomax=27
       icomatsav=0
       ipdirsav=0
       fcvet(1:nrefl) = we(1:nrefl)
       if (mpi_prog_numprocs > 1) then
           return
       endif
       call CreaListaRiflessi
!
       if (active_file == REFL_FILE) then
          if (kequirip.eq.1) call noequi
          if(kequirip.eq.1) write(lo,368)
          if(kequirip.eq.0) write(lo,369)
!
          call PreparaRiflessiOrdinati
!
       endif
       call PreparaRiflessiOrdinati
       iy=67543
       call CreaListaRiflessiOrdinati(iy)
368    format(//,'   The overall intensity for each group of ',            &
       'overlapping',/,'   reflections is equipartitioned (default)')
369    format(//,'   The overall intensity for each group of ',            &
       'overlapping',/,'   reflections is not equipartitioned      ')
   endif
   call xdump
900 continue
   return
   end subroutine data3
