module definespace_interop
   use iso_c_binding

   implicit none

   type, bind(C) :: c_spgext_info_t
     integer(c_int)                   :: num      
     character(c_char), dimension(12) :: symb
     character(c_char), dimension(21) :: exts
     real(c_float)                    :: fig      
     integer(c_int)                   :: nabs    
     integer(c_int)                   :: nasym  
     integer(c_int)                   :: freq_no  
     real(c_float)                    :: freq_perc
     integer(c_int)                   :: freq_rank
     integer(c_int)                   :: chiral   
   end type c_spgext_info_t

   type, bind(C) :: c_absref_type
      integer(c_int)                   :: num    
      integer(c_int), dimension(3)     :: hkl    
      character(c_char), dimension(51) :: string 
      real(c_float)                    :: prob   
      integer(c_int)                   :: ktype  
   end type c_absref_type

   contains

   function cast_spgext_to_c(f_spgext) result(c_spgext)
   use spgext, only: spgext_info_t
   use strutil, only: toCString
   use arrayutil, only: logical_to_int
   type(spgext_info_t)   :: f_spgext
   type(c_spgext_info_t) :: c_spgext
!
   c_spgext%num       = int(f_spgext%num,c_int)
   c_spgext%symb(:len_trim(f_spgext%symb)+1) = toCString(f_spgext%symb)
   c_spgext%exts(:len_trim(f_spgext%exts)+1) = toCString(f_spgext%exts)
   c_spgext%fig       = real(f_spgext%fig,c_float)
   c_spgext%nabs      = int(f_spgext%nabs,c_int)
   c_spgext%nasym     = int(f_spgext%nasym,c_int)
   c_spgext%freq_no   = int(f_spgext%freq_no,c_int)
   c_spgext%freq_perc = real(f_spgext%freq_perc,c_float)
   c_spgext%freq_rank = int(f_spgext%freq_rank,c_int)
   c_spgext%chiral    = int(logical_to_int(f_spgext%chiral),c_int)
!
   end function cast_spgext_to_c

!---------------------------------------------------------------------------------
 
   function cast_absref_to_c(f_absref) result(c_absref)
   use spacecorr, only: absref_type
   use strutil, only: toCString
   type(absref_type), intent(in) :: f_absref
   type(c_absref_type)           :: c_absref

   c_absref%num    = int(f_absref%num,c_int)
   c_absref%hkl    = int(f_absref%hkl,c_int)
   c_absref%string(:len_trim(f_absref%string)+1) = toCString(f_absref%string)
   c_absref%prob   = real(f_absref%prob,c_float)
   c_absref%ktype  = int(f_absref%ktype,c_int)

   end function cast_absref_to_c 

end module definespace_interop

   subroutine open_spacegroup_list(kselect)
   use spgext, only: spglist
   use definespace_interop
   implicit none
   interface
      subroutine c_open_spacegroup(spgl,size_list,kselect) bind(C,name="c_open_spacegroup")
      use iso_c_binding
      use definespace_interop
      type(c_spgext_info_t), dimension(*) :: spgl 
      integer(c_int), value, intent(in)   :: size_list
      integer(c_int)                      :: kselect
      end subroutine c_open_spacegroup
   end interface
   integer, intent(out)                             :: kselect
   type(c_spgext_info_t), dimension(:), allocatable :: c_list
   integer                                          :: i
!
   allocate(c_list(size(spglist)))
   do i=1,size(spglist)
      c_list(i) = cast_spgext_to_c(spglist(i))
   enddo
   call c_open_spacegroup(c_list,size(spglist),kselect)
!
   end subroutine open_spacegroup_list

!---------------------------------------------------------------------------------

   subroutine spg_list() bind(C,name="spg_list")
   USE spgext
   USE General, only: POW_FILE
   USE enable_amb
   USE errormod
   USE datamod
   USE variables, only: dataset, cryst
   USE view
   USE reflection_type_util, only: reflection_type
   implicit none
   integer                                          :: kselect,iflagref,ier
   type(error_type)                                 :: err
   type(reflection_type), dimension(:), allocatable :: refsav
!
   call abilita_tasti('indexwin',0)
   call abilita_tasti('peaks',0)
!
!  Use cryst(1) to force update of reflections of laue class
   call cryst(1)%get_reflections(refsav)
   call cryst(1)%set_reflections(reflaue)
   call vedinew(8)
!
   call open_spacegroup_list(kselect)
   if (kselect > 0) then
       call make_new_spg(spglist(kselect)%symb,2,err)
       if (err%signal) then
           call err%print()
           return
       endif
!
       call gener(dataset(1)%wave,iflagref,.true.)
!
       call data3(POW_FILE,ier)
!
       call restart_command() 
!
   else
!
!      restore reflections for close event
       call cryst(1)%set_reflections(refsav)
       call vedinew(8)
   endif
   call abilita_tasti('indexwin',1)
   call abilita_tasti('peaks',1)
!
   end subroutine spg_list

!---------------------------------------------------------------------------------

   subroutine definespace2(active_file,imer,exit_code,ier)
   USE Riflessi
   USE SPACECORR
   USE General
   USE Symmetry, only: ksys
   USE Molcom, only: jscreen
   USE PatternRef, only: kalpha   
   USE counts, only: dvalue
   USE spgext  
   USE strutil, only: lower
   USE enable_amb
   USE arrayutil
   USE errormod
   USE variables, only: dataset, ref
   USE datamod
   USE extramodule, only: indov,eosse,weigt,allocate_extra
   USE reflection_type_util
!
   implicit none
!
   integer, intent(in) :: active_file
   integer, intent(in)  :: imer
   integer, intent(out) :: exit_code
   integer, intent(out) :: ier
   common/dummy/tab(13,8,6),tfo,tsfo,tfosfo,lh,lk,ll,fo,sfo,weigg   &
     ,msys,numcond(13),ncont(13),nocond(13),isolu(30),tabnew(13,8)  &
     ,lsolu(30),foclass(13),ipow,rhosog,nrefus,tabfomax(13,8,2)     &
     ,numcond0(13),wclass(13)
   real :: tab,tfo,tsfo,tfosfo,fo,sfo,weigg,tabnew,foclass,rhosog,tabfomax,wclass
   integer :: lh,lk,ll,msys,numcond,ncont,nocond,isolu,lsolu,ipow,nrefus,numcond0
   common/dumm3/indla,indexx,numsol,ilegen
   integer :: indla,indexx,numsol,ilegen
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common/fordata/nsymbo,symbs(200,2),fmerit(200,2),numerosy(200,2)
   integer :: nsymbo,numerosy
   real :: fmerit
   common/spaceforplot/numsymb,spacesymb(400),figsymb(400),isymb(400),numerosymb(400)
   real :: figsymb
   integer :: numsymb,isymb,numerosymb
   common /pseudoappl/iapplpse(74),iappl     
   integer :: iapplpse, iappl
   common/ylimi/nfl,lfl,idummy(4),nfls
   integer :: nfl,lfl,idummy,nfls
   common/spaceindexing /ispaceindexing,fspaceindex(660),    &
          ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing, ntotmult1,ntotmult2
   real :: fspaceindex, percind, errespac
   character(len=20) :: lauecell
   common /prova_kalpha/ kalphasav
   integer :: kalphasav
   integer, dimension(4) :: reflim
   integer :: i,nn,nrife,kselect,icsys,nsb,propt,iflagref
   real :: sog,dval
   character*20 symbs     
   character*20 spacesymb     
   character*50 space_group
   integer, dimension(:), allocatable :: vnabs
   type(error_type)                   :: errspg
!
   nfl=1
   ier = 0
   exit_code = 0
   if(ksys.ne.1)then
      sog=0.8
      call defovlrs(sog)
      call data3(active_file,ier)
      call normal_forspace(ier)
      if(ier.lt.0) return
   else
      msys=0
      indov(:) = 1
!corr        !write(0,*)'indov=',size(indov),nrefl
!corr      if (.not.allocated(indov)) then
!corr          call allocate_extra(nrefl)
!corr          indov(:) = 1
!corr      endif
   endif
   if(msys.eq.0)then
      numsymb=0
      nsymbo=1
      symbs(1,1)='P 1'      
      symbs(1,2)='P 1'      
      fmerit(1,1)=1.0                
      fmerit(1,2)=1.0                
!!!!!    imer=1
      numerosy(1,1) = 217                      
      numerosy(1,2) = 217                      
   endif
   nn=numsymb
   do i=1,nsymbo
      if((nn+1).gt.400)go to 222
      if(fmerit(i,1).gt.0.0)then
        nn=nn+1
        spacesymb(nn)=symbs(i,1)
        figsymb(nn)=fmerit(i,1)
        isymb(nn)=imer
        numerosymb(nn) = numerosy(i,1)
        if(iappl.eq.1)then
          nn=nn+1
          spacesymb(nn)=symbs(i,2)
          figsymb(nn)=fmerit(i,2)
          isymb(nn)=imer
          numerosymb(nn) = numerosy(i,2)
        endif
      endif
222   continue
   enddo
   numsymb=nn
   if(imer.ne.nlin)return
!CANGELA
   numsol=1
!CANGELA
   reflim=0
   do i=1,nrefl
      dval = dvalue(tthet(i),alambda)
      if (dval >= 2.25) reflim(1)=i+1
      if (dval >= 2.25) reflim(2)=i+1
      if (dval >= 2.0)  reflim(3)=i+1
      if (dval >= 5.0)  reflim(4)=i+1
      if (dval <  2.0)  exit
   enddo
!
!  Save array to return on space group list if GUI is active
   spglaue = spacegr
   call new_array(hklspg,[1,1],[nrefl,3]) 
   hklspg=IndHKL(1:nrefl,1:3)
   call new_array(indspg,nrefl)
   indspg=indov(1:nrefl)
   call new_array(weispg,nrefl); 
   weispg=weigt(:nrefl)*eosse(:nrefl)
   call copy_ref(reflaue, ref)
!
!  Make array vnabs with number of single absent reflections
   call new_array(vnabs,numsymb/1+iappl)
   nsb = 0
   do i=1,numsymb,1+iappl
      reflim=abs(reflim)
      nrife=numerosymb(i)
      icsys = csys_from_exts_num(nrife)
      call search_extintion(adjustl(spacesymb(i)),hklspg,icsys,indspg,weispg,spglaue,reflim=reflim)
      nsb = nsb + 1
      !vnabs(nsb) = count(vethkl(:) == 1)  ! only absent single reflections
      vnabs(nsb) = count(vethkl(:) > 0)    ! all absences
   end do
!
   if (ispaceindexing==0) then
       write(lo,'(/a)')' Extinction Group         Fig.Mer    '
       if (iappl==0) then
          do i=1,numsymb
             write(lo,'(4x,a,f9.3)') spacesymb(i),figsymb(i)
          end do
       else
          do i=1,numsymb,2
             write(lo,'(4x,a,f9.3)') spacesymb(i),figsymb(i)
          end do
          write(lo,'(//3x,31("*")/,3x,a/,3x,31("*")/)')'Correction by Pseudotranslation'
          write(lo,*)' Extinction Group         Fig.Mer    '
          do i=2,numsymb,2
             write(lo,'(4x,a,f9.3)') spacesymb(i),figsymb(i)
          end do
       end if
!corr       write(lo,'(//3x,40("*")/,4x,a/,3x,40("*")/)')'For more details look at stat.spg file'
       close(47)
   endif
!
   if (ispaceindexing == 1) then
      if (ksys /= 1) then
!
!         Create list of space groups from extinction symbols;  pseudo (iapply=1) is excluded
          call spg_from_exts(numerosymb(1:numsymb:1+iappl),figsymb(1:numsymb:1+iappl),vnabs,spglist,celem)
          space_group = spglist(1)%symb
      else
          space_group = ' p -1'
      endif
   else
!     se ho dato peaksearch + findspace non rifaccio correzione kalpha
      if (kalphasav.eq.1.and.kalpha.eq.0) kalpha = 1
!
!     Create list of space groups from extinction symbols;  pseudo (iapply=1) is excluded
      call spg_from_exts(numerosymb(1:numsymb:1+iappl),figsymb(1:numsymb:1+iappl),vnabs,spglist,celem)
      write(lo,*)
      call print_spglist(spglist,lo)
!
      if (ifAutomatic == 1 .or. jscreen == 0) then
          space_group = spglist(1)%symb
      else
          call toolbar_set_state('continue'//char(0),0)
          call abilita_tasti('window_pattern')
          call open_spacegroup_list(kselect)
          if (kselect > 0) then
              space_group = spglist(kselect)%symb
              call toolbar_set_state('continue'//char(0),1)
          else
              exit_code = STOP_EVENT
          endif
      endif
   endif
   call delete_array(vethkl) ! disable visualization of absent refle.
!
   if (exit_code == STOP_EVENT) return
   propt = 2
   if (ispaceindexing /= 0) propt = 0
   call make_new_spg(space_group,propt,errspg)
   if (errspg%signal) then
       call errspg%print()
       exit_code = STOP_EVENT
       return
   endif
   call xfl01
   call yfl02
!
   if (ispaceindexing==0) then 
       call gener(dataset(1)%wave,iflagref,.true.)
   endif
!
   if(imer.eq.nlin)iesspace=0
   return
   end

!---------------------------------------------------------------------------------

   subroutine spg_selection(rowspace) bind(C,name="spg_selection")
!
!  Callback associated to the selection on the list of space groups
!
   USE SPACECORR
   USE View
   USE General, only: nrefl
   USE PatternRef, only: pos_absref
!corr   USE Conteggi, only:npunti
   USE plotstyle
   USE spginfom, only: csys_code_from_num
   USE spgext
   use iso_c_binding
   implicit none
   interface
      subroutine add_systematic_absences(sysabs,nsysabs) bind(C,name="add_systematic_absences")
      use iso_c_binding
      integer(c_int), dimension(*)      :: sysabs
      integer(c_int), intent(in), value :: nsysabs
      end subroutine add_systematic_absences
   end interface
   integer(c_int), intent(in), value         :: rowspace
   integer             :: isys
   integer(c_int)                            :: nsysabs
   integer(c_int), dimension(:), allocatable :: sysabs
   integer                                   :: i
!
   !write(0,*)'SPG_SELECTION=',rowspace,spglist(rowspace)%num,spglist(rowspace)%symb
   pos_absref = 0    ! azzera la posizione del riflesso estinto da visualizzare
!
!  Cerca riflessi estinti
   isys = csys_code_from_num(spglist(rowspace)%num)
   call search_extintion(spglist(rowspace)%exts,hklspg,isys,indspg,weispg,spglaue)
   !call vedinew(pinioss=1,pfinoss=npunti)
!
!  Add systematic absences to graph  
   nsysabs = count(vethkl > 0)
   allocate(sysabs(nsysabs))
   if (nsysabs > 0) then
       nsysabs = 0
       do i=1,size(vethkl)
          if (vethkl(i) > 0) then
              nsysabs = nsysabs + 1
              sysabs(nsysabs) = i - 1 ! array start from 0 in C++
   endif
       enddo
   endif
   call add_systematic_absences(sysabs,nsysabs)
   !write(0,*)"N: ",size(sysabs)," VET: ",sysabs
!
!qt   if (style(STYLE_BG)%colline == 1) then                       ! Scrivi messaggio
!qt       call PrintMess('Systematic absent reflections are indicated by black bars '// &
!qt                '(single refl. or all the overlapped group extinct) or magenta bars')
!qt   else
!qt       call PrintMess('Systematic absent reflections are indicated by white bars '// &
!qt        '(single refl. or all the overlapped group extinct) or magenta bars')
!qt   endif
!
   end subroutine spg_selection

!---------------------------------------------------------------------

   subroutine get_nabsref(rowspace, nabsref_forC) bind(C,name="get_nabsref")
!
!  Callback associated to the click on the list of reflection conditions
!
   USE General, only: iscra
   USE SPACECORR
!corr   USE strutil
   USE spgext
   USE spginfom, only: csys_code_from_num
   use iso_c_binding
!
   implicit none
   integer(c_int), intent(in), value  :: rowspace
   integer(c_int), intent(out)        :: nabsref_forC
   real, dimension(size(weispg))      :: prob
!corr   integer, dimension(:), allocatable :: iord
   integer                            :: isys
!
!  Genera probabilita' che il riflesso sia estinto
   prob(:) = 1 - weispg(:)
   where(prob(:) < 0.0) prob(:) = 0
!
!  Genera absref, vettore di informazioni  sui riflessi estinti
   isys = csys_code_from_num(spglist(rowspace)%num)
   call search_extintion(spglist(rowspace)%exts,hklspg,isys,indspg,prob,spglaue,kpr=iscra)
   nabsref_forC = nabsref
!   
!  Ordina in base alla stringa dell'operatore
!corr   if (nabsref > 1) then
!corr       allocate(iord(nabsref))
!corr       call svec_sort_heap_a_index(nabsref,absref(:nabsref)%string,iord)
!corr       absref(:nabsref) = absref(iord)
!corr           !write(0,*)'ABS=',count(absref(:nabsref)%ktype == 1),count(vethkl(:) == 1)
!corr   endif
!
   end subroutine get_nabsref

!-----------------------------------------------------------------------

   subroutine get_absref(absref_forC) bind(C,name="get_absref")
   USE SPACECORR
   use definespace_interop
!
   implicit none
   type(c_absref_type), dimension(*) :: absref_forC
   integer :: i
   
!corr   absref_forC(1:nabsref) = absref(:nabsref)
!corr   do i = 1, nabsref
!corr      absref_forC(i)%string = trim(absref(i)%string) // char(0)
!corr   end do
   do i = 1, nabsref
      absref_forC(i) = cast_absref_to_c(absref(i))
   end do
!
   end subroutine get_absref

!-----------------------------------------------------------------------
!corr
!corr   subroutine SelectedNum(num) bind(C,name="SelectedNum")
!corr!
!corr!  Click on list of reflection conditions
!corr!
!corr   USE PatternRef, only:pos_absref
!corr   USE View
!corr   use iso_c_binding
!corr                 !USE SPACECORR
!corr   implicit none
!corr   integer(c_int), intent(in), value :: num
!corr!
!corr   pos_absref = num
!corr!cor   write(0,*)'REF:',num,absref(num)%num
!corr   call vedinew()
!corr!
!corr   end subroutine SelectedNum
!corr
!---------------------------------------------------------------------------------

   subroutine spg_determine() bind(C,name="spg_determine")
   USE General, only: POW_FILE
   USE commandsmod
   USE comandiexpo, only: icomat,ipdir
   USE enable_amb, only: STOP_EVENT
   implicit none
   integer :: kfindspace, ier, exit_code
!
   if (ncommands(commands) == 0) then
!
!      spg required after Import Diffraction Pattern
       call add_command(commands,0,[EXTRA_CMD,CONTINUE_CMD])
       !call print_commands(commands,0)
       icomat = icomat + 1
       ipdir = 0
       call data_for_spg()
   endif
!
   kfindspace = 1
   call cell_spg_dialog(kfindspace,exit_code)
   if (exit_code == STOP_EVENT) return
   if (kfindspace == 1) then
       call datanew(POW_FILE,.true.,.true.,exit_code,ier)
       if (exit_code == STOP_EVENT) return
   endif
!
   call restart_command()
!
   end subroutine spg_determine

!---------------------------------------------------------------------------------

   subroutine update_spaceg(spaceg,updated,err)
   use spginfom
   use General, only: spacegr
   use errormod
   implicit none
   character(len=*), intent(in)  :: spaceg
   logical, intent(out)          :: updated
   type(error_type), intent(out) :: err
   type(spaceg_type)             :: spg
!
   updated = .false.
   spg = init_spaceg_type(spaceg)
   if (spg%undef()) then
       call err%set("Unknown space group name: "//trim(spaceg))
       return
   endif
   if (spg_equal(spg,spacegr)) return
   call make_new_spg1(spg,0)
   updated = .true.
!
   end subroutine update_spaceg

!---------------------------------------------------------------------------------

   subroutine make_new_spg1(spg_new,propt)
   USE General, only: spacegr,lo,cell
   USE variables, only: cryst
   USE strutil
   USE spginfom
   !USE errormod
   USE sinterface
   USE prog_constants
   USE unit_cell
   USE messagemod
   implicit none
   type(spaceg_type)             :: spg_new
   integer, intent(in)           :: propt
   character(len=16)             :: space_group
   integer                       :: ishif,ier,ier_std,iExCode,i
   integer, dimension(3)         :: kshif
   type(spaceg_type)             :: spg
   type(cell_type)               :: cellt
!
   spacegr = spg_new
   if (propt > 0) then
       if (propt > 1) write(lo,'(/,45h The following spacegroup has been selected: ,a,/)') trim(spacegr%symbol_xhm)
       if (is_standardize(spacegr)) then
           spg = standard_spg(spacegr)
           call MsgWinErr('Warning',   &
           'Do you want to transform to standard setting '//trim(spg%symbol_xhm)   &
           ,QUEST_WINDOW_NO_YES,iExCode)
           if (iExCode == 1) then
               cellt = set_cell_type(cell)
               call standard_symmetry(spacegr,cellt,lo,ier_std)
               if (ier_std == 0) then
                   cell(:6) = cellt%get_par()
                   call cryst(1)%set_unit_cell(cellt)
                   call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
               endif
           endif
       endif
   endif
   call cryst(1)%set_spg(spacegr)
   space_group = lower(spacegr%symbol_xhm)
   call StandardSPG(space_group,ier)
   if (ier == 0) then
       ishif = 0
       kshif(:) = 0
       call insym(ishif,kshif,ier)
   else
       call set_spaceg_new(spacegr)
   endif
   if (propt > 0) then
       call WriteSymmInfo(lo)
   endif
   call xspinb()
   call yfrm02
   !corr - call sc014(ix)
!
   end subroutine make_new_spg1

!---------------------------------------------------------------------------------

   subroutine make_new_spg(spgstring,propt,err)
!corr   USE General, only: spacegr,lo,cell
!corr   USE variables, only: cryst
!corr   USE strutil
   USE spginfom
   USE errormod
!corr   USE sinterface
!corr   USE prog_constants
!corr   USE unit_cell
   implicit none
   character(len=*), intent(in)  :: spgstring
   integer, intent(in)           :: propt
   type(error_type), intent(out) :: err
!corr   character(len=16)             :: space_group
!corr   integer                       :: ishif,ier,ix,ier_std,iExCode,i
!corr   integer, dimension(3)         :: kshif
   type(spaceg_type)             :: spg
!corr   type(cell_type)               :: cellt
!
   spg = init_spaceg_type(spgstring)
   if (spg%undef()) then
       call err%set("Unknown space group name: "//trim(spgstring))
       return
   endif
   call make_new_spg1(spg,propt)
!corr   if (propt > 0) then
!corr       if (propt > 1) write(lo,'(/,45h The following spacegroup has been selected: ,a,/)') trim(spgstring)
!corr       if (is_standardize(spacegr)) then
!corr           spg = standard_spg(spacegr)
!corr           call MsgWinErr('Warning',   &
!corr           'Do you want to transform to standard setting '//trim(spg%symbol_xhm)   &
!corr           ,QUEST_WINDOW_YES_NO,iExCode)
!corr           if (iExCode == 1) then
!corr               cellt = set_cell_type(cell)
!corr               call standard_symmetry(spacegr,cellt,lo,ier_std)
!corr               if (ier_std == 0) then
!corr                   cell(:6) = cellt%get_par()
!corr                   call cryst(1)%set_unit_cell(cellt)
!corr                   call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
!corr               endif
!corr           endif
!corr       endif
!corr   endif
!corr   call cryst(1)%set_spg(spacegr)
!corr   space_group = lower(spacegr%symbol_xhm)
!corr   call StandardSPG(space_group,ier)
!corr   if (ier == 0) then
!corr       ishif = 0
!corr       kshif(:) = 0
!corr       call insym(ishif,kshif,ier)
!corr   else
!corr       call set_spaceg_new(spacegr)
!corr   endif
!corr   if (propt > 0) then
!corr       call WriteSymmInfo(lo)
!corr   endif
!corr   call xspinb()
!corr   call yfrm02
!corr   call sc014(ix)
!
   end subroutine make_new_spg

!---------------------------------------------------------------------------------

   subroutine data_for_spg()
   implicit none
   common /ksearch/ ksearch
   integer :: ksearch
   common/iesext/iesext
   integer :: iesext
   common /profp/dumpp,nprof,ipol
   real :: dumpp
   integer :: nprof,ipol
   iesext = 0
   ksearch = 0
   nprof = 1
   call init_recycle()  ! initialize variable for recycle
   end subroutine data_for_spg
