#include "exploretrialsdialog.h"
#include "ui_exploretrialsdialog.h"
#include "checkboxheader.h"
#include "floatdelegate.h"
#include "checkboxdelegate.h"
#if !defined(JAV_WIDGET)
#include "mainwindow.h"
#endif
#include <QDebug>
#include <QPushButton>

extern "C" void c_finish_restart(int kproc, int saf, int selected[], trialInfo tInfo[], int *selset);
extern "C" void TrialSelected(int selrow);
extern "C" void quit_restart(int kproc, int ksaf);

ExploreTrialsDialog::ExploreTrialsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ExploreTrialsDialog)
{
    ui->setupUi(this);

    ui->buttonBox->button(QDialogButtonBox::Apply)->setText("Explore");
    ui->buttonBox->button(QDialogButtonBox::Apply)->setIcon(QIcon(":/pixmaps/pixmaps/go.png"));

    createTrialsModel();
    setTrialsTableView();
}

void ExploreTrialsDialog::setTrials(int kproc, int saf, int nset, int selset, trialInfo tInfo[])
{
    ui->fourProcedureComboBox->setCurrentIndex(kproc-1);
    ui->safCheckBox->setChecked(saf);

    fillTrialsModel(nset, tInfo);

    ui->trialsTableView->resizeColumnsToContents();
    //ui->trialsTableView->horizontalHeader()->stretchLastSection(); //NOT WORKING!
    ui->trialsTableView->horizontalHeader()->setSectionResizeMode(4,QHeaderView::Stretch);
    QModelIndex index = trialsProxyModel->mapFromSource(trialsModel->index(selset-1, 0));
    ui->trialsTableView->selectRow(index.row());
    if (exploredTrialsCount() == trialsModel->rowCount())
        ui->trialsTableView->sortByColumn(4,Qt::AscendingOrder);
}

ExploreTrialsDialog::~ExploreTrialsDialog()
{
    delete ui;
}

void ExploreTrialsDialog::createTrialsModel()
{
    trialsModel = new QStandardItemModel(this);
    trialsProxyModel = new QSortFilterProxyModel(this);
    trialsProxyModel->setSourceModel(trialsModel);
    trialsModel->setColumnCount(5);
    QStringList titles = {"Explore", "Set", "cfom", "done", "RF"};
    trialsModel->setHorizontalHeaderLabels(titles);
}

void ExploreTrialsDialog::fillTrialsModel(int ntrials, trialInfo tInfo[])
{
    trialsModel->setRowCount(ntrials);
    if (ntrials > 0) {

        for (int r = 0; r < ntrials; r++) {
            QStandardItem *item = new QStandardItem();
            item->setData(0,Qt::DisplayRole);
            trialsModel->setItem(r,0,item);

            item = new QStandardItem();
            item->setData(tInfo[r].set,Qt::DisplayRole);
            item->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
            trialsModel->setItem(r,1,item);

            item = new QStandardItem();
            item->setData(tInfo[r].cfom,Qt::DisplayRole);
            trialsModel->setItem(r,2,item);

            item = new QStandardItem(tInfo[r].done ? "Yes" : "No");
            trialsModel->setItem(r,3,item);

            item = new QStandardItem();
            if (tInfo[r].done)
                item->setData(tInfo[r].rf,Qt::DisplayRole);
            else
                item->setData("-",Qt::DisplayRole);
            trialsModel->setItem(r,4,item);
        }
    }
}

void ExploreTrialsDialog::updateTrialsModel(trialInfo tInfo[])
{
    for (int r = 0; r < trialsModel->rowCount(); r++) {
        trialsModel->item(r,3)->setData((tInfo[r].done ? "Yes" : "No"),Qt::DisplayRole);
        if (tInfo[r].done)
            trialsModel->item(r,4)->setData(tInfo[r].rf,Qt::DisplayRole);
        else
            trialsModel->item(r,4)->setData("-",Qt::DisplayRole);
    }
}

void ExploreTrialsDialog::setTrialsTableView()
{
    ui->trialsTableView->setModel(trialsProxyModel);    
    ui->trialsTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->trialsTableView->setSortingEnabled(true);
    ui->trialsTableView->sortByColumn(1,Qt::AscendingOrder);
    ui->trialsTableView->verticalHeader()->hide();
    ui->trialsTableView->setShowGrid(false);
    ui->trialsTableView->setAlternatingRowColors(true);
    ui->trialsTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->trialsTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->trialsTableView->horizontalHeader()->stretchLastSection();
    ui->trialsTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);

    auto floatDelegate = new FloatDelegate(ui->trialsTableView);
    floatDelegate->setDecimals(3);
    ui->trialsTableView->setItemDelegateForColumn(2,floatDelegate);
    ui->trialsTableView->setItemDelegateForColumn(4,floatDelegate);
    auto checkDelegate = new CheckBoxDelegate(ui->trialsTableView);
    ui->trialsTableView->setItemDelegateForColumn(0, checkDelegate);
    connect(checkDelegate, &CheckBoxDelegate::rowChecked, this, &ExploreTrialsDialog::rowCheckBoxClicked);
    cbHeader = new CheckBoxHeader(Qt::Horizontal, {0}, true, ui->trialsTableView);
    ui->trialsTableView->setHorizontalHeader(cbHeader);
    cbHeader->setChecked(0, false);
    connect(cbHeader, &CheckBoxHeader::checkBoxClicked, this, &ExploreTrialsDialog::headerCheckBoxClicked);

    ui->trialsTableView->sortByColumn(1,Qt::AscendingOrder);

    connect(ui->trialsTableView->selectionModel(), &QItemSelectionModel::currentRowChanged, this, &ExploreTrialsDialog::trialSelected);
}

int ExploreTrialsDialog::rowCheckBoxClickedCount()
{
    int count = 0;
    for (int r = 0; r < trialsModel->rowCount(); r++) {
        QModelIndex index = trialsModel->index(r,0,QModelIndex());
        if(trialsModel->data(index).toBool()) count++;
    }
    return count;
}

int ExploreTrialsDialog::exploredTrialsCount()
{
    int count = 0;
    for (int row = 0; row < trialsModel->rowCount(); row++) {
        QModelIndex index = trialsModel->index(row, 3, QModelIndex());
        if (trialsModel->data(index).toString() == "Yes") count++;
    }
    return count;
}

void ExploreTrialsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Apply) {
        if (rowCheckBoxClickedCount() > 0) {
            int kproc = ui->fourProcedureComboBox->currentIndex()+1;
            int ntrials = trialsModel->rowCount();
            int *selected = new int[ntrials];
            trialInfo *tInfo = new trialInfo[ntrials];
            for (int r = 0; r < trialsModel->rowCount(); r++) {
                QModelIndex index = trialsModel->index(r,0,QModelIndex());
                selected[r] = trialsModel->data(index).toBool();
                trialsModel->setData(index,false);
            }
            ui->exploreCheckBox->setChecked(false);

            this->hide();
            int saf = ui->safCheckBox->isChecked();
            int selset;
            c_finish_restart(kproc, saf, selected, tInfo, &selset);

            updateTrialsModel(tInfo);
            delete [] selected;
            delete [] tInfo;
            this->show();

            QModelIndex index = trialsProxyModel->mapFromSource(trialsModel->index(selset-1, 0));
            ui->trialsTableView->selectRow(index.row());
            if (exploredTrialsCount() == trialsModel->rowCount())
                ui->trialsTableView->sortByColumn(4,Qt::AscendingOrder);

        }
    } else if (stdButton == QDialogButtonBox::Close) {
        int kproc = ui->fourProcedureComboBox->currentIndex()+1;
        int saf = ui->safCheckBox->isChecked();
        quit_restart(kproc,saf);
    }
}

void ExploreTrialsDialog::trialSelected(const QModelIndex &index)
{
    QModelIndex pindex = trialsProxyModel->mapToSource(index);
    TrialSelected(pindex.row()+1);
#if !defined(JAV_WIDGET)
    //Raise JAV window
    MainWindow *mw = getMainWindow();
    if (!mw) return;

    JavWindow *jav = mw->getJav();
    if (jav->isVisible()) {
        if (!this->window()->geometry().intersects(jav->geometry()))
            jav->raise();
    }
#endif
}

void ExploreTrialsDialog::rowCheckBoxClicked(const QModelIndex &index, bool state)
{    
    //If state is false uncheck the checkBoxHeader
    if (!state && cbHeader->isChecked(index.column())) {
        qInfo() << "Change chekboxheader";
        cbHeader->setChecked(index.column(), false);
        return;
    }

    //If state is true check the checkBoxheader if all checkboxes have been clicked
    if (state && !cbHeader->isChecked(index.column())) {
        if (rowCheckBoxClickedCount() == trialsModel->rowCount())
            cbHeader->setChecked(index.column(), true);
    }
}

void ExploreTrialsDialog::headerCheckBoxClicked(bool state)
{
    for (int row = 0; row < trialsModel->rowCount(); row++) {
        QModelIndex index = trialsModel->index(row, 0, QModelIndex());
        trialsModel->setData(index, QVariant(state?1:0));
    }
}

void ExploreTrialsDialog::on_exploreCheckBox_stateChanged(int arg1)
{
    int state = arg1 == Qt::Checked ? 1 : 0;
    bool stateChanged = false;
    for (int row = 0; row < trialsModel->rowCount(); row++) {
        QModelIndex index = trialsModel->index(row, 3, QModelIndex());
        if (trialsModel->data(index).toString() == "No") {
            index = trialsModel->index(row, 0, QModelIndex());
            trialsModel->setData(index, QVariant(state));
            stateChanged = true;
        }
    }
    if (stateChanged) rowCheckBoxClicked(QModelIndex(), state);
}
