#include <QDebug>
#include <QColor>
#include <algorithm>
#include "libcomune.h"
#include "models.h"
#include "properties.h"
#include "glutils.h"

typedef struct {
                 unsigned long ulDimensionX;
                 unsigned long ulDimensionY;
                 int            n_channels;
                 int            rowstride;
                 GLenum         format;
                 unsigned char  *pcData;
               } Texture ;

#define CACHE_SIZE      240

void disegnaSfera (double radius, int slices, bool textureCoords)
{
    GLint i,j;
    GLfloat sinCache1a[CACHE_SIZE];
    GLfloat cosCache1a[CACHE_SIZE];
    GLfloat sinCache2a[CACHE_SIZE];
    GLfloat cosCache2a[CACHE_SIZE];
    GLfloat sinCache1b[CACHE_SIZE];
    GLfloat cosCache1b[CACHE_SIZE];
    GLfloat sinCache2b[CACHE_SIZE];
    GLfloat cosCache2b[CACHE_SIZE];
    GLfloat angle;
    GLfloat zLow, zHigh;
    GLfloat sintemp1 = 0.0, sintemp2 = 0.0, sintemp3 = 0.0, sintemp4 = 0.0;
    GLfloat costemp3 = 0.0, costemp4 = 0.0;
    GLint start, finish;
    float PI= (float) M_PI;

    if (slices >= CACHE_SIZE) slices = CACHE_SIZE-1;
    if (slices < 2 || radius < 0.0) {
         return;
    }

    /* Cache is the vertex locations cache */
    /* Cache2 is the various normals at the vertices themselves */


    for (i = 0; i < slices; i++) {
         angle = 2 * PI * i / slices;
         sinCache1a[i] = sinf(angle);
         cosCache1a[i] = cosf(angle);
         sinCache2a[i] = sinCache1a[i];
         cosCache2a[i] = cosCache1a[i];
    }

    for (j = 0; j <= slices; j++) {
         angle = PI * j / slices;
         sinCache2b[j] = sinf(angle);
         cosCache2b[j] = cosf(angle);
         sinCache1b[j] = (float)radius * sinf(angle);
         cosCache1b[j] = (float)radius * cosf(angle);
    }
    sinCache1b[0] = 0;
    sinCache1b[slices] = 0;

    sinCache1a[slices] = sinCache1a[0];
    cosCache1a[slices] = cosCache1a[0];
    sinCache2a[slices] = sinCache2a[0];
    cosCache2a[slices] = cosCache2a[0];

// Do ends of sphere as TRIANGLE_FAN's (if not texturing)
    if (!(textureCoords))
    {
        start = 1;
        finish = slices - 1;

// Low end first (j == 0 iteration)
        sintemp2 = sinCache1b[1];
        zHigh = cosCache1b[1];
        sintemp3 = sinCache2b[1];
        costemp3 = cosCache2b[1];
        glNormal3f(sinCache2a[0] * sinCache2b[0],
        cosCache2a[0] * sinCache2b[0],
        cosCache2b[0]);
        glBegin(GL_TRIANGLE_FAN);
        glVertex3d(0.0, 0.0, radius);
        for (i = slices; i >= 0; i--)
        {
             glNormal3f(sinCache2a[i] * sintemp3,
             cosCache2a[i] * sintemp3,
             costemp3);
             glVertex3f(sintemp2 * sinCache1a[i],
                 sintemp2 * cosCache1a[i], zHigh);
        }
        glEnd();

// High end next (j == slices-1 iteration) 
        sintemp2 = sinCache1b[slices-1];
        zHigh = cosCache1b[slices-1];
        sintemp3 = sinCache2b[slices-1];
        costemp3 = cosCache2b[slices-1];
        glNormal3f(sinCache2a[slices] * sinCache2b[slices],
           cosCache2a[slices] * sinCache2b[slices], cosCache2b[slices]);
        glBegin(GL_TRIANGLE_FAN);
        glVertex3d(0.0, 0.0, -radius);
        for (i = 0; i <= slices; i++)
        {
             glNormal3f(sinCache2a[i] * sintemp3,
                cosCache2a[i] * sintemp3, costemp3);
             glVertex3f(sintemp2 * sinCache1a[i],
                sintemp2 * cosCache1a[i], zHigh);
        }
        glEnd();
    }
    else
    {
        start = 0;
        finish = slices;
    }
    for (j = start; j < finish; j++)
    {
         zLow = cosCache1b[j];
         zHigh = cosCache1b[j+1];
         sintemp1 = sinCache1b[j];
         sintemp2 = sinCache1b[j+1];
         sintemp3 = sinCache2b[j+1];
         costemp3 = cosCache2b[j+1];
         sintemp4 = sinCache2b[j];
         costemp4 = cosCache2b[j];

         glBegin(GL_QUAD_STRIP);
         for (i = 0; i <= slices; i++)
         {
              glNormal3f(sinCache2a[i] * sintemp3,
                cosCache2a[i] * sintemp3, costemp3);
              if (textureCoords)
                  glTexCoord2f(1-(float)i/slices, 1-(float)(j+1)/slices);
              glVertex3f(sintemp2 * sinCache1a[i],
                sintemp2 * cosCache1a[i], zHigh);
              glNormal3f(sinCache2a[i] * sintemp4,
                cosCache2a[i] * sintemp4, costemp4);
              if (textureCoords)
                  glTexCoord2f(1-(float)i/slices, 1-(float)j/ slices);
              glVertex3f(sintemp1 * sinCache1a[i],
                sintemp1 * cosCache1a[i], zLow);
         }
         glEnd();
    }
}

void disegnaCilindro(double radius, double h, int slices)
{
    int  DEF_D=5;
    if(slices > 180)
       DEF_D = max(1, (int)(900.f/(float)slices));

    glBegin(GL_QUAD_STRIP);
    for (int j=0;j<=360;j+=DEF_D)
    {
      double c1 = Cos(j);
      double s1 = Sin(j);
      glNormal3d(radius*c1,radius*s1,h);
      glVertex3d(radius*c1,radius*s1,h);
      glNormal3d(radius*c1,radius*s1,0);
      glVertex3d(radius*c1,radius*s1,0);
    }
    glEnd();
}

void disegnaCono(double raggio, double h, int subdiv)
{
    size_t  DEF_D=5, UNO=1, ang0=0, ang1=360;
    if(subdiv > 180)
       DEF_D = max(UNO, static_cast<size_t>(900.f/static_cast<float>(subdiv)));
    size_t nAngs = 360/DEF_D + 1;
    double *cs = new double [nAngs]();
    double *ss = new double [nAngs]();
    size_t i=0;
    for (size_t k=ang0;k<=ang1;k+=DEF_D)
    {
         cs[i] = Cos(k);
         ss[i] = Sin(k);
         i++;
    }
   
    // sides 
    i = 0;
    glBegin(GL_TRIANGLES);
    for (size_t k=ang0;k<ang1;k+=DEF_D)
    {
      glNormal3d(0,0,h);
      glVertex3d(0,0,h);
      glNormal3d(raggio*cs[i],raggio*ss[i],0);
      glVertex3d(raggio*cs[i],raggio*ss[i],0);
      glNormal3d(raggio*cs[i+1],raggio*ss[i+1],0);
      glVertex3d(raggio*cs[i+1],raggio*ss[i+1],0);
      i++;
    }
    glEnd();
 
    // bottom circle  (rotate back)
    glRotated(90,1,0,0);
    i = 0;
    glBegin(GL_TRIANGLES);
    for (size_t k=ang0;k<ang1;k+=DEF_D)
    {
      glVertex3d(0,0,0);
      glVertex3d(raggio*cs[i],0,raggio*ss[i]);
      glVertex3d(raggio*cs[i+1],0,raggio*ss[i+1]);
      i++;
    }
    glEnd();
}

void render3DArrow(vet3 p1, vet3 p2, double raggio, int subdivisions)
{
  glPushMatrix();
  double v;
  mat4x4 m = MRCylinderMatrix(&v, p1, p2, MRVectorZero);
  glTranslated( p1.x, p1.y, p1.z);
  glMultMatrixd (m.m);
  disegnaCilindro(raggio, v*0.65, subdivisions);
  glTranslatef( 0,0,(float) v*0.65f );
  disegnaCono(raggio*2.0, v*0.35, subdivisions);
  glPopMatrix();
}

bool FromWinToCoord(GLdouble winx, GLdouble winy, GLdouble winz,
                const GLfloat modelMatrix[16],
                const GLfloat projMatrix[16],
                const GLint viewport[4],
                GLdouble *objx, GLdouble *objy, GLdouble *objz)
{
    bool result;
    vet4 in = MRVector4Make(winx, winy, winz, 1.0);
    mat4x4 modelM = MRMatrixMakeFromArray(modelMatrix);
    mat4x4 projM = MRMatrixMakeFromArray(projMatrix);
    mat4x4 finalM = MRMatrixInvert(MRMatrixMultiply(modelM, projM), &result);
    if(!result)
    {
       qDebug() << "not inverted\n";
       return false;
    }

    // Map x and y from window coordinates
    vet4 out = MRVector4MapFromViewport(in, viewport);

    // Map to range -1 to 1
    in = MRVector4AddScalar(MRVector4MultiplyScalar(out, 2), -1);

    out = MRMatrixMultiplyVector(finalM, in);
    if (abs(out.v[3]) < 1e-10)
        return false;
    in = MRVector4DivideScalar(out, out.v[3]);
    *objx = in.v[0];
    *objy = in.v[1];
    *objz = in.v[2];
    return true;
}
bool FromCoordToWin(GLdouble objx, GLdouble objy, GLdouble objz,
              const GLfloat modelMatrix[16],
              const GLfloat projMatrix[16],
              const GLint viewport[4],
              GLdouble *winx, GLdouble *winy, GLdouble *winz)
{
    vet4 in = MRVector4Make(objx, objy, objz, 1.0);
    mat4x4 modelM = MRMatrixMakeFromArray(modelMatrix);
    mat4x4 projM = MRMatrixMakeFromArray(projMatrix);
    vet4 out = MRMatrixMultiplyVector(modelM, in);
    in = MRMatrixMultiplyVector(projM, out);

    if (abs(in.v[3]) < 1e-10)
        return false;
    out = MRVector4DivideScalar(in, in.v[3]);
    // Map x, y and z to range 0-1
    in = MRVector4AddScalar(MRVector4MultiplyScalar(out, 0.5), 0.5);

    // Map x,y to viewport
    out = MRVector4MapToViewport(in, viewport);

    *winx=out.v[0];
    *winy=out.v[1];
    *winz=out.v[2];
    return true;
}

vet3 DrawDistLines(int n, vet3 points[])
{
   float c[4];
   FromQColor(Properties::DistColor, c);
   glColor4fv(c);
   glBegin(GL_LINE_STRIP);
   for(int k=0; k<n; k++)
       glVertex3dv(points[k].v);
   glEnd();
   return MRVectorMedium(points[0], points[1]);
}

vet3 DrawDistDihedral(vet3 p[])
{
   float c[4];
   FromQColor(Properties::DistColor, c);

   glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ) ;
   glColor4fv(c);

   glBegin(GL_POLYGON);
   vet3 P0 = MRVectorMedium(p[0], p[1]);
   glVertex3dv(P0.v);
   glVertex3dv(p[1].v);
   vet3 P = MRVectorMedium(p[1], p[2]);
   glVertex3dv(P.v);
   vet3 v1 = MRVectorSub(P, p[1]);
   vet3 P1 = MRVectorAdd(P0, v1);
   glVertex3dv(P1.v);
   glEnd();

   glBegin(GL_POLYGON);
   P0 = MRVectorMedium(p[1], p[2]);
   glVertex3dv(P0.v);
   glVertex3dv(p[2].v);
   P = MRVectorMedium(p[2], p[3]);
   glVertex3dv(P.v);
   vet3 v2 = MRVectorSub(P, p[2]);
   P = MRVectorAdd(P0, v2);
   glVertex3dv(P.v);
   glEnd();

   return MRVectorMedium(P1, P);
}

vet3 DrawDistAngles(int n, vet3 p[], double ang)
{
   glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ) ;
// Angles
   double rad = MRVectorDistance(p[0], p[1]);
   for(int k=1; k<n-1; k++)
   {
       double d =  MRVectorDistance(p[k], p[k+1]);
       if(d < rad)
          rad = d;
   }
   rad *= 0.5;
   vet3 u = MRVectorSub(p[0], p[1]);
   vet3 v = MRVectorSub(p[2], p[1]);
   vet3 vnorm1 = MRVectorCrossProduct(u, v);
   vet3 w = MRVectorCrossProduct(vnorm1, u);
   u = MRVectorNormalize(u);
   w = MRVectorNormalize(w);
   float c[4];
   FromQColor(Properties::DistColor, c);

   glBegin(GL_LINE_STRIP);
   glColor4fv(c);
   vet3 points;
   for(int deg=1; deg <=(int) ang; deg++)
   {
       double coseno =  cos(DEGTORAD(deg));
       double seno   =  sin(DEGTORAD(deg));
       vet3 tmp = MRVectorAdd(
                     MRVectorMultiplyScalar(u, coseno),
                     MRVectorMultiplyScalar(w, seno));
       tmp = MRVectorMultiplyScalar(tmp, rad);
       points = MRVectorAdd(p[1], tmp);
       glVertex3dv(points.v);
   }
   glEnd();
   int degc = static_cast<int>(0.5 * ang);
   double coseno =  cos(DEGTORAD(degc));
   double seno   =  sin(DEGTORAD(degc));
   vet3 tmp = MRVectorAdd(
                     MRVectorMultiplyScalar(u, coseno),
                     MRVectorMultiplyScalar(w, seno));
   tmp = MRVectorMultiplyScalar(tmp, rad);
   return MRVectorAdd(p[1], tmp);
}

void Sfera(bool solid, double radius, int div)
{
    int val[2];
    glGetIntegerv(GL_POLYGON_MODE, val);
    if(solid)
       glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    else
       glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
    disegnaSfera(radius, div);
}

QColor ToQColor(float c[], int n)
{
    QColor color;
    color.setRedF(static_cast<qreal>(c[0]));
    color.setGreenF(static_cast<qreal>(c[1]));
    color.setBlueF(static_cast<qreal>(c[2]));
    if(n > 3)
        color.setAlphaF(static_cast<qreal>(c[3]));
    return color;
}

QColor ToQColor(float r, float g, float b, float alpha)
{
    QColor color;
    color.setRedF(static_cast<qreal>(r));
    color.setGreenF(static_cast<qreal>(g));
    color.setBlueF(static_cast<qreal>(b));
    color.setAlphaF(static_cast<qreal>(alpha));
    return color;

}

void FromQColor(QColor c, float color[], int n)
{
    color[0] = c.redF();
    color[1] = c.greenF();
    color[2] = c.blueF();
    if(n > 3)
        color[3] = c.alphaF();
}

void renderSphere(float x, float y, float z, double raggio, int subdivisions,
                  bool iftexture)
{
      glPushMatrix();
      glTranslatef( x,y,z );
      disegnaSfera(raggio, subdivisions, iftexture);
      glPopMatrix();
}

void DrawCylinder(double v, double raggio, int subdivisions,
                      int d1, int d2, int ordleg)
{
  double rc=raggio*0.7;

  float delt = 1.1f*(float)raggio;

  switch (ordleg)
  {
  default:
  case 1:
  case 4:
       disegnaCilindro(raggio, v, subdivisions);
       break;
  case 2:
  case 5:
       glTranslatef(0,-delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(0,2*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       break;
  case 3:
       glTranslatef(-delt, 0,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(delt,-1.73f*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(delt,1.73f*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       break;
  }
  if(!d1)
     disegnaSfera(raggio, subdivisions);
  glTranslatef( 0,0,(float) v );
  if(!d2)
     disegnaSfera(raggio, subdivisions);
}

void SetLightSystem(QOpenGLFunctions *context, float colore[], float alpha, int blend)
{
     float color1[4];
     context->glEnable (GL_LIGHTING);
     context->glEnable (GL_NORMALIZE);
     glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
     context->glEnable(GL_COLOR_MATERIAL);
     for(int i=0; i<3; i++)
         color1[i] = colore[i];
     color1[3] = alpha;
     glColor4fv(color1);
     context->glLineWidth(1.) ;
     glShadeModel(GL_SMOOTH);
     glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
     context->glEnable (GL_BLEND);
     switch (blend)
     {
     default:
     case 0:
          context->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
          break;
     case 1:
//          context->glBlendFunc (GL_SRC_ALPHA, GL_SRC_COLOR);
          context->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
          break;
     }
}

void SetLightForSelected(QOpenGLFunctions *context, int blend, float mat_e[], float alpha)
{
//  float mat_emission[4] = {1.0f, 0.8f, 1.0f,   1.0f};
  float mat_emission[4] = {0.9f, 0.7f, 0.9f,   1.0f};
  if(mat_e != nullptr)
     for(int i=0; i<4; i++)
         mat_emission[i] = mat_e[i];

  SetLightSystem(context, mat_emission, alpha, blend);
  context->glDepthMask (GL_FALSE);
}

void ResetLightForSelected(QOpenGLFunctions *context)
{
  context->glDepthMask (GL_TRUE);
  context->glDisable(GL_BLEND);
  context->glDisable(GL_COLOR_MATERIAL);
  context->glEnable(GL_LIGHTING);
  context->glEnable(GL_NORMALIZE);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
}

void DrawSelection(QOpenGLFunctions *context)
{
  context->glDisable (GL_DEPTH_TEST);
  SetLightForSelected(context, 1, nullptr, 0.2f);
  context->glDepthMask (GL_FALSE);
  glPushMatrix();
  glLoadIdentity();
  DrawPolygon(Properties::SelectionRect);
  context->glEnable (GL_DEPTH_TEST);
  context->glDepthMask (GL_TRUE);
  ResetLightForSelected(context);
  glPopMatrix();
}

void DrawPolygon(Vert v[])
{
    glBegin(GL_POLYGON);
    for(int i=0; i<4; i++)
    {
        glColor4fv(v[i].c);
        glVertex3f(v[i].x, v[i].y, v[i].z);
    }
    glEnd();
}

int processHitsm (GLint hits, GLuint buffer[],
                  vector<PickResult> &vresult)
{
   int i;
   unsigned int j;
   GLuint names, *ptr;
   int result = 0;

   ptr = (GLuint *) buffer;
   vresult.clear();
   for (i = 0; i < hits; i++)  //  for each hit
   {
      PickResult r;
      names = *ptr;
      ptr++;
      float z1 =  (float) *ptr/0x7fffffff;
      ptr++;
      float z2 =  (float) *ptr/0x7fffffff;
      r.zm = (z1 + z2) / 2.f;
      ptr++;
      for (j = 0; j < names; j++)      //  for each name
      {
         if(j == 0)
            r.type = *ptr;
         else if(j == 1)
            r.model = *ptr;
         else
            r.object = *ptr;
         ptr++;
      }
      vresult.push_back(r);
      result = 1;
   }
   return result;
}

void pickMatrix(double x, double y, double deltax, double deltay,
                  GLint viewport[4])
{
    if (deltax <= 0 || deltay <= 0) {
        return;
    }

// Translate and scale the picked region to the entire window
    glTranslated((viewport[2] - 2 * (x - viewport[0])) / deltax,
            (viewport[3] - 2 * (y - viewport[1])) / deltay, 0);
    glScaled(viewport[2] / deltax, viewport[3] / deltay, 1.0);
}

float CalcolaInten(vet3 vv)
{
    float m[4][4];
    mat3x3 m2;
    glGetFloatv(GL_MODELVIEW_MATRIX, &m[0][0]);
    for(int i=0; i<3; i++)
        for (int j=0; j<3; j++)
             m2.m[3*i+j] = static_cast<double>(m[j][i]);
    vet3 x2 = MRMatrix3MultiplyVector3(m2, vv);

    float size = (float) MRVectorLength(x2);
    return (max(0.1f, (float) abs(x2.z)/size));
}

float CalcolaInten(float v1, float v2, float v3)
{
    vet3 v = MRVectorMake(v1, v2, v3);
    v = MRVectorNormalize(v);
    return CalcolaInten(v);
}

void GetSizeandInten(Knot *knot)
{
    vet3 x1 = MRVectorMake(knot->vnx, knot->vny, knot->vnz);
    vet3 x2 = MRVectorMake(knot->hnx, knot->hny, knot->hnz);
    if(abs(knot->vsize) < EPS )
        knot->vsize = sqrt( knot->vnx*knot->vnx +
                            knot->vny*knot->vny +
                            knot->vnz*knot->vnz );
    knot->vinten = max(0.1f, abs(knot->vnz)/knot->vsize);
    knot->vinten = CalcolaInten(x1);

    if( abs(knot->hsize) < EPS )
        knot->hsize = sqrt( knot->vnx*knot->hnx +
                            knot->vny*knot->hny +
                            knot->vnz*knot->hnz );
    knot->hinten = max(0.1f, abs(knot->hnz)/knot->hsize);
    knot->hinten = CalcolaInten(x2);
}

void drawCylinder( vet3 p1, vet3 p2,
                   float raggio, float color1[])
{
  float colore[4];
  float color2[] = {1.0f, 0.85f, 1.0f, 0.5f};
  double Raggio = static_cast<double>(raggio);

  if(!color1)
     memcpy(colore, color2, 4*sizeof(float));
  else
     memcpy(colore, color1, 4*sizeof(float));
  vet3 p1p2N = MRVectorSub(p1, p2);
  vet3 firstPerpV = MRVectorFirstPerp(p1p2N);
  vet3 secondPerpV = MRVectorCrossProduct(p1p2N, firstPerpV);
  secondPerpV = MRVectorNormalize(secondPerpV);

  const int ANZ = 32;
  const double FULL = 2.0*M_PI;
  Vert points[ANZ+1];

  for (int i=0; i<ANZ; i++)
  {
    double angle = FULL*((double) i/(double) ANZ);

    points[i].x = (float) (Raggio*(cos(angle)*firstPerpV.x+sin(angle)*secondPerpV.x));
    points[i].y = (float) (Raggio*(cos(angle)*firstPerpV.y+sin(angle)*secondPerpV.y));
    points[i].z = (float) (Raggio*(cos(angle)*firstPerpV.z+sin(angle)*secondPerpV.z));
    memcpy(points[i].c, colore,  4*sizeof(float));
    if(color1 != nullptr)
    {
       float intens = 0.9f*CalcolaInten(points[i].x, points[i].y, points[i].z);
       TuneHSV( points[i].c, intens);
    }
  }
    points[ANZ].x = points[0].x;
    points[ANZ].y = points[0].y;
    points[ANZ].z = points[0].z;
    memcpy(points[ANZ].c, points[0].c, 4*sizeof(float));
    glBegin(GL_TRIANGLE_FAN);
    glVertex3dv(p1.v);
    for (int i=0; i<=ANZ; i++)
    {
      glColor4fv(points[i].c);
      glVertex3f((float)p1.x+points[i].x,
                 (float)p1.y+points[i].y,
                 (float)p1.z+points[i].z);
    }
    glEnd();

    glBegin(GL_TRIANGLE_FAN);
    glVertex3dv(p2.v);
    for (int i=0; i<=ANZ; i++)
    {
      glColor4fv(points[i].c);
      glVertex3f((float)p2.x+points[i].x,
                 (float)p2.y+points[i].y,
                 (float)p2.z+points[i].z);
    }
    glEnd();

    glBegin(GL_QUAD_STRIP);
    for (int i=0; i<=ANZ; i++) {
      glColor4fv(points[i].c);
      glVertex3f((float)p1.x+points[i].x,
                 (float)p1.y+points[i].y,
                 (float)p1.z+points[i].z);
      glVertex3f((float)p2.x+points[i].x,
                 (float)p2.y+points[i].y,
                 (float)p2.z+points[i].z);
    }
    glEnd();
  }


void renderCylinder(QOpenGLFunctions *f, vet3 p1, vet3 p2,
                    double raggio, float color1[],
                    int subdivisions, int d1, int d2)
{
    f->glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    f->glEnable(GL_COLOR_MATERIAL);
    f->glEnable (GL_BLEND);
    f->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    drawCylinder(p1, p2, (float) raggio, color1);

    glPushMatrix();

    f->glDisable(GL_COLOR_MATERIAL);
    f->glEnable(GL_LIGHTING);
    f->glEnable(GL_NORMALIZE);

    glTranslated( p1.x, p1.y, p1.z);
    if(!d1)
       disegnaSfera(raggio, subdivisions);
    glPopMatrix();
    glPushMatrix();
    glTranslated( p2.z, p2.z, p2.z);
    if(!d2)
       disegnaSfera(raggio, subdivisions);
    glPopMatrix();
}
void drawCylinderWithCaps(double radius, double h, int slices,
      bool ifCaps=true)
{
    int  DEF_D=5;
    if(slices > 180)
       DEF_D = (int) max(1, (int)(900.f/(float)slices));

    disegnaCilindro(radius, h, slices);
    if(!ifCaps)
       return;

    // top and bottom circles
    for (int j=0;j<=1; j++)
    {
      double i = (double)j*h;
      glBegin(GL_TRIANGLE_FAN);
      for (int k=0;k<=360;k+=DEF_D)
      {
        glNormal3d(radius*Cos(k),radius*Sin(k),i);
        glVertex3d(radius*Cos(k),radius*Sin(k),i);
      }
      glEnd();
    }
}

void renderCircs(float x, float y, float z, double radius, int subdivisions,
               float scala[], float mat[4][4])
{
      float he = 0.003f;
      double raggio = radius;

      if((radius / he) > 5)
          raggio = radius + he/2;
      else
          raggio = radius;

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], scala[1], 1);
      glRotatef (90.0, 0.0, 0.0, 1.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(1, scala[1], scala[2]);
      glRotatef (90.0, 0.0, 1.0, 0.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], 1, scala[2]);
      glRotatef (90.0, 1.0, 0.0, 0.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();
}

void renderEll(float x, float y, float z, double raggio, int subdivisions,
               float scala[], float mat[4][4])
{
      glPushMatrix();
      glTranslatef( x,y,z );
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], scala[1], scala[2]);
      disegnaSfera(raggio, subdivisions);
      glScalef(Properties::Scale, Properties::Scale, Properties::Scale);
      glPopMatrix();
}
