/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "highlighter.h"
//#include <QRegularExpression>

const QString startp = "(\\s|^)";
const QString endp   = "(\\s|$)";

Highlighter::Highlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
}

//Qt6 not compatible
//void Highlighter::highlightBlock(const QString &text)
//{
//    foreach (const HighlightingRule &rule, highlightingRules) {
//        QRegExp expression(rule.pattern);
//        int index = expression.indexIn(text);
//        while (index >= 0) {
//            int length = expression.matchedLength();
//            setFormat(index, length, rule.format);
//            index = expression.indexIn(text, index + length);
//        }
//    }
//    return;
//}

//Qt6/Qt5 compatible
void Highlighter::highlightBlock(const QString &text)
{
    foreach (const HighlightingRule &rule, highlightingRules) {
        QRegularExpression expression(rule.pattern);
        auto matches = expression.globalMatch(text);
        while (matches.hasNext()) {
            auto match = matches.next();
            setFormat(match.capturedStart(), match.capturedLength(), rule.format);
        }
    }
    return;
}

void Highlighter::setCommentRule(QColor color)
{
    HighlightingRule rule;
    QTextCharFormat singleLineCommentFormat;

    singleLineCommentFormat.setForeground(color);
    //rule.pattern = QRegExp("(!|//|>)[^\n]*");
    rule.pattern = QRegularExpression("(!|//|>)[^\n]*");
    rule.format = singleLineCommentFormat;
    highlightingRules.append(rule);
}

//Qt6 not compatible
//void Highlighter::setMixedRules(QStringList keywordPatterns, QColor color, QString perc)
//{
//    HighlightingRule rule;
//    QTextCharFormat commandFormat;
//    const int MINLEN = 4; //minimum length of substring

//    commandFormat.setFontWeight(QFont::Bold);
//    commandFormat.setForeground(color);
//    foreach (const QString &pattern, keywordPatterns) {
//        //extract all substrings contained in pattern
//        int start = pattern.length() <= MINLEN ? pattern.length() : MINLEN;
//        for (int i = start; i <= pattern.length(); i++) {
//            if(perc.length() > 0)
//               //rule.pattern = QRegExp(startp+perc+pattern.left(i)+endp, Qt::CaseInsensitive);
//                rule.pattern = QRegularExpression(startp+perc+pattern.left(i)+endp, Qt::CaseInsensitive);
//            else
//                //rule.pattern = QRegExp(startp+pattern.left(i)+endp, Qt::CaseInsensitive);
//                rule.pattern = QRegularExpression(startp+pattern.left(i)+endp, Qt::CaseInsensitive);
//            rule.format = commandFormat;
//            highlightingRules.append(rule);
//        }
//    }
//}

//Qt6/Qt5 compatible
void Highlighter::setMixedRules(QStringList keywordPatterns, QColor color, QString perc)
{
    HighlightingRule rule;
    QTextCharFormat commandFormat;
    const int MINLEN = 4; //minimum length of substring

    commandFormat.setFontWeight(QFont::Bold);
    commandFormat.setForeground(color);
    foreach (const QString &pattern, keywordPatterns) {
        //extract all substrings contained in pattern
        int start = pattern.length() <= MINLEN ? pattern.length() : MINLEN;
        for (int i = start; i <= pattern.length(); i++) {
            QRegularExpression regExp;
            if(perc.length() > 0)
                regExp.setPattern(QString(startp)+perc+QString(pattern.left(i))+QString(endp));
            else
                regExp.setPattern(QString(startp)+pattern.left(i)+QString(endp));
            regExp.setPatternOptions(QRegularExpression::CaseInsensitiveOption);
            rule.pattern = regExp;
            rule.format = commandFormat;
            highlightingRules.append(rule);
        }
    }
}


void Highlighter::setCommandsRule(QStringList keywordPatterns, QColor color)
{
    setMixedRules(keywordPatterns, color, "%");
}

void Highlighter::setDirectivesRule(QStringList keywordPatterns, QColor color)
{
    setMixedRules(keywordPatterns, color, "");
}
