module specif_interop
   
   implicit none

   contains

   subroutine open_summary(filename, length) bind(C)
   use iso_c_binding, only: c_char, c_int
   use fileutil
   use strutil
   implicit none
   character(c_char), intent(in)      :: filename(*)
   integer(c_int), intent(in), value  :: length
   character(len=:), allocatable      :: filenam
   type(file_handle)                  :: filew
!    
   filenam = toFortranString(filename,length)
!
   call filew%fopen(filenam,ios='w')
   if (.not.filew%good()) return
   call GenInfoV2(filew%handle())
   call filew%fclose()
!   
   end subroutine open_summary

! -----------------------------------------------------------------------------------------------------

   function CellVolumeForC(cell) result(volume) bind(C,name="CellVolumeForC")
   use iso_c_binding
   use unit_cell
   real(c_double), dimension(6), intent(in) :: cell
   real(c_double)                           :: volume

   volume = cell_volume(real(cell))

   end function CellVolumeForC

! -----------------------------------------------------------------------------------------------------

   subroutine GetCellCode(spg,lenspg,cell,code) bind(C,name="GetCellCode")
   use iso_c_binding
   use strutil
   use spginfom
   character(c_char), dimension(*), intent(in) :: spg
   integer(c_int), value, intent(in)           :: lenspg
   real(c_double), dimension(6), intent(in)    :: cell
   integer(c_int), dimension(6)                :: code
   character(len=:), allocatable               :: spgf
   type(spaceg_type)                           :: spgr
   integer                                     :: ier
!
   spgf = toFortranString(spg,lenspg)
   spgr = init_spaceg_type(spgf)
   if (spgr%undef()) then
       code(:) = 0
       return
   endif
   call spg_get_cell_code(spgr,real(cell),code,ier)
   if (ier /= 0 .and. spgr%csys_code == CS_Trigonal) then  ! force hexagonal cell
       code(2) = -1          ! b=a
       code(3) = 0
       code(4:5) = -90       ! al=bet=90
       code(6) = -120        ! gam=120
   endif
!
   end subroutine GetCellCode

! -----------------------------------------------------------------------------------------------------

   function ValidateFormula(formula,lenformula) result(val) bind(C,name="ValidateFormula")
   use iso_c_binding
   use strutil
   use elements
   character(c_char), dimension(*), intent(in)   :: formula
   integer(c_int), value, intent(in)             :: lenformula
   integer(c_int)                                :: val
   character(len=:), allocatable                 :: formulaf
   type(element_type), dimension(:), allocatable :: elem
   integer                                       :: ier

   formulaf = toFortranString(formula,lenformula)
   call read_chemformula(formulaf,ier,elem=elem)
   if (ier > 0) then
       val = 0
   else
       val = size(elem)
   endif

   end function ValidateFormula

! -----------------------------------------------------------------------------------------------------

   function ValidateFormulaInfo(formula,lenformula,volume,info) result(val) bind(C,name="ValidateFormulaInfo")
   use iso_c_binding
   use strutil
   use elements
   use atom_type_util
   character(c_char), dimension(*), intent(in)   :: formula
   integer(c_int), value, intent(in)             :: lenformula
   real(c_double), value, intent(in)             :: volume
   real(c_double), dimension(4)                  :: info
   integer(c_int)                                :: val
   character(len=:), allocatable                 :: formulaf
   type(element_type), dimension(:), allocatable :: elem
   integer                                       :: ier

   formulaf = toFortranString(formula,lenformula)
   call read_chemformula(formulaf,ier,elem=elem)
   if (ier > 0) then
       val = 0
       info = 0
   else
       val = size(elem)
       info(1) = content_volume(elem%ptab,elem%nw)
       info(2) = volume_per_atom(elem,real(volume))
       info(4) = molecular_weight(elem%ptab,elem%nw)
       info(3) = density_value(real(info(4)),real(volume),1)
   endif

   end function ValidateFormulaInfo

! -----------------------------------------------------------------------------------------------------

   subroutine send_event(event) bind(C,name="send_event")
   USE iso_c_binding, only: c_int
   use dialogo, only: ifBrekke
   integer(c_int), intent(in), value :: event

   ifBrekke = event

   end subroutine send_event

! -----------------------------------------------------------------------------------------------------

   subroutine importa_file(filename, length, ftype) bind(C,name="importa_file")
   USE iso_c_binding, only: c_char, c_int
   USE General, only: structurename
   USE ilmviewmod
   USE fileutil
   USE strutil
   USE datamod, only: abilita_tasti2
!qt   USE enable_amb
!
   character(kind=c_char), intent(in) :: filename(*) ! full filename including folder name
   integer(c_int), intent(in), value  :: length
   integer(c_int), intent(in), value  :: ftype
   integer                            :: ier
   character(len=:), allocatable      :: basename
   character(len=:), allocatable      :: filnam
!
   filnam = toFortranString(filename,length)
   basename = file_rem_ext(file_get_name(filnam))
   call ilmview_open(basename)
   call jav_import_structure(filnam,ftype,1,ier)
   call abilita_tasti2()
   if (len_trim(structurename) == 0) structurename = basename
!
   end subroutine importa_file

! -----------------------------------------------------------------------------------------------------

   subroutine importa_file_from_string(cstring, length, ier) bind(C,name="importa_file_from_string")
   use iso_c_binding, only: c_char, c_int
   use strutil
   use variables, only: cryst
   use drawatoms, only: importa0
   use enable_amb
   use datamod, only: abilita_tasti2
   character(kind=c_char), intent(in) :: cstring(*) ! full filename including folder name
   integer(c_int), intent(in), value  :: length
   character(len=:), allocatable      :: fstring
   integer(c_int)                     :: ier
!
   fstring = toFortranString(cstring,length)
!
   call importa0(fstring,cryst(1)%at,cryst(1)%bond,ier,kadd=cryst(1)%natoms()>0)
   if (ier == 0) then
       !call abilita_tasti(PatternAndStructureAction)
       call abilita_tasti2()
   endif
!
   end  subroutine importa_file_from_string

! -----------------------------------------------------------------------------------------------------

   subroutine set_profile_enabled(enabled) bind(C,name="set_profile_enabled")
   use iso_c_binding, only: c_int
   use PatternRef, only: kprofile
   use view, only: vedinew
   integer(c_int), value :: enabled
!
   kprofile = enabled
   call vedinew(rescale=0)
!
   end subroutine set_profile_enabled

! -----------------------------------------------------------------------------------------------------

   integer(c_int) function is_extraction_active() bind(C,name="is_extraction_active")
   use iso_c_binding, only: c_int
   use General, only: is_extraction
!
   is_extraction_active = is_extraction
!
   end function is_extraction_active

end module specif_interop

module background_extra_interop
   implicit none

   private 

   contains

   subroutine get_back_ncoeff(ncoeff) bind(C,name="get_back_ncoeff")
   use iso_c_binding
   integer(c_int), dimension(*) :: ncoeff

   common /step/ thet1(10),thet2(10),ncoun1(10),ncoun2(10)        &
                 ,nref1(10),nref2(10),ymx(10),nnt                 &
                 ,thet11(10),thet22(10),ncoun11(10),ncoun22(10)   &
                 ,nref11(10),nref22(10),sscl(10),scala(10)
   integer :: ncoun1,ncoun2,nref1,nref2,nnt,ncoun11,ncoun22,nref11,nref22
   common /backp2/ npol(10)
   integer :: npol
   real    :: thet1, thet2,ymx,thet11,thet22,sscl,scala
   integer :: i

   do i=2,nnt
      ncoeff(i-1) = npol(i)
   enddo

   end subroutine get_back_ncoeff

! -----------------------------------------------------------------------------------------------------

   subroutine change_polynomial(interval,ncoeff) bind(C,name="change_polynomial")
   use iso_c_binding
   use PatternRef, only: jjj
   use view
   integer(c_int), value :: interval, ncoeff
   common /backp2/ npol(10)
   integer :: npol
   external CalcoBacExt

   jjj = interval + 1
   npol(jjj) = ncoeff
   call CalcoBacExt(1)
   call vedinew()

   end subroutine change_polynomial

end module background_extra_interop

module interop_range
   implicit none

   private
   
   contains

   subroutine set_range(kaction, tminvalue, tmaxvalue) bind(C,name="set_range")
   use iso_c_binding
   use variables, only: dataset
   use arrayutil
   use patternref, only: thminp,thmaxp,minrange,maxrange,modify_range,thmin,thmax
   use peak_mod
   use conteggi, only: theta_int,npunti
   use view
   use enable_amb
   use datamod
   use general, only: nrefl
   integer(c_int), value, intent(in)       :: kaction
   real(c_double)                          :: tminvalue
   real(c_double)                          :: tmaxvalue
   real, allocatable, dimension(:,:), save :: thetac
   integer, save                           :: npuntisav
   logical, save                           :: modif
   integer                                 :: iflagref
   common /limiti/ nlini, nlfin
   integer :: nlini, nlfin
!
   select case(kaction)
     case (0)     ! Open dialog
       call abilita_tasti('window_pattern')
       call toolbar_set_state('continue'//char(0),0)
!
       tminvalue = theta_int(1, 7)
       tmaxvalue = theta_int(npunti, 7)
!
!      Save counts
       allocate(thetac(npunti,7))
       thetac(:,:) = theta_int(:,:)
       npuntisav = npunti
       modif = .false.

     case (1,2)                    ! 1=Apply, 2=OK
       nlini = clocate(theta_int(:,7),real(tminvalue))
       nlfin = clocate(theta_int(:,7),real(tmaxvalue))
       if (nlini /= 1 .or. nlfin /= npunti) then
           thminp = theta_int(nlini, 7)
           thmaxp = theta_int(nlfin, 7)
           call dataset(1)%resize(thminp,thmaxp)
!
!          resize peaks
           if (numpeaks(pkind) > 0) then
               call peak_select(pkind,thminp,thmaxp,DELETE_PEAK,outside=.true.)
               call peak_delete_selected(pkind)
           endif
!
           call modifica_range()
           modif = .true.
       endif

       call vedinew(pinioss=1,pfinoss=npunti)

       if (kaction == 2) then
           deallocate(thetac)
           call abilita_tasti('pattern')
           call wmenu_set_state('Range'//char(0),1)
           call toolbar_set_state('continue'//char(0),1)
           if (modif) then  ! update modify_range for input file 
               modify_range = .true.
               minrange = thmin
               maxrange = thmax
           endif
       endif

     case (3)                    ! Cancel
!
!      Ripristina i conteggi in caso di modifica
       if (npunti /= npuntisav) then
           call AllocaConteggi(2)
           npunti = npuntisav
           call AllocaConteggi(1)
!!!!FIX PROBLEM: update dataset or Cancel doesn't work properly
!!!! use %update_data or %save/%restore
           theta_int(:,:) = thetac(:,:)
           thmin = theta_int(1,7)
           thmax = theta_int(npunti,7)
           if (nrefl > 0) call gener(dataset(1)%wave,iflagref,.false.)
       endif
!
!      Disegna anche per forzare la scomparsa delle barre
       !call vedinew(pinioss=1,pfinoss=npunti)
!
       deallocate(thetac)
       call abilita_tasti('pattern')
       call wmenu_set_state('Range'//char(0),1)
       call toolbar_set_state('continue'//char(0),1)
       modif = .false.

   end select
!
   end subroutine set_range

end module interop_range

module interop_smooth
   use ssmoothing
   use iso_c_binding

   implicit none

   private

   type, bind(C) :: c_smooth_condition_type
     integer(c_int) :: method      = SAVGOL
     integer(c_int) :: npoints_sg  = 25
     integer(c_int) :: npoints_ave = 12
     integer(c_int) :: pol_order   = 5
   end type c_smooth_condition_type
  
   contains

   elemental function cast_to_c(f_type) result(c_type)
   type(smooth_condition_type), intent(in) :: f_type
   type(c_smooth_condition_type)           :: c_type
!
   c_type%method = int(f_type%method,c_int)
   c_type%npoints_sg = int(f_type%npoints_sg,c_int)
   c_type%npoints_ave = int(f_type%npoints_ave,c_int)
   c_type%pol_order = int(f_type%pol_order,c_int)
!
   end function cast_to_c

! -----------------------------------------------------------------------------------------------------

   elemental function cast_to_f(c_type) result(f_type)
   use background, only: back_condition_type
   type(c_smooth_condition_type), intent(in) :: c_type
   type(smooth_condition_type)               :: f_type
!
   f_type%method = int(c_type%method)
   f_type%npoints_sg = int(c_type%npoints_sg)
   f_type%npoints_ave = int(c_type%npoints_ave)
   f_type%pol_order = int(c_type%pol_order)
!
   end function cast_to_f

! -----------------------------------------------------------------------------------------------------

   subroutine set_smooth(kaction,scond) bind(C,name="set_smooth")
   use variables, only: dataset
   use view
   use plotstyle
   use conteggi, only: theta_int
   integer(c_int), value, intent(in) :: kaction
   type(c_smooth_condition_type)     :: scond
!
   select case(kaction)
     case (0)       ! Open Dialog
       scond = cast_to_c(dataset(1)%scond)
       call dataset(1)%smooth_calculate()
       style(STYLE_SMOOTH)%vis = 1
       call vedinew(8,lsetstyle=.false.)

     case (1,2)     ! Apply change: 1 = points, 2 = polynomial order
       dataset(1)%scond = cast_to_f(scond)
       call dataset(1)%smooth_calculate()
       call vedinew(8,lsetstyle=.false.)

     case (3)     ! Cancel 
       dataset(1)%scond = cast_to_f(scond)
       call dataset(1)%smooth_calculate()
       style(STYLE_SMOOTH)%vis = 0
       call vedinew(8,lsetstyle=.false.)

     case (4)     ! Ok
       call dataset(1)%smooth_apply()
       theta_int(:,2) = dataset(1)%y
       style(STYLE_SMOOTH)%vis = 0
       call vedinew(8,lsetstyle=.false.)

   end select
!
   end subroutine set_smooth

end module interop_smooth

module fourier_interop
   use iso_c_binding

   implicit none

   type, bind(C) :: c_fourier_conditions
     integer(c_int) :: fourtype
     integer(c_int) :: npeaks
     integer(c_int) :: weitype
     integer(c_int) :: refSelection
     real(c_float)  :: refPerc
     integer(c_int) :: showFourier
   end type c_fourier_conditions

   contains

   subroutine prefourier(natsC,nadd) bind(C,name="prefourier")
   USE General, only: celem, spacegr, neutro
   USE elements
   USE variables, only: atm
   USE atom_type_util
!
   integer(c_int), intent(out) :: natsC
   integer(c_int), intent(out) :: nadd
   real                        :: anat0, anat1
!
   natsC = numatoms(atm)
!
!--computes number of peaks to add
!
   anat1 = natom_cell(atm,spacegr%nsymop,neutro==0)
   anat0 = nasym_unit(celem,spacegr%nsymop,neutro==0)
   nadd = nint(( anat0 - anat1 ) / spacegr%nsymop)
   if (nadd < 5) then
       nadd = 5
   endif
!
   end subroutine prefourier

! -----------------------------------------------------------------------------------------------------

   subroutine startfourier(fcond,ier) bind(C,name="startfourier")
   USE ilmviewmod
   USE fouriermap
!
   type(c_fourier_conditions), intent(in), value :: fcond
   integer(c_int), intent(out)                   :: ier
   !integer kFinalState(6)
   integer :: showfourier, icalfo,npikfo,iweifo,iselect_ref
   real :: preff
!
   call InitFour()
!
!  icalfo = 1 --> Fourier coeff.= ( Fo )
!  icalfo = 2 --> Fourier coeff.= (2 Fo - Fc)
!  icalfo = 3 --> Fourier coeff.= (Fo - Fc)
!
   icalfo = fcond%fourtype
!
!  npikfo  --> num. di picchi da aggiungere
!
   npikfo = fcond%npeaks
!
!  iweifo = 0 --> wwt = 1.
!  iweifo = 1 --> wwt = peso di tipo Sim
!
   iweifo = fcond%weitype
!
!  iselect_ref = 0 --> ordina in base al 2-theta
!  iselect_ref = 1 --> ordina in base all' F osservato
!  iselect_ref = 2 --> ordina in base al peso
!
   iselect_ref = fcond%refSelection
!
!  preff --> percentuale di riflessi usata
!
   preff = fcond%refPerc
!
   showfourier = fcond%showfourier
!
   call save_action(FOURIER,0,1)
   call MakeFour(icalfo,npikfo,iweifo,iselect_ref,preff,showfourier,ier)
   if (ier == 0) call save_action(FOURIER,1,1)
!
   end subroutine startfourier

end module fourier_interop

module hkl_interop
   use iso_c_binding

   implicit none

   type, bind(C) :: c_hkl_info
      integer(c_int), dimension(3) :: hkl         ! indici hkl
      integer(c_int)               :: m = 1       ! molteplicità del riflesso
      real(c_float)                :: tthd        ! 2-theta del riflesso in gradi 
      real(c_float)                :: d           ! d-value
      real(c_float)                :: fo = 0.0    ! fattori di struttura osservati 
      real(c_float)                :: fc = 0.0    ! fattori di struttura calcolati dal modello 
      integer(c_int)               :: ph = 0      ! fasi calcolate dal modello
      integer(c_int)               :: idph        ! id of crystal phase
   end type c_hkl_info

   contains

   elemental function cast_to_hkl_info(ref) result(info)
   use reflection_type_util, only: reflection_type
   type(reflection_type), intent(in) :: ref
   type(c_hkl_info)                  :: info
!
   info%hkl  = ref%hkl      
   info%m    = ref%m   
   info%tthd = ref%tthd(1)
   info%d    = ref%slaq
   info%fo   = ref%fo  
   info%fc   = ref%fc  
   info%ph   = ref%ph  
!
   end function cast_to_hkl_info

end module hkl_interop

module info_interop
   use iso_c_binding

   implicit none

   contains

   subroutine open_hkl_list() bind(C,name="open_hkl_list")
   use view
   use reflection_type_util, only: reflection_type
   use hkl_interop
   integer, dimension(9)                            :: vis
   type(reflection_type), dimension(:), allocatable :: ref
   type(c_hkl_info), dimension(:), allocatable      :: hkl_info
   integer                                          :: ier
   integer, dimension(:), allocatable               :: id_phase
   interface
      subroutine openHklList(hkl_info,nref,vis) bind(C,name="openHklList")
      use iso_c_binding
      use hkl_interop
      type(c_hkl_info), dimension(*)    :: hkl_info
      integer(c_int), intent(in), value :: nref
      integer(c_int), dimension(*)      :: vis
      end subroutine openHklList
   end interface

   call set_reflection_table(ref,vis,1,ier,id_phase)
   if (ier == 0) then
       hkl_info = cast_to_hkl_info(ref)
       hkl_info(:)%idph = id_phase(:)
       call openHklList(hkl_info, size(hkl_info), vis)
   endif

   end subroutine open_hkl_list

! -----------------------------------------------------------------------------------------------------

   subroutine save_hklinfo(filnam, lname, ncol, vet, ftype) bind(C, name="save_hklinfo")
   USE reflection_type_util
   USE fileutil
   USE view
   USE importexport
   USE strutil
   implicit none
   character(c_char), dimension(*), intent(in)      :: filnam
   integer(c_int), intent(in), value                :: lname
   integer(c_int), intent(in), value                :: ncol, ftype
   integer(c_int), dimension(ncol), intent(in)      :: vet
   integer                                          :: ier
   integer, dimension(9)                            :: vetr
   type(reflection_type), dimension(:), allocatable :: table
   type(file_handle)                                :: fref
   character(len=:), allocatable                    :: filename
!
! ftype = 1   --> Fo Fc
!       = 2   --> Fo^2 Fc^2
!
   filename = toFortranString(filnam,lname)
   call fref%fopen(filename,'w')
   if (fref%good()) then
       call set_reflection_table(table,vetr,ftype,ier,idph)   ! ricrea la tabella
       call write_reflection_table(table,vet,fref%handle(),ftype)       ! stampa la tabella
       call fref%fclose()
   endif
!
   end subroutine save_hklinfo

end module info_interop

module missing__info_interop

   implicit none

   contains

   function is_missing_info() result(is_miss) bind(C,name="is_missing_info")
   use iso_c_binding
   use arrayutil
   use variables, only: cryst
   integer(c_int) :: is_miss
!
   is_miss = logical_to_int(.not.cryst(1)%has_symmetry() .or. .not.cryst(1)%spg%defined)
!
   end function is_missing_info

! -----------------------------------------------------------------------------------------------------

   function is_missing_structure() result(is_miss) bind(C,name="is_missing_structure")
   use iso_c_binding
   use arrayutil
   use variables, only: cryst
   use crystal_phase, only: numphase
   integer(c_int) :: is_miss
!
   is_miss = 1
   if (numphase(cryst) == 0) return
   if (cryst(1)%natoms() == 0) return

   is_miss = 0
!
   end function is_missing_structure

! -----------------------------------------------------------------------------------------------------

   subroutine get_symmetry_info(phaseid, spg_index, c_space, c_cell) bind(C,name="get_symmetry_info")
   use iso_c_binding
   use variables, only: cryst
   use strutil
   use spginfom
   use crystal_phase
   integer(c_int), intent(in), value            :: phaseid
   integer(c_int), intent(out)                  :: spg_index
   character(c_char), dimension(*), intent(out) :: c_space
   real(c_double), dimension(6)                 :: c_cell
   integer                                      :: lens

   spg_index = cryst(phaseid)%spg%get_id()
   if (spg_index < 0) then
       c_space(:1) = c_null_char
   else
       lens = len_trim(cryst(phaseid)%spg%symbol_xhm)
       c_space(:lens+1) = toCString(cryst(phaseid)%spg%symbol_xhm)
   endif
   if (cryst(phaseid)%is_cell()) then
       c_cell = real(cryst(phaseid)%get_cellpar(),c_double)
   else
       c_cell = -1  ! these force empty entries for cell
   endif

   end subroutine get_symmetry_info

! -----------------------------------------------------------------------------------------------------

   subroutine put_cell_info(phaseid, c_cell, coderef) bind(C,name="put_cell_info")
   use iso_c_binding
   use variables, only: cryst
   integer(c_int), intent(in), value :: phaseid
   real(c_double), dimension(*)      :: c_cell
   integer(c_int), intent(in), value :: coderef
!
   call cryst(phaseid)%set_cell(real(c_cell(1:6)))
   call cryst(phaseid)%set_refine_cell(coderef)
!
   end subroutine put_cell_info

! -----------------------------------------------------------------------------------------------------

   subroutine put_symmetry_info(c_space, len_space, c_cell, ier) bind(C,name="put_symmetry_info")
   use iso_c_binding
   use strutil
   use datamod, only: update_cell, gener
   use errormod
   use variables, only: dataset, cryst
   use unit_cell
   use atom_type_util
   character(c_char), intent(in)     :: c_space(*)
   integer(c_int), intent(in), value :: len_space
   real(c_double), dimension(*)      :: c_cell
   integer(c_int), intent(out)       :: ier
   type(error_type)                  :: err
   character(len=:), allocatable     :: space
   logical                           :: spg_updated,cell_updated
   integer                           :: iflagref
   type(cell_type)                   :: old_cell
!
   ier = 0
!
!  Check space group
   space = toFortranString(c_space,len_space)
   call update_spaceg(space,spg_updated,err)
   if (err%signal) go to 10
!
!  Check cell parameters
   old_cell = cryst(1)%cell
   call update_cell(real(c_cell(1:6)),cell_updated)
   if (spg_updated .or. cell_updated) then
       call gener(dataset(1)%wave,iflagref,.false.)
   endif
!
!  Check for coordinates
   if (cryst(1)%natoms() > 0 .and. cell_updated) then
       call coord_in_newcell(cryst(1)%at,old_cell,cryst(1)%cell)
   endif
!
   return

10 if (err%signal) call err%print()
   ier = 1

   end subroutine put_symmetry_info

end module missing__info_interop

module intervals_interop

   implicit none

   contains

   function get_intervals_number() bind(C, name="get_intervals_number")
   use iso_c_binding
   integer(c_int) :: get_intervals_number
   common /step/ thet1(10),thet2(10),ncoun1(10),ncoun2(10)        &
                 ,nref1(10),nref2(10),ymx(10),nnt                 &
                 ,thet11(10),thet22(10),ncoun11(10),ncoun22(10)   &
                 ,nref11(10),nref22(10),sscl(10),scala(10)
   integer :: ncoun1,ncoun2,nref1,nref2,nnt,ncoun11,ncoun22,nref11,nref22
   real    :: thet1, thet2,ymx,thet11,thet22,sscl,scala

   get_intervals_number = nnt - 1

   end function get_intervals_number
  
! -----------------------------------------------------------------------------------------------------

   subroutine get_intervals_limit(tmin, tmax) bind(C, name="get_intervals_limit")
   use iso_c_binding
   real(c_float), dimension(*) :: tmin, tmax
   common /step/ thet1(10),thet2(10),ncoun1(10),ncoun2(10)        &
                 ,nref1(10),nref2(10),ymx(10),nnt                 &
                 ,thet11(10),thet22(10),ncoun11(10),ncoun22(10)   &
                 ,nref11(10),nref22(10),sscl(10),scala(10)
   integer :: ncoun1,ncoun2,nref1,nref2,nnt,ncoun11,ncoun22,nref11,nref22
   real    :: thet1,thet2,ymx,thet11,thet22,sscl,scala
   integer :: i

   do i=1,nnt-1
      tmin(i) = thet1(i)
      tmax(i) = thet2(i)
   enddo

   end subroutine get_intervals_limit

! -----------------------------------------------------------------------------------------------------

   subroutine put_intervals_limit(tmin, tmax) bind(C, name="put_intervals_limit")
   USE Conteggi, only: theta_int, npunti
   USE PatternRef, only: thmax
   USE arrayutil
   USE view, only: vedinew
   use iso_c_binding
   real(c_float), dimension(*) :: tmin, tmax
   common /step/ thet1(10),thet2(10),ncoun1(10),ncoun2(10)        &
                 ,nref1(10),nref2(10),ymx(10),nnt                 &
                 ,thet11(10),thet22(10),ncoun11(10),ncoun22(10)   &
                 ,nref11(10),nref22(10),sscl(10),scala(10)
   integer :: ncoun1,ncoun2,nref1,nref2,nnt,ncoun11,ncoun22,nref11,nref22
   real    :: thet1,thet2,ymx,thet11,thet22,sscl,scala
   integer :: i, isav, isav0, ier
   common/nmmn/ nmm1(10),nmm2(10),nmm11(10),nmm22(10)
   integer :: nmm1, nmm2, nmm11, nmm22

   isav0=0
   isav=nnt-1
   do i=1,nnt-1
      write(0,*)'INT ',i,tmin(i),tmax(i)
!
      nmm1(i) = ncoun1(i)
      nmm2(i) = ncoun2(i)
      nmm11(i) = ncoun11(i)
      nmm22(i) = ncoun22(i)
!
      ncoun1(i) = clocate(theta_int(:,7),tmin(i))
      ncoun2(i) = clocate(theta_int(:,7),tmax(i))
      if(isav0.eq.0.and.tmax(i).ge.thmax)then
        isav=i
        isav0=1
      endif
!
   enddo
!
   ier = 0
   nnt=isav+1
   call changeint(ier)
   call vedinew(2,1,npunti)

   end subroutine put_intervals_limit

end module intervals_interop
