#include <QPainter>
#include <QCoreApplication>
#include <QOpenGLTexture>
#include <QImage>
#include <QMenu>
#include "myglwidget.h"
#include "glutils.h"
#include "properties.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif


void MyGLWidget::ReadSettings(bool fromFile)
{
       if(fromFile)
          Properties::ReadSettings();
       m_bgType  = Properties::BgType;
       m_bg0 = Properties::BgColor1;
       m_bg1 = Properties::BgColor2;
       FromQColor(Properties::BgColor, m_bgColor, 3);
}

void MyGLWidget::initializeGL()
{
    // Set up the rendering context, load shaders and other resources, etc.:

#if defined(JAV_WIDGET)
    m_jav = getJav();
#else
    m_jav = getJav();
#endif
    ReadSettings();
    Properties::Set_AA(param_SUPPORT, (m_maxSamples>0)?1:0);
    m_jav->SyncToggles();

    //Aggiunte da Corrado
    QSurfaceFormat format;
    format.setProfile(QSurfaceFormat::CompatibilityProfile);
    format.setVersion(2, 1); // OpenGL 2.1
    format.setDepthBufferSize(24);
    context()->setFormat(format);

    m_context = QOpenGLContext::currentContext()->functions();
    m_context->initializeOpenGLFunctions();

#if 0
	if(Properties::IsMSEnabled())
    {
       GLint  iMultiSample = 0;
       GLint  iNumSamples = 0;
       m_context->glEnable(GL_MULTISAMPLE);
       m_context->glHint(GL_MULTISAMPLE_FILTER_HINT_NV, GL_NICEST);
       m_context->glGetIntegerv(GL_SAMPLE_BUFFERS, &iMultiSample);
       m_context->glGetIntegerv(GL_SAMPLES, &iNumSamples);
	   QString str("GL_MULTISAMPLE ");
       if (iMultiSample <= 0)
	       str.append("not ");
	   str.append("enabled");
       qStdOut() << str << &endl;
       qStdOut() << "num samples: " << iNumSamples << &endl;
       qStdOut() << "num samples in format: " << format().samples() << &endl;
    }
#endif
    m_context->glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
    QImage t = QImage(":/pixmaps/pixmaps/marble.png").mirrored();
    m_context->glGenTextures(1, &(Properties::iTexture[0]));
    m_context->glBindTexture(GL_TEXTURE_2D, Properties::iTexture[0]);
    m_context->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    m_context->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    m_context->glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, t.width(), t.height(),
                0, GL_RGB, GL_UNSIGNED_BYTE, t.bits());
    in_selection = false;
    activeLegend = -1;
    m_motion_type = NO_MOVE;
    m_cursorZoom = QCursor(QPixmap(":/pixmaps/pixmaps/preview22.png"));
    m_cursorCross = QCursor(QPixmap(":/pixmaps/pixmaps/selecton.png"));
    m_cursorMove = QCursor(QPixmap(":/pixmaps/pixmaps/curs.png"));
    m_cursorWait = QCursor(Qt::WaitCursor);
    nclicked = 0;
    setMouseTracking(true);
    glClearDepth (1.0);
    glMaterialf (GL_FRONT, GL_SHININESS, 100.f);
    glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);

    m_context->glFrontFace (GL_CW);
    m_context->glEnable (GL_LIGHTING);
    SetDefaultLights(m_lightInfo);

    m_context->glEnable (GL_AUTO_NORMAL);
    m_context->glEnable (GL_NORMALIZE);
    m_context->glEnable (GL_DEPTH_TEST);
    m_context->glDepthFunc (GL_LESS);

    for(int i=1; i<=20; i++)
    {
        glNewList (i, GL_COMPILE);
        Sfera (true, 0.03/i, i*i+3);
        glEndList ();
     }

     m_view_quat_diff = MRVector4Identity;
     m_view_quat = MRVector4Identity;
     m_Rotm = MRMatrixIdentity;
     // qDebug() << "\nOpenGL renderer vendor: " << (char *)m_context->glGetString(GL_VENDOR)
     //             << "\nOpenGL renderer string: " << (char *)m_context->glGetString(GL_RENDERER)
     //             << "\nOpenGL renderer version: " << (char *)m_context->glGetString(GL_VERSION);
     // if(Properties::IsMSEnabled() && Properties::IsMSSupported())
     // qDebug() <<  "Multisampling enabled: resolution: " << Properties::MSBuffers() << "x";
}

void MyGLWidget::resizeGL(int w, int h)
{
        // Update projection matrix and other size related settings:
        m_context->glViewport(0, 0, w, h);
        m_r = geometry();
        m_Width = w;
        m_Height = h;
        m_Range = 1;
        Properties::Width = w;
        Properties::Height = h;

        if(h == 0)
        {
            h = 1;
            m_aspectRatio = 1.;
        }
        else
            m_aspectRatio = static_cast<GLfloat>(w) / static_cast<GLfloat>(h);

        if(m_aspectRatio <= 1)
           Properties::npixels = static_cast<float>(h);
        else
           Properties::npixels = static_cast<float>(w);

        ResetProjectionMatrix(1.f);
        ResetModelMatrix(0);
}

void MyGLWidget::SetViewpoint(float factor)
{
        m_Range = factor;
        float sc_factor = 1.f / m_Range;
        if(sc_factor < 1e-6)
           sc_factor = 1;
        Properties::sc_factor = sc_factor;
        double nRange = SCALA * m_Range;

        m_projection.setToIdentity();
        if(Properties::ifOrtho)
        {
           if (m_aspectRatio <= 1)
                m_projection.ortho (-nRange, nRange, -nRange / m_aspectRatio,
                                   nRange / m_aspectRatio,
                                   -10*nRange, 10*nRange);
           else
                m_projection.ortho (-nRange * m_aspectRatio,
                                   nRange * m_aspectRatio, -nRange, nRange,
                                   -10*nRange, 10*nRange);
        }
        else
        {
            QVector3D eye(0.5,0.5,2*nRange), center( 0.0,0.0,0.0), up(0.0,1.0,0.0);
            //m_projection.perspective(60.0f, m_aspectRatio, 0.5f*nRange, 3*nRange);
            m_projection.perspective(60.0f, m_aspectRatio, 0.01f, 100.0f);
            m_projection.lookAt(eye, center, up);
        }
        glMultMatrixf(m_projection.data());
}

void MyGLWidget::ResetProjectionMatrix(float factor)
{
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        SetViewpoint(factor);
}

void MyGLWidget::ResetModelMatrix(int sign)
{
        double subd = 200. * Properties::cylinder * Properties::sc_bonds * Properties::sc_factor;
        vet3 pos = Properties::obj_pos;
        float Scale = Properties::Scale;
        Properties::subdivisions = (Scale > 1.)? ((int) (subd * Scale)) : ((int) subd);
        Properties::subdivisions = max(Properties::subdivisions, 3);

        m_model.setToIdentity();
        if(sign)
            m_model.translate(pos.x+sign*10.f/m_Width, pos.y, -pos.z);
        else
            m_model.translate(pos.x, pos.y, -pos.z);
        m_model.scale(Scale);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glMultMatrixf(m_model.data());
        glMultMatrixd (m_Rotm.m);
}

void MyGLWidget::ResetView(bool azzera, bool redraw)
{
        float factor = m_Range;
        if(azzera)
        {
           m_view_quat_diff = MRVector4Identity;
           m_view_quat = MRVector4Identity;
           m_Rotm = MRMatrixIdentity;
           m_view_quat = MRMatrixToQuat(m_RotB);
           m_Rotm = MRMatrix4MakeFromQuat(m_view_quat);
           factor = 1.f;
        }
        Properties::Scale = 1;
        Properties::obj_pos = MRVectorZero;
        Properties::change_clip_value();
        ResetProjectionMatrix(factor);
        ResetModelMatrix(0);
        if(redraw)
            update();
}

void MyGLWidget::DrawGrid()
{
    float x0 = 0, y0 = 0, z0 = 0;
    float x1 = 1, y1 = 1, z1 = 1;
// le seguenti per una griglia cubica (dimensioni in Angstrom)
    float delta = 2.f;
    int xdivisions = (int)(Properties::Cell[0]/delta);
    int ydivisions = (int)(Properties::Cell[1]/delta);
    int zdivisions = (int)(Properties::Cell[2]/delta);
//  questa per una griglia cristallografica
    xdivisions = ydivisions = zdivisions = Properties::nDivisions;

    float deltax = 1.f/xdivisions;
    float deltay = 1.f/ydivisions;
    float deltaz = 1.f/zdivisions;

    if(Properties::GridPattern > 0)
    {
       glLineStipple(1, (GLushort)Properties::GridPattern);
       glEnable(GL_LINE_STIPPLE);
    }
    else
       glDisable(GL_LINE_STIPPLE);
    for(int iz=0; iz<=zdivisions; iz++)
    {
        int k = 0;
        vet3 *gp;
        int size_gp = 2*(xdivisions+1);
        gp = new vet3 [size_gp]();
        for(int i=0; i<=xdivisions; i++)
        {
           vet3 x = {{x0, y0, z0}};
           vet3 v1 = Properties::FractToCart(x);
           gp[k] = Properties::Trasforma(v1);
           vet3 xx = {{x0, y1, z0}};
           v1 = Properties::FractToCart(xx);
           gp[k+1] = Properties::Trasforma(v1);
           k+=2;
           x0 += deltax;
        }
        glDisable (GL_LIGHTING);
        glPushMatrix();
        float color1[3] = {0.7f, 0.7f, 0.7f};
        if(Properties::ifDesaturate())
           Desaturate(color1);
        glColor3fv(color1);
        glBegin(GL_LINES);
        for(int i=0; i<2*(xdivisions+1); i++)
            glVertex3dv(gp[i].v);
        glEnd();

        glPopMatrix();
        delete [] gp;
        z0 += deltaz;
        x0 = 0;
    }

    x0 = y0 = z0 = 0;
    for(int iz=0; iz<=zdivisions; iz++)
    {
        int k = 0;
        vet3 *gp;
        int size_gp = 2*(ydivisions+1);
        gp = new vet3 [size_gp]();
        for(int i=0; i<=ydivisions; i++)
        {
           vet3 x = {{x0, y0, z0}}, v1;
           v1 = Properties::FractToCart(x);
           gp[k] = Properties::Trasforma(v1);
           vet3 xx = {{x1, y0, z0}};
           v1 = Properties::FractToCart(xx);
           gp[k+1] = Properties::Trasforma(v1);
           k+=2;
           y0 += deltay;
        }
        glDisable (GL_LIGHTING);
        glPushMatrix();
        float color1[3] = {0.7f, 0.7f, 0.7f};
        if(Properties::ifDesaturate())
           Desaturate(color1);
        glColor3fv(color1);
        glBegin(GL_LINES);
        for(int i=0; i<2*(ydivisions+1); i++)
            glVertex3dv(gp[i].v);
        glEnd();

        glPopMatrix();
        delete [] gp;
        z0 += deltaz;
        y0 = 0;
    }

    x0 = y0 = z0 = 0;
    for(int iy=0; iy<=ydivisions; iy++)
    {
        int k = 0;
        vet3 *gp;
        int size_gp = 2*(xdivisions+1);
        gp = new vet3 [size_gp]();
        for(int i=0; i<=xdivisions; i++)
        {
           vet3 x = {{x0, y0, z0}}, v1;
           v1 = Properties::FractToCart( x);
           gp[k] = Properties::Trasforma(v1);
           vet3 xx = {{x0, y0, z1}};
           v1 = Properties::FractToCart( xx);
           gp[k+1] = Properties::Trasforma(v1);
           k+=2;
           x0 += deltax;
        }
        glDisable (GL_LIGHTING);
        glPushMatrix();
        float color1[3] = {0.7f, 0.7f, 0.7f};
        if(Properties::ifDesaturate())
           Desaturate(color1);
        glColor3fv(color1);
        glBegin(GL_LINES);
        for(int i=0; i<2*(xdivisions+1); i++)
            glVertex3dv(gp[i].v);
        glEnd();

        glPopMatrix();
        delete [] gp;
        y0 += deltay;
        x0 = 0;
    }

    x0 = 0;
    y0 = 0;
    for(int ix=0; ix<=xdivisions; ix++)
    {
        int k = 0;
        vet3 *gp;
        int size_gp = 2*(ydivisions+1);
        gp = new vet3 [size_gp]();
        for(int i=0; i<=ydivisions; i++)
        {
           vet3 x = {{x0, y0, z0}}, v1;
           v1 = Properties::FractToCart( x);
           gp[k] = Properties::Trasforma(v1);
           vet3 xx = {{x0, y0, z1}};
           v1 = Properties::FractToCart( xx);
           gp[k+1] = Properties::Trasforma(v1);
           k+=2;
           y0 += deltay;
        }
        glDisable (GL_LIGHTING);
        glPushMatrix();
        float color1[3] = {0.7f, 0.7f, 0.7f};
        if(Properties::ifDesaturate())
           Desaturate(color1);
        glColor3fv(color1);
        glBegin(GL_LINES);
        for(int i=0; i<2*(ydivisions+1); i++)
            glVertex3dv(gp[i].v);
        glEnd();

        glPopMatrix();
        delete [] gp;
        x0 += deltax;
        y0 = 0;
    }
    glDisable(GL_LINE_STIPPLE);
}

void MyGLWidget::paintGL()
{
  if(Properties::ifDesaturate())
     RenderSceneStereo();
  else
     RenderSceneNormal();
  m_context->glGetFloatv(GL_MODELVIEW_MATRIX, modelG);
  m_context->glGetFloatv(GL_PROJECTION_MATRIX, projG);
  m_context->glGetIntegerv(GL_VIEWPORT, viewG);
  for(int i=0; i<16;i++)
  {
      Properties::modelG[i] = modelG[i];
      Properties::projG[i] = projG[i];
  }
  for(int i=0; i<4;i++)
      Properties::viewG[i] = viewG[i];
}

void MyGLWidget::on_move(int axis, float val)
{
         val *= m_Range;
         if(axis == 2)
         {
            double oldScale = Properties::Scale;
            if(Properties::ifOrtho)
               Properties::Scale += val;
            else
               Properties::Scale -= val;
            Properties::change_clip_value();
            double mzoom = Properties::Scale / oldScale;
            Properties::obj_pos.x *= mzoom;
            Properties::obj_pos.y *= mzoom;
         }
         else
            Properties::obj_pos.v[axis] += val;
         update();
}

void MyGLWidget::move_pix(int axis, float sign)
{
    m_Step = m_jav->getStep();
    float step = sign * static_cast<float>(m_Step);
    float *pos1 = FromPixel(0.f, 0.f);
    float *pos2 = FromPixel(step, step);
    Properties::obj_pos.v[axis] += (pos2[axis] - pos1[axis]);
    update();
}

void MyGLWidget::on_rotate(int axis, float sign, bool useStep)
{
         m_RotStep = m_jav->getRotStep();
         vet3 a = {{0, 0, 0}};
         a.v[axis] = 1;
         double rad = m_RotStep * M_PI/180;
         if(useStep)
            m_view_quat_diff = MRAxisToQuat(a, sign*rad);
         else
            m_view_quat_diff = MRAxisToQuat(a, sign);
         m_view_quat = MRAddQuats (m_view_quat_diff, m_view_quat);
         m_Rotm = MRMatrix4MakeFromQuat(m_view_quat);
         update();
}

void MyGLWidget::wheelEvent(QWheelEvent *event)
{
    QPoint nPix = event->pixelDelta();
    if(!nPix.isNull() && (event->source() == Qt::MouseEventSynthesizedBySystem))
    {
        if(nPix.y() < 0)
            on_move(2, 20.f/width());
        else if(nPix.y() > 0)
            on_move(2, -20.f/width());
        return;
    }
    if(event->angleDelta().y() < 0)
       on_move(2, -20.f/width());
    else if(event->angleDelta().y() > 0)
       on_move(2, 20.f/width());
}

void MyGLWidget::mousePressEvent(QMouseEvent *event)
{
#if 0
    m_context->glViewport(viewG[0], viewG[1], viewG[2], viewG[3]);
        glMatrixMode(GL_PROJECTION);
        glLoadMatrixf(projG);
        glMatrixMode(GL_MODELVIEW);
        glLoadMatrixf(modelG);
#endif
        makeCurrent();
        bool ifShift=false, ifCtrl=false;
        m_moving = false;
        in_selection = false;
        activeLegend = -1;
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
        int x = event->x();
        int y = event->y();
#else
        int x = event->position().x();
        int y = event->position().y();
#endif
        Qt::KeyboardModifiers mod = event->modifiers();
        ifShift = mod & Qt::ShiftModifier;
        ifCtrl  = mod & Qt::ControlModifier;

        //if (event->buttons() & Qt::RightButton) {
        if(event->button() == Qt::RightButton) {
#ifdef HAVE_LEGEND
           if(ManageClickLegend(x, y, 0))
              return;
#endif
           if(m_motion_type == NO_MOVE)
               m_motion_type = TRASL;
           else
               return;
        }
        else {
            m_moving = true;
            m_Xstart = x;
            m_Ystart = y;
            if((m_motion_type == RECT_SELECT) || (m_motion_type == ZOOM_MODE))
            {
               m_moving = false;
               in_selection = true;
               return;
            }
#ifdef HAVE_LEGEND
            if(ManageClickLegend(x, y, 1))
               return FALSE;
#endif
            if(m_motion_type == NO_MOVE)
            {
               if(ifCtrl)
                   m_motion_type = TRASL ;
               else if(ifShift)
                   m_motion_type = ZOOM ;
               else
                   m_motion_type = ROT ;
            }
// per abilitare la rotazione durante i calcoli geometrici
            else if((m_motion_type < (GEOM+1)) || (m_motion_type >= (MOVE_LEGEND)) )
                  return;
        }
        switch(m_motion_type )
        {
        case ROT :   // rotation
        case ZOOM :  // zoom
        case TRASL : // translation
        case ON_SELECT : // selection
        case MOL_SELECT : // selection
    // per abilitare la rotazione durante i calcoli geometrici
        case GEOM+1 :             // sphere
        case GEOM+2:          // Select 2, 3 or 4 atoms
        case GEOM+3:
        case GEOM+4:
             m_view_quat_diff = MRVector4Identity;
             break ;
        default :
             break ;
        }
}

void MyGLWidget::mouseMoveEvent(QMouseEvent *event)
{
#if 0
        m_context->glViewport(viewG[0], viewG[1], viewG[2], viewG[3]);
        glMatrixMode(GL_PROJECTION);
        glLoadMatrixf(projG);
        glMatrixMode(GL_MODELVIEW);
        glLoadMatrixf(modelG);
#endif
        makeCurrent();
        double w = static_cast<double>(width());
        double h = static_cast<double>(height());
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
        int x = event->x();
        int y = event->y();
#else
        int x = event->position().x();
        int y = event->position().y();
#endif
        float val;

        if(event->buttons() == Qt::NoButton)
        {
            //return;


            int yy = height() - y;
            // aggiunte le 4 seguenti per gestire i PC meno dotati
            if((abs(x-m_Xstart) < 10) && (abs(y-m_Ystart) < 10))
               return;
            m_Xstart = x;
            m_Ystart = y;
            if(Properties::nModels)
               m_jav->IsAtom(x, yy);
            return;
        }
        if((event->buttons() & Qt::RightButton) && (m_motion_type == TRASL))
        {
           m_moving = true;
           val = ((float) Properties::obj_pos.x +  (float)(x-m_Xstart))/w;
           on_move(0, val);
           val = ((float) Properties::obj_pos.y -  (float)(y-m_Ystart))/h;
           on_move(1, val);
           m_Xstart = x;
           m_Ystart = y;
           update();
           return;
        }
        else if (!(event->buttons() & Qt::LeftButton))
           return;
        switch( m_motion_type )
        {
#ifdef HAVE_LEGEND
        case MOVE_LEGEND:         // Move Legend
           if(activeLegend != -1)
           {
              Legend[activeLegend].SetPos(x, y);
              update();
           }
           return;
           break;
#endif
        case RECT_SELECT:       // Rectangular sel.
        case ZOOM_MODE:         // zoom
             if(in_selection)
             {
                float *pos2 = FromPixel(x, y);
                float *pos1 = FromPixel(m_Xstart, m_Ystart);
                BuildRect(Properties::SelectionRect, pos1, pos2);
                update();
                return;
             }
             break;
        case ON_SELECT:          // Selection
        case MOL_SELECT:          // Selection
        case ROT :             // rotation
      // per abilitare la rotazione durante i calcoli geometrici
        case GEOM+1 :             // sphere
        case GEOM+2:          // Select 2, 3 or 4 atoms
        case GEOM+3:
        case GEOM+4:
             m_moving = true;
             if (event->buttons() & Qt::LeftButton)
             {
                 m_view_quat_diff = MRTrackBall (
                        (2.0 * static_cast<double> (m_Xstart) - w) /w,
                        (h - 2.0 * static_cast<double> (m_Ystart)) /h,
                        (2.0 * static_cast<double> (x) - w) / w,
                        (h - 2.0 * static_cast<double> (y)) / h);
                  m_view_quat = MRAddQuats(m_view_quat_diff, m_view_quat);
                  m_Rotm = MRMatrix4MakeFromQuat (m_view_quat);
                  update();
             }
             break ;
        case ZOOM :
               m_moving = true;
               val =  (float)((y-m_Ystart)/w);
               on_move(2, val);
               update();
          break ;
        }
        m_Xstart = x;
        m_Ystart = y;
}

void MyGLWidget::contextMenuEvent(QContextMenuEvent *event)
{
    int x = event->x();
    int y = height() - event->y();
    int result, *p;

    p = pickObjects(x, y);

    result = p[0];
    if(result)
    {
       QMenu menu;
       if(m_jav->doPopup(p, &menu))
       {
          menu.exec(event->globalPos());
       }
    }
    QOpenGLWidget::contextMenuEvent(event);
}

void MyGLWidget::SetMotionType(int mode, int flag)
{
        m_motion_type = mode;
        switch(flag)
        {
        case 1:
        case 3:
            setCursor(m_cursorCross);
            break;
        case 2:
            setCursor(m_cursorMove);
            break;
        case 4:
            setCursor(m_cursorZoom);
            break;
        case 5:
            setCursor(m_cursorWait);
            break;
        default:
            unsetCursor();
            break;
        }
        if(flag == 3)
        {
           nclicked = 0;
           m_jav->setUnselect();
        }
}

void MyGLWidget::mouseReleaseEvent(QMouseEvent *event)
{
#if 0
        m_context->glViewport(viewG[0], viewG[1], viewG[2], viewG[3]);
        glMatrixMode(GL_PROJECTION);
        glLoadMatrixf(projG);
        glMatrixMode(GL_MODELVIEW);
        glLoadMatrixf(modelG);
#endif
        makeCurrent();

//        if(event->buttons() & Qt::RightButton)
        if(event->button() == Qt::RightButton)
        {
            if(m_motion_type == TRASL)
               m_motion_type = NO_MOVE ;
            update();
            return;
        }
        // 22/07/2019 ???????????
        //if (event->button() == Qt::LeftButton && m_moving) {
            //drawLineTo(event->pos());
        //    m_moving = false;
        //}
        bool ifmov = m_moving;
        m_moving = false;
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
        int x = event->x();
        int y = event->y();
#else
        int x = event->position().x();
        int y = event->position().y();
#endif
        int yy = height() - y;

        if(m_motion_type < RECT_SELECT)
           in_selection = false;
        switch(m_motion_type)
        {
        case ON_SELECT:       // Select Objects
        case MOL_SELECT:       // Select Objects
             m_jav->selectObject(x, yy);
             if(m_motion_type == MOL_SELECT)
                m_jav->do_actionExpand_to_Molecule();
            update();
            return;
            break;
        case SELECT_DEST:     // Select atom to minimize distances
             if(m_jav->selectObject(x, yy, true, true))
                m_jav->do_move_atoms();
              SetMotionType(NO_MOVE, 0);
//              PrintMess(" ");
              m_jav->updateModify(1);
              break;
#ifdef HAVE_LEGEND
        case MOVE_LEGEND:     // Move Legend
             activeLegend = -1;
             SetMotionType(NO_MOVE, 0);
             update();
             return;
             break;
#endif
        case ZOOM_MODE:      // Zoom
             if(in_selection)
             {
                 double wp = abs(m_Xstart -x);
                 double hp = abs(m_Ystart -y);
                 if((wp < 1e-6) && (hp < 1e-6))
                 {
                     float *pos2 = FromPixel(x, y);
                     Properties::obj_pos.x += -pos2[0];
                     Properties::obj_pos.y += -pos2[1];
                     double mzoom = 2;
                     Properties::obj_pos.x *= mzoom;
                     Properties::obj_pos.y *= mzoom;
                     Properties::Scale *= (float) mzoom;
                     ResetModelMatrix(0);
                 }
                 else
                 {
                     double mzoom = min((double) width()/wp, (double) height()/hp);
                     float *pos2 = FromPixel(x, y);
                     float *pos1 = FromPixel(m_Xstart, m_Ystart);
                     BuildRect(Properties::SelectionRect, pos1, pos2);
                     Properties::obj_pos.x += -((pos1[0] + pos2[0]) / 2.f);
                     Properties::obj_pos.y += -((pos1[1] + pos2[1]) / 2.f);
     //nuovo metodo
                     Properties::obj_pos.x *= mzoom;
                     Properties::obj_pos.y *= mzoom;
                     Properties::Scale *= (float) mzoom;
                     ResetModelMatrix(0);
     //end
     //                ResetProjectionMatrix((float)m_Range/mzoom);
                 }
             }
             in_selection = false;
             update();
             return;
             break;
        case RECT_SELECT:    // Rectangular selection
             if(in_selection)
             {
                 int xc = (int) ((m_Xstart+x)*0.5f);
                 int yc = height() - (int) ((m_Ystart+y)*0.5f);
                 double wp = abs(m_Xstart -x);
                 double hp = abs(m_Ystart -y);
                 if((wp < 1e-6) && (hp < 1e-6))
                     m_jav->selectObject(x, yy);
                 else
                 {
                     m_jav->selectObjects(xc, yc, wp, hp);
                     float *pos2 = FromPixel(x, y);
                     float *pos1 = FromPixel(m_Xstart, m_Ystart);
                     BuildRect(Properties::SelectionRect, pos1, pos2);
                 }
                 in_selection = false;
                 SetMotionType(NO_MOVE, 0);
             }
             break;
        case GEOM+1:          // Sphere
        case GEOM+2:          // Select 2, 3 or 4 atoms
        case GEOM+3:
        case GEOM+4:
             if(!m_jav->SelectAtoms(x, yy))
                return;
             if(nclicked == 0)
                SetMotionType(m_motion_type, 1);
             return;
             break;
#ifdef MAPS
        case SELECT_CENTRE: // Select atom
             m_jav->SelectAtom(x, yy);
             break;
#endif
        default:
             nclicked = 0;
             break;
        }
        m_motion_type = NO_MOVE;
        if(ifmov)
           update();
}

float *MyGLWidget::FromPixel(int x, int y)
{
        GLfloat nRange = SCALA * m_Range;
        float *pos;
        pos = new float [2]();
        if((x < 0) || (x > width()))
           x = width();
        if((y < 0) || (y > height()))
           y = height();
        y = height() - y;
        if (m_aspectRatio <= 1)
        {
            pos[0] = 2.f * nRange * (float) x / (float) width()  - nRange;
            pos[1] = 2.f * nRange / m_aspectRatio * (float) y / (float) height()  - nRange / m_aspectRatio;
        }
        else
        {
            pos[0] = 2.f * nRange * m_aspectRatio * (float) x / (float) width()  - nRange * m_aspectRatio;
            pos[1] = 2.f * nRange * (float) y / (float) height()  - nRange;
        }
        return pos;
}

float *MyGLWidget::FromPixel(float x, float y)
{
        GLfloat nRange = SCALA * m_Range;
        float *pos;
        pos = new float [2]();
#if 0
        if((x < 0) || (x > width()))
           x = width();
        if((y < 0) || (y > height()))
           y = height();
#endif
        y = (float) height() - y;
        if (m_aspectRatio <= 1)
        {
            pos[0] = 2.f * nRange * x / (float) width()  - nRange;
            pos[1] = 2.f * nRange / m_aspectRatio * y / (float) height()  - nRange / m_aspectRatio;
        }
        else
        {
            pos[0] = 2.f * nRange * m_aspectRatio * x / (float) width()  - nRange * m_aspectRatio;
            pos[1] = 2.f * nRange * y / (float) height()  - nRange;
        }
        return pos;
}

void MyGLWidget::SetLights()
{
        glMatrixMode (GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();

       if( m_lightInfo[JAV_AMBIENT].enabled)
          glLightfv(GL_LIGHT0, GL_AMBIENT, m_lightInfo[JAV_AMBIENT].diffuse);
       else
          glLightfv(GL_LIGHT0, GL_AMBIENT, no_mat);
       glMaterialfv (GL_FRONT, GL_SPECULAR, m_lightInfo[JAV_LIGHT0].specular);
       if( m_lightInfo[JAV_LIGHT0].enabled)
       {
          glEnable (GL_LIGHT0);
          glLightfv(GL_LIGHT0, GL_DIFFUSE,  m_lightInfo[JAV_LIGHT0].diffuse);
          glLightfv(GL_LIGHT0, GL_POSITION, m_lightInfo[JAV_LIGHT0].position);
          glLightfv(GL_LIGHT0, GL_SPECULAR, m_lightInfo[JAV_LIGHT0].specular);
       }
       else
          glDisable (GL_LIGHT0);
       if(m_lightInfo[JAV_LIGHT1].enabled)
       {
          glEnable (GL_LIGHT1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE,  m_lightInfo[JAV_LIGHT1].diffuse);
          glLightfv(GL_LIGHT1, GL_POSITION, m_lightInfo[JAV_LIGHT1].position);
          glLightfv(GL_LIGHT1, GL_SPECULAR, m_lightInfo[JAV_LIGHT1].specular);
       }
       else
          glDisable (GL_LIGHT1);
        glPopMatrix();
}

void MyGLWidget::SetDefaultLights(GL_LightInfo l_info[])
{
//Light 0
     l_info[JAV_LIGHT0].enabled = Properties::lightInfo[JAV_LIGHT0].enabled;
     for(int i=0; i<4; i++)
         l_info[JAV_LIGHT0].position[i] =
		 Properties::lightInfo[JAV_LIGHT0].position[i];

     FromQColor(Properties::lightInfo[JAV_LIGHT0].specular,
			      l_info[JAV_LIGHT0].specular, 4);
     FromQColor(Properties::lightInfo[JAV_LIGHT0].diffuse,
			      l_info[JAV_LIGHT0].diffuse, 4);

//Light 1
     l_info[JAV_LIGHT1].enabled = Properties::lightInfo[JAV_LIGHT1].enabled;
     for(int i=0; i<4; i++)
         l_info[JAV_LIGHT1].position[i] =
		 Properties::lightInfo[JAV_LIGHT1].position[i];

     FromQColor(Properties::lightInfo[JAV_LIGHT1].specular,
			      l_info[JAV_LIGHT1].specular, 4);
     FromQColor(Properties::lightInfo[JAV_LIGHT1].diffuse,
			      l_info[JAV_LIGHT1].diffuse, 4);

// Ambient
     l_info[JAV_AMBIENT].enabled = Properties::lightInfo[JAV_AMBIENT].enabled;
     FromQColor(Properties::lightInfo[JAV_AMBIENT].diffuse,
			      l_info[JAV_AMBIENT].diffuse, 4);
     SetLights();
}

void MyGLWidget::DrawBackground()
{
        QPainter painter(this);
        QLinearGradient gradient(0,0,0,0);
        gradient.setSpread(QGradient::PadSpread);
        gradient.setColorAt(0, m_bg0);
        gradient.setColorAt(0.5, Blend(m_bg0,m_bg1));
        gradient.setColorAt(1, m_bg1);

        QRadialGradient radialGrad(QPointF(width()/2, height()/2),
			max(width()/2, height()/2));
        radialGrad.setColorAt(0, m_bg1);
        radialGrad.setColorAt(0.6, Blend(m_bg0,m_bg1));
        radialGrad.setColorAt(1, m_bg0);

        switch(m_bgType)
        {
        case ColorWidget::TYPE_BLACK:
            m_context->glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
            m_context->glClear(GL_COLOR_BUFFER_BIT);
            break;
        case ColorWidget::TYPE_WHITE:
            m_context->glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
            m_context->glClear(GL_COLOR_BUFFER_BIT);
            break;
        case ColorWidget::TYPE_FLAT:
            m_context->glClearColor(m_bgColor[0], m_bgColor[1], m_bgColor[2], 1.f);
            m_context->glClear(GL_COLOR_BUFFER_BIT);
            break;
        case ColorWidget::TYPE_HGRAD:
        {
            gradient.setStart(m_r.topLeft());
            gradient.setFinalStop(m_r.topRight());
            QBrush brush(gradient);
            painter.fillRect(m_r, brush);
            break;
        }
        case ColorWidget::TYPE_VGRAD:
        {
            gradient.setStart(m_r.topLeft());
            gradient.setFinalStop(m_r.bottomLeft());
            QBrush brush(gradient);
            painter.fillRect(m_r, brush);
            break;
        }
        case ColorWidget::TYPE_DGRAD:
        {
//            gradient.setStart(m_r.topRight());
//            gradient.setFinalStop(m_r.bottomLeft());
            gradient.setStart(m_r.topLeft());
            gradient.setFinalStop(m_r.bottomRight());
            QBrush brush(gradient);
            painter.fillRect(m_r, brush);
            break;
        }
        case ColorWidget::TYPE_DGRAD2:
        {
            gradient.setStart(m_r.bottomLeft());
            gradient.setFinalStop(m_r.topRight());
            QBrush brush(gradient);
            painter.fillRect(m_r, brush);
            break;
        }
        case ColorWidget::TYPE_RGRAD:
        {
            gradient.setStart(m_r.topLeft());
            gradient.setFinalStop(m_r.bottomRight());
            QBrush brush(radialGrad);
            painter.fillRect(m_r, brush);
            break;
        }
        }
        painter.end();
}

void MyGLWidget::DrawSplash()
{
    QFont myFont;
    QString text("Just Another Viewer");
    QPainter painter(this);
    QColor color_text("#d682d3");
    myFont.setBold(true);
    myFont.setItalic(true);
    myFont.setPixelSize(36);
    painter.setFont(myFont);
    QFontMetrics fm = painter.fontMetrics();
    QRectF rect_text = fm.boundingRect(text);
    int xt = static_cast<int>(std::max(0., m_r.width()-rect_text.width()-20));
    int yt = static_cast<int>(std::max(0., m_r.height()-rect_text.height()-20));

    painter.setPen(color_text);
    painter.drawText(xt, yt, text);
    painter.end();
}

void MyGLWidget::DrawPar(vet3 VC[8], QColor c, int sm, bool drawAxes)
{
        m_context->glDisable(GL_LIGHTING);
        glPushMatrix();
        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
        m_context->glDepthMask (GL_TRUE);
        m_context->glEnable (GL_DEPTH_TEST);
        if(sm)
        {
           m_context->glEnable (GL_LINE_SMOOTH);
           m_context->glEnable (GL_BLEND);
           m_context->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
           m_context->glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
           m_context->glLineWidth(1.5f);
        }
        float color1[3];
        FromQColor(c, color1, 3);
        glColor3fv(color1);
        glBegin(GL_QUAD_STRIP);
            glVertex3dv(VC[1].v);       //V2
            glVertex3dv(VC[0].v);       //V1
            glVertex3dv(VC[3].v);       //V4
            glVertex3dv(VC[2].v);       //V3
            glVertex3dv(VC[5].v);       //V6
            glVertex3dv(VC[4].v);       //V5
            glVertex3dv(VC[7].v);       //V8
            glVertex3dv(VC[6].v);       //V7
            glVertex3dv(VC[1].v);       //V2
            glVertex3dv(VC[0].v);       //V1
        glEnd();
        if(drawAxes)
        {
           m_context->glDepthMask (GL_FALSE);
           m_context->glDisable (GL_DEPTH_TEST);
           glBegin(GL_LINES);
               glColor4f(1, 0, 0, 1);
               glVertex3dv(VC[4].v);
               glVertex3dv(VC[2].v);
               glColor4f(0, 1, 0, 1);
               glVertex3dv(VC[4].v);
               glVertex3dv(VC[5].v);
               glColor4f(0, 0, 1, 1);
               glVertex3dv(VC[4].v);
               glVertex3dv(VC[6].v);
           glEnd();
        //glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
           JavDrawText(VC[4], "O", Properties::CellColor);
           JavDrawText(VC[2], "a", Properties::CellColor);
           JavDrawText(VC[5], "b", Properties::CellColor);
           JavDrawText(VC[6], "c", Properties::CellColor);
           m_context->glDepthMask (GL_TRUE);
           m_context->glEnable (GL_DEPTH_TEST);
        }

        glPopMatrix();
        m_context->glDisable (GL_LINE_SMOOTH);
        m_context->glDisable (GL_BLEND);
}

void MyGLWidget::DrawCell()
{
        QColor color1 = QColor(Properties::CellColor);
        if(Properties::Stereo && (m_bgType < ColorWidget::TYPE_FLAT))
           Desaturate(color1);
        if(Properties::ifDrawAxes)
           DrawPar(Properties::vcN, color1, Properties::smooth, true);
        else
           DrawPar(Properties::vcN, color1, Properties::smooth);
}

void MyGLWidget::JavDrawText(vet3 pos, QString str, QColor color_text)
{
        glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
        QFont myFont;
        QPainter painter(this);
        myFont.setPixelSize(12);
        myFont.setBold(true);
        painter.setFont(myFont);
        painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
        painter.setPen(color_text);
        painter.drawText(ToWin(pos), str);
        painter.end();
}

QPoint MyGLWidget::ToWin(vet3 p)
{
        float *model = new float [16]();
        float *proj  = new float [16]();
        GLint view[4];
        GLdouble winx, winy, winz;
        m_context->glGetFloatv(GL_MODELVIEW_MATRIX, model);
        m_context->glGetFloatv(GL_PROJECTION_MATRIX, proj);
        m_context->glGetIntegerv(GL_VIEWPORT, view);
        view[2] /= devicePixelRatio();
        view[3] /= devicePixelRatio();
        FromCoordToWin(p.x,p.y, p.z,
                       model, proj, view, &winx, &winy, &winz);
        QPoint P1(static_cast<int>(winx), static_cast<int>(height()-winy));
        return P1;
}

void MyGLWidget::EnableClip()
{
        if(Properties::Clip)
        {
           m_context->glEnable(GL_CLIP_PLANE0);
           m_context->glEnable(GL_CLIP_PLANE1);
        }
        else
        {
           m_context->glDisable(GL_CLIP_PLANE0);
           m_context->glDisable(GL_CLIP_PLANE1);
        }
}

void MyGLWidget::DisableClip()
{
        m_context->glDisable(GL_CLIP_PLANE0);
        m_context->glDisable(GL_CLIP_PLANE1);
}

void MyGLWidget::RenderSceneNormal()
{
  // clear
    m_context->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    ResetModelMatrix(0);
    RenderScene();
    m_context->glFlush();
}
	
void MyGLWidget::RenderSceneStereo()
{
  // clear
    m_context->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);

 // left eye
    m_context->glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    switch (Properties::glasses_type)
    {
        case REDBLUE:
        case REDGREEN:
        case REDCYAN:
            m_context->glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_TRUE);
            break;
        case BLUERED:
            m_context->glColorMask(GL_FALSE, GL_FALSE, GL_TRUE, GL_TRUE);
            break;
        case GREENRED:
            m_context->glColorMask(GL_FALSE, GL_TRUE, GL_FALSE, GL_TRUE);
            break;
        case CYANRED:
            m_context->glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_TRUE);
            break;
    }
    ResetModelMatrix(-1);
    RenderScene();
    m_context->glFlush();
    m_context->glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
    glAccum(GL_LOAD,1.0);
    glDrawBuffer(GL_BACK);
    m_context->glClear(GL_DEPTH_BUFFER_BIT);
// right eye
    switch (Properties::glasses_type)
    {
        case REDBLUE:
            m_context->glColorMask(GL_FALSE, GL_FALSE, GL_TRUE, GL_TRUE);
            break;
        case REDGREEN:
            m_context->glColorMask(GL_FALSE, GL_TRUE, GL_FALSE, GL_TRUE);
            break;
        case REDCYAN:
            m_context->glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_TRUE);
            break;
        case BLUERED:
        case GREENRED:
        case CYANRED:
            m_context->glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_TRUE);
            break;
    }
    ResetModelMatrix(1);
    RenderScene();
    m_context->glFlush();
    m_context->glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
}
    

void MyGLWidget::RenderScene()
{
        ReadSettings(false);
        DrawBackground();

        if( (Properties::nModels == 0) && (Properties::nMaps == 0) )
        {
            if(!DrawTrigs(m_context))
                DrawSplash();
            return;
        }
        glEnable (GL_LIGHTING);
        glDisable(GL_FOG);

      // draw cell

        if(Properties::ifCell)
           DrawCell();
        if(Properties::ifGrid)
           DrawGrid();
        if(Properties::ifDrawPar)
           DrawPar(Properties::VertParN, Properties::AxesColor, Properties::smooth);
        // define clip planes

          glMatrixMode(GL_MODELVIEW);
          glPushMatrix();
          glLoadIdentity();
          glClipPlane(GL_CLIP_PLANE0, Properties::eq0);
          glClipPlane(GL_CLIP_PLANE1, Properties::eq1);
          EnableClip();
          glPopMatrix();

        // define fog type

         if(Properties::Fog)  // && (BgType <= TYPE_FLAT))
         {
            m_context->glEnable(GL_FOG);
            GLfloat fogColor[4],
                    bg0[4]={static_cast<float>(m_bg0.redF()),
                            static_cast<float>(m_bg0.greenF()),
                            static_cast<float>(m_bg0.blueF()),
                            static_cast<float>(m_bg0.alphaF())},
                    bg1[4]={static_cast<float>(m_bg1.redF()),
                            static_cast<float>(m_bg1.greenF()),
                            static_cast<float>(m_bg1.blueF()),
                            static_cast<float>(m_bg1.alphaF())};
            if (m_bgType <= ColorWidget::TYPE_FLAT)
            {
                for(int j=0; j<3;j++)
                    fogColor[j] = m_bgColor[j];
                fogColor[3] = 1;
            }
            else
                for(int j=0; j<4;j++)
                    fogColor[j] = (bg0[j]+bg1[j])/2;
            GLint fogMode = GL_EXP2;
            glFogi (GL_FOG_MODE, fogMode);
            glFogfv (GL_FOG_COLOR, fogColor);
            glFogf (GL_FOG_DENSITY, Properties::FogDensity);
            m_context->glHint (GL_FOG_HINT, GL_NICEST);  //  per pixel
            if(fogMode == GL_LINEAR)
            {
               glFogf(GL_FOG_START, 0.4f);
               glFogf(GL_FOG_END, 10.f);
            }
         }
         else
            m_context->glDisable(GL_FOG);

// draw model(s)
         m_jav->DrawModels();
         DisableClip();
         DrawDistAng();
         drawSphere();
         if(Properties::ifAxes)
            DrawAxes();
#ifdef MAPS
         m_jav->DrawMaps();
#endif
         m_jav->DrawCentroids();
}

void MyGLWidget::drawSphere()
{
   if(Properties::DrawSphere)
   {
       float mat_e[4] = {0.837f, 1.0f, 1.0f,   1.0f};//light cyan
       SetLightForSelected(m_context, 1, mat_e, 0.3f);
       glPushMatrix();
       glTranslatef( Properties::SphereCentre[0], Properties::SphereCentre[1],
               Properties::SphereCentre[2]);
       float SphereRadius = 2.f * Properties::sphereRadius / Properties::deltav;
       int subd = (int) (250.f * SphereRadius);
       Sfera (true, SphereRadius, subd);
       glPopMatrix();
       ResetLightForSelected(m_context);
   }
}

void MyGLWidget::DrawAxes()
{
    float vp[4];
    float oldRange = m_Range;
    const char *lb[] = {"a", "b", "c"};
    int border = min(100, (int) ((float)width() * 0.2f));
    vet3 xyz[4] = { Properties::xyzsN[0], Properties::xyzsN[1],
	    Properties::xyzsN[2], Properties::xyzsN[3]};

    glDisable(GL_FOG);
    glEnable(GL_LIGHTING);
    glDisable(GL_COLOR_MATERIAL);
    glEnable(GL_NORMALIZE);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    ResetProjectionMatrix(1.f);
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();
    glGetFloatv(GL_VIEWPORT, vp);

    glViewport(10, 10, border-20, border-20);
    glLoadIdentity();
    glTranslatef( 0, 0, 0);

    glMultMatrixd (m_Rotm.m);

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    double rad = 0.06;
    for (int i=0; i<3; i++)
    {
         float colorlab[] = {0, 0, 0, 1};
         colorlab[i] = 1;
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colorlab);
         render3DArrow(xyz[0], xyz[i+1], rad, 10);
         JavDrawText(xyz[i+1], lb[i], ToQColor(colorlab));
         glViewport(10, 10, border-20, border-20);
    }
    float color1[] = {0.95f, 0.95f, 1, 1};
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
    glTranslated(xyz[0].x, xyz[0].y, xyz[0].z);         //O
    Sfera (true, 0.1, 10);

    glColor3f(1.0f, 1.0f, 1.0f);
    glViewport((GLsizei)vp[0], (GLsizei)vp[1], (GLsizei)vp[2], (GLsizei)vp[3]);
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    ResetProjectionMatrix(oldRange);
    glMatrixMode(GL_MODELVIEW);    
}

void MyGLWidget::ViewPlane(int whichPlane, Model md[])
{
   vet3 Ovet       =  Properties::vcN[4];
   vet3 Cellvet[3] =  {Properties::vcN[2], Properties::vcN[5],
                            Properties::vcN[6]};

   int a = tolower(whichPlane) - 'a';
   int a1 = (a+1) % 3;
   int a2 = (a+2) % 3;
   mat4x4 Mat = Properties::ViewPlane(whichPlane, md);
   SetQuat(MRMatrixToQuat(Mat));
   SetQuatDiff(MRVector4Identity);
   SetRotmFromQuat();

   if(a < 3)
   {
           float *model = new float [16]();
           float *proj = new float [16]();
           GLint view[4];
           GLdouble winx, winy, winz;
           GLdouble winx0, winy0, winz0;
           GLdouble winx1, winy1, winz1;
           GLdouble winx2, winy2, winz2;
           GLdouble x = Cellvet[a].x, y = Cellvet[a].y, z = Cellvet[a].z;
           GLdouble x1 = Cellvet[a1].x, y1 = Cellvet[a1].y, z1 = Cellvet[a1].z;
           GLdouble x2 = Cellvet[a2].x, y2 = Cellvet[a2].y, z2 = Cellvet[a2].z;
           GLdouble x0 = Ovet.x, y0 = Ovet.y, z0 = Ovet.z;

    // allineo il primo asse con l'asse x (perlomeno ci provo)
           ResetModelMatrix(0);
           m_context->glGetFloatv(GL_MODELVIEW_MATRIX, model);
           m_context->glGetFloatv(GL_PROJECTION_MATRIX, proj);
           m_context->glGetIntegerv(GL_VIEWPORT, view);
           FromCoordToWin(x0, y0, z0, model, proj, view, &winx0, &winy0, &winz0);
           FromCoordToWin(x1, y1, z1, model, proj, view, &winx1, &winy1, &winz1);
           double dist = sqrt((winx1-winx0)*(winx1-winx0) + (winy1-winy0)*(winy1-winy0));
           double xang = asin((winy1-winy0)/dist);
           if(abs(xang) > 1e-7)
             on_rotate(2, (float) xang, false);

    // origine della cella in basso
           ResetModelMatrix(0);
           m_context->glGetFloatv(GL_MODELVIEW_MATRIX, model);
           m_context->glGetFloatv(GL_PROJECTION_MATRIX, proj);
           m_context->glGetIntegerv(GL_VIEWPORT, view);
           FromCoordToWin(x, y, z, model, proj, view, &winx, &winy, &winz);
           FromCoordToWin(x0, y0, z0, model, proj, view, &winx0, &winy0, &winz0);
           FromCoordToWin(x2, y2, z2, model, proj, view, &winx2, &winy2, &winz2);
           if(winy2 < winy0)
             on_rotate(0, -(float) M_PI, false);
   }
   update();
}

int *MyGLWidget::pickObjects(int x, int y, bool only_atoms,
                     double wp, double hp, bool ifMultiple)
{
        int BUFSIZE = 0;
        GLint viewport[4];

        if(ifMultiple)
           BUFSIZE = 5000;
        else
           BUFSIZE = 100;
        GLuint *selectBuf = new GLuint [BUFSIZE]();
        GLint hits;

        makeCurrent();
        m_context->glGetIntegerv (GL_VIEWPORT, viewport);
        viewport[2] /= devicePixelRatio();
        viewport[3] /= devicePixelRatio();
        glSelectBuffer (BUFSIZE, selectBuf);
        (void) glRenderMode (GL_SELECT);

        glMatrixMode(GL_PROJECTION);
        glPushMatrix ();
        glLoadIdentity();

     //  create wpxhp pixel picking region near cursor location
        pickMatrix (x, y, wp, hp, viewport);

        SetViewpoint(m_Range);

        glMatrixMode(GL_MODELVIEW);

        glInitNames();
        m_jav->DrawForSelect(only_atoms);

        glMatrixMode (GL_PROJECTION);
        glPopMatrix ();
        m_context->glFlush ();

        hits = glRenderMode (GL_RENDER);
        glMatrixMode(GL_MODELVIEW);
        int *vres = new int [4]();
        if(hits > 0)
        {
           std::vector<PickResult> vresult;
           int result = processHitsm (hits, selectBuf, vresult);
           vres[0] = result;
           if(result)
           {
               int sz = (int) vresult.size();
               if(ifMultiple)
               {
                  delete [] vres;
                  vres = new int [3*sz+1]();
                  vres[0] = sz;
                  for(int i=0; i<sz; i++)
                  {
                      PickResult v = vresult.at(i);
                      vres[3*i+1] = v.type;
                      vres[3*i+2] = v.model;
                      vres[3*i+3] = v.object;
                  }
               }
               else
               {
                  int found = 0;
                  float zmold = 1e6;
                  for(int i=0; i<sz; i++)
                  {
                      PickResult v = vresult.at(i);
                      if((v.type != 1) && (v.zm <= zmold))
                      {
                         vres[1] = v.type;
                         vres[2] = v.model;
                         vres[3] = v.object;
                         zmold   = v.zm;
                         found++;
                      }
                  }
                  if(found == 0)
                  {
                     zmold = 1e6;
                     for(int i=0; i<sz; i++)
                     {
                         PickResult v = vresult.at(i);
                         if(v.zm <= zmold)
                         {
                            vres[1] = v.type;
                            vres[2] = v.model;
                            vres[3] = v.object;
                            zmold   = v.zm;
                         }
                     }
                  }
               }
           }
        }
        delete [] selectBuf;
        return vres;
}

void MyGLWidget::DrawForSelect(bool only_atoms, Model md, int i)
{
        if (md.ifVis)
        {
            if(only_atoms)
                md.DrawAtoms(i);
            else if (md.IsProtein())         // Proteins
            {
                if(md.plotStyle >= RIBBONS)
                   md.DrawRibbons(m_context, false, i);
                else
                {
                    md.DrawBonds2(m_context, i);
                    if(md.plotStyle != C_ALPHA)
                       md.DrawAtoms(i);
                }
                md.DrawBonds2(m_context, i, true);    // Ligands
            }
            else
            {
                md.DrawBonds(m_context, i);
                md.DrawAtoms(i);
            }
        }
}

void MyGLWidget::ChangeOrtho()
{
    makeCurrent();
    ResetProjectionMatrix(1.f);
    ResetModelMatrix(0);
    update();
    doneCurrent();
}

void MyGLWidget::ResetLights()
{
    makeCurrent();
    m_context->glEnable (GL_LIGHTING);
    SetDefaultLights(m_lightInfo);
    update();
    doneCurrent();
}

void MyGLWidget::DrawDistAng()
{
    if(Properties::DistAng_vet.size() <= 0)
       return;
    vet3 strpos;
    QString str;

    glDisable (GL_DEPTH_TEST);
    glLineStipple(1, 0x00FF);
    glEnable(GL_LINE_STIPPLE);
    glLineWidth(1.5);
    glPushMatrix();
    glEnable (GL_BLEND);
    glDisable (GL_LIGHTING);
    if(Properties::smooth)
    {
       glEnable (GL_LINE_SMOOTH);
       glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    }
    for(size_t i=0; i<Properties::DistAng_vet.size(); i++)
    {
        int t = Properties::DistAng_vet.at(i).op;
        str = QString::number(Properties::DistAng_vet.at(i).result, 'f', 3);
        vet3 *p = new vet3 [t]();
        double ang = (double) Properties::DistAng_vet.at(i).result;
        for(int k=0; k<t; k++)
        {
            int wm = Properties::DistAng_vet.at(i).wM[k];
            int wa = Properties::DistAng_vet.at(i).wA[k];
            p[k] = MRVectorMake(m_jav->GetModel(wm+1)->atm[wa].xyzN);
        }
        if(t <= 3)
        {
           strpos = DrawDistLines(t, p);
           if(t == 3)
              strpos = DrawDistAngles(t, p, ang);
        }
        else
           strpos = DrawDistDihedral(p);
        JavDrawText(strpos, str, Properties::DistColor);
    }
    glDisable(GL_LINE_STIPPLE);
    glDisable(GL_LINE_SMOOTH);
    glDisable(GL_BLEND);
    glPopMatrix();
    glEnable (GL_DEPTH_TEST);
    glLineWidth(1);
}
