#ifndef OPENGLWINDOW_H
#define OPENGLWINDOW_H
#include <QOpenGLWidget>
#include <QtGui/QOpenGLFunctions>
#include <QMatrix4x4>
#include <QMouseEvent>
#include "libmatrix.h"
#include "libcomune.h"
#include "models.h"

typedef struct {
                 bool enabled;
                 float diffuse[4];
                 float specular[4];
                 float position[4];
               } GL_LightInfo;

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif


class MyGLWidget : public QOpenGLWidget
{
public:
    MyGLWidget(QWidget *parent) : QOpenGLWidget(parent),
       m_RotStep(5), m_Step(5) {}

    void DrawBackground();
    void ReadSettings(bool fromFile=true);
    void DrawSplash();
    void SetLights();
    void SetDefaultLights(GL_LightInfo l_info[]);
    QOpenGLFunctions *GetContext() {return m_context;}
    ColorWidget::ColorType  GetBgType() {return m_bgType;}
    void SetQuat(vet4 q) {MRQuatCopy(q, &m_view_quat);}
    void SetQuatDiff(vet4 q) {MRQuatCopy(q, &m_view_quat_diff);}
    void SetRotB(mat4x4 m1) {MRMatrixCopy(m1, &m_RotB);}
    void SetRotmFromQuat() {m_Rotm=MRMatrix4MakeFromQuat(m_view_quat);}
    void SetQuatFromRotB() {m_view_quat=MRMatrixToQuat(m_RotB);}
    void ResetProjectionMatrix(float factor);
    void SetViewpoint(float factor);
    void ResetModelMatrix(int sign);
    void DrawPar(vet3 VC[8], QColor c, int sm, bool DrawAxes=false);
    void DrawCell();
    void DrawGrid();
    void JavDrawText(vet3 pos, QString str, QColor c);
    QPoint ToWin(vet3 p);
    void RenderScene();
    void RenderSceneNormal();
    void RenderSceneStereo();
    void EnableClip();
    void DisableClip();
    bool InSelection() {return in_selection;}
    void SetMotionType(int mode, int flag);
    float *FromPixel(int x, int y);
    float *FromPixel(float x, float y);
    void on_move(int axis, float val);
    void move_pix(int axis, float sign);
    void on_rotate(int axis, float sign, bool useStep=true);
    void ResetView(bool azzera=false, bool redraw=true);
    void SetRange(float val) {m_Range=val;}
    void ViewPlane(int whichPlane, Model md[]);
    int *pickObjects(int x, int y, bool only_atoms=false, double wp = 5,
                     double hp = 5, bool ifMultiple=false);
    void DrawForSelect(bool only_atoms, Model md, int i);
    int MotionType() {return m_motion_type;}
    double RotStep() {return m_RotStep;}
    void SetRotStep(double step) {m_RotStep=step;}
    mat4x4 GetRotm() {return m_Rotm;}
    void DrawAxes();
    void ChangeOrtho();
    void DrawDistAng();
    void ResetLights();
    void setMaxSamples(int num) {m_maxSamples=num;}
    int GetMotionType() {return m_motion_type;}
    int GetNClicked() {return nclicked;}
    void drawSphere();
    double getStep() {return m_Step;}
    void setStep(double step) {m_Step = step;}

protected:
    void initializeGL() override;
    void resizeGL(int w, int h) override;
    void paintGL() override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void contextMenuEvent(QContextMenuEvent *event) override;
    void wheelEvent(QWheelEvent *event) override;

private:
    ColorWidget::ColorType m_bgType;
    float  m_bgColor[3];
    QColor m_bg0;
    QColor m_bg1;
    QRect  m_r;
    QMatrix4x4 m_projection;
    QMatrix4x4 m_model;
    float  m_Range;
    float  m_aspectRatio;
    bool   m_moving;
    vet4   m_view_quat_diff;
    vet4   m_view_quat;
    int    m_Xstart;
    int    m_Ystart;
    double m_RotStep;
    double m_Step;
    mat4x4 m_Rotm;
    mat4x4 m_RotB;
#if defined(JAV_WIDGET)
    CrystalViewWidget *m_jav;
#else
    JavWindow *m_jav;
#endif
    QOpenGLFunctions *m_context;
    GL_LightInfo m_lightInfo[3];
    int m_Width;
    int m_Height;
    bool in_selection;
    int m_motion_type;
    int activeLegend;
    QCursor m_cursorZoom;
    QCursor m_cursorCross;
    QCursor m_cursorMove;
    QCursor m_cursorWait;
    int m_maxSamples;
    float modelG[16], projG[16];
    int viewG[4];
    int nclicked;

private slots:
    void deleteselectedab();
};

#endif // OPENGLWINDOW_H
