#ifndef OVERLAYINFO_H
#define OVERLAYINFO_H

#include <QDialog>
#include <QStandardItemModel>
#include <QItemSelection>
#include <QStringList>
#include <QVector>
#include "libcomune.h"

typedef struct {
   int   nfound;    // numero di trovati nel modello 2
   float distmin;   // distanza minima
   float distm;     // distanza media
   float rmsd;      // RMSD calculated on all atoms
   float rmsd_nonH; // RMSD calculated on all non H-atoms
   int   *nv1;      // atomo 1 dell'associazione
   int   *nv2;      // aromo 2 dell'associazione
   float *vdist;    // distanza dell'associazione
   float errph;     // errore di fase
   int   nref;      // numero di riflessi per l'errore di fase
   float shift[4];  // shift applicato
   atom_type *at1;  // modello 1
   atom_type *at2;  // modello 2
   int   nass;      // numero di atomi associati nel modello 1
                } compare_info_type;

namespace Ui {
class OverlayInfo;
}

class OverlayInfo : public QDialog
{
    Q_OBJECT

public:
    explicit OverlayInfo(QWidget *parent = 0);
    ~OverlayInfo();
    void setModel(int k) {kmodel = k;}
    void setOverlayModel(int k) {ovmodel = k;}
    int getModel() {return kmodel;}
    int getOverlayModel() {return ovmodel;}
    float getDist();
    void fillComboModels(QStringList list);
    void setDist(float val);
    void fillResults(compare_info_type *info, QString nome1, int n1, int n2);

private:
    Ui::OverlayInfo *ui;
    QStandardItemModel *matchesList;
    int kmodel;
    int ovmodel;

private slots:
    void on_closeButton_clicked();
    void on_spinDist_valueChanged(double );
    void on_comboModels_currentIndexChanged(int index);
    void rowSelectionChanged(const QItemSelection &, const QItemSelection &);

signals:
    void overlaySelectionChanged(QVector<int> sel);
    void distChanged();
    void overlayModelSelected();
};

#endif // OVERLAYINFO_H
