#include "plotstyledialog.h"
#include "ui_plotstyledialog.h"
#include "xpdutils.h"
#include "mainwindow.h"

//#include <QDebug>

PlotStyleDialog::PlotStyleDialog(QWidget *parent) :
    QDialog(parent),    
    ui(new Ui::PlotStyleDialog),
    mYOffset(0)
{
    ui->setupUi(this);

    ui->tthetaRadioButton->setText('2'+QChar(0x03B8)); //2theta    
    ui->oneOverD2RadioButton->setText(QString("q=1/d")+QChar(0x00B2)); //1/d**2

    //Observed    
    connect(ui->obsCheckBox, &QCheckBox::clicked, this, [=](bool state){
        obsVisible[ui->observedComboBox->currentIndex()] = state;
        enableApplyButton();
    });
    connect(ui->observedLineWidget,&LineStyleWidget::lineColorChanged,this,[=](QColor c){
        obsPen[ui->observedComboBox->currentIndex()].setColor(c);        
        enableApplyButton();
    });
    connect(ui->observedLineWidget,&LineStyleWidget::lineWidthChanged,this,[=](int i){
        obsPen[ui->observedComboBox->currentIndex()].setWidth(i);        
        enableApplyButton();
    });
    connect(ui->observedLineWidget,&LineStyleWidget::lineStyleChanged,this,[=](Qt::PenStyle style) {
        obsPen[ui->observedComboBox->currentIndex()].setStyle(style);        
        enableApplyButton();
    });
    connect(ui->observedMarkerWidget,&MarkerStyleWidget::markerShapeChanged, this, [=](QCPScatterStyle::ScatterShape shape) {
        obsScatter[ui->observedComboBox->currentIndex()].setShape(shape);        
        enableApplyButton();
    });
    connect(ui->observedMarkerWidget, &MarkerStyleWidget::markerColorChanged, this, [=](QColor c) {
        QPen pen = obsScatter[ui->observedComboBox->currentIndex()].pen();
        pen.setColor(c);
        obsScatter[ui->observedComboBox->currentIndex()].setPen(pen);        
        enableApplyButton();
    });
    connect(ui->observedMarkerWidget, &MarkerStyleWidget::markerSizeChanged, this, [=](int size) {
        obsScatter[ui->observedComboBox->currentIndex()].setSize(size);        
        enableApplyButton();
    });

    //Background
    connect(ui->backCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connectLineWidget(ui->backgroundLineWidget);
    connect(ui->backpCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->backgroundPointWidget, &MarkerStyleWidget::markerShapeChanged, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->backgroundPointWidget, &MarkerStyleWidget::markerColorChanged, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->backgroundPointWidget, &MarkerStyleWidget::markerSizeChanged, this, &PlotStyleDialog::enableApplyButton);

    //Calculated
    connect(ui->calcCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connectLineWidget(ui->calculatedLineWidget);

    //Difference
    connect(ui->diffCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connectLineWidget(ui->differenceLineWidget);

    //Cumulative Difference
    connect(ui->cDiffCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connectLineWidget(ui->cumulativeLineWidget);

    //Peaks
    connect(ui->peaksCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connectLineWidget(ui->peaksLineWidget);

    //Reflections
    connect(ui->reflCheckBox, &QCheckBox::clicked, this, [=](bool state){
        reflVisible[ui->reflComboBox->currentIndex()] = state;        
        enableApplyButton();
    });
    connect(ui->reflectionsLineWidget, &LineStyleWidget::lineColorChanged, this, [=](QColor c){
        reflPen[ui->reflComboBox->currentIndex()].setColor(c);        
        enableApplyButton();
    });
    connect(ui->reflectionsLineWidget, &LineStyleWidget::lineWidthChanged, this, [=](int i) {
        reflPen[ui->reflComboBox->currentIndex()].setWidth(i);        
        enableApplyButton();
    });
    connect(ui->reflectionsLineWidget, &LineStyleWidget::lineStyleChanged, this, [=](Qt::PenStyle style) {
        reflPen[ui->reflComboBox->currentIndex()].setStyle(style);        
        enableApplyButton();
    });

    //Axis
    connect(ui->tthetaRadioButton, &QRadioButton::clicked, this, [=](){
       if (mAbscissa != xpdutils::TTHETA) {
           enableApplyButton();
           mAbscissa = xpdutils::TTHETA;
       }
    });
    connect(ui->dRadioButton, &QRadioButton::clicked, this, [=](){
       if (mAbscissa != xpdutils::DVALUE) {
           enableApplyButton();
           mAbscissa = xpdutils::DVALUE;
       }
    });
    connect(ui->oneOverDRadioButton, &QRadioButton::clicked, this, [=](){
       if (mAbscissa != xpdutils::ONE_OVER_DVALUE) {
           enableApplyButton();
           mAbscissa = xpdutils::ONE_OVER_DVALUE;
       }
    });
    connect(ui->oneOverD2RadioButton, &QRadioButton::clicked, this, [=](){
       if (mAbscissa != xpdutils::ONE_OVER_DVALUE2) {
           enableApplyButton();
           mAbscissa = xpdutils::ONE_OVER_DVALUE2;
       }
    });
    connect(ui->linearRadioButton, &QRadioButton::clicked, this, [=](){
       if (mYScaleType != QCPAxis::stLinear) {
           enableApplyButton();
           mYScaleType = QCPAxis::stLinear;
       }
    });
    connect(ui->logarithmicRadioButton, &QRadioButton::clicked, this, [=](){
       if (mYScaleType != QCPAxis::stLogarithmic) {
           enableApplyButton();
           mYScaleType = QCPAxis::stLogarithmic;
       }
    });
    connect(ui->axisComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), ui->axisStackedWidget, &QStackedWidget::setCurrentIndex);
    connectLineWidget(ui->xAxisLSWidget);
    connectLineWidget(ui->yAxisLSWidget);
    connectLineWidget(ui->xAxis2LSWidget);
    connectLineWidget(ui->yAxis2LSWidget);
    ui->yOffsetLineEdit->setValidator(new QDoubleValidator);    
    connect(ui->yOffsetCheckBox, &QCheckBox::clicked, this, [=](bool state){
        ui->yOffsetLineEdit->setEnabled(state);
        if (state) ui->yOffsetLineEdit->setFocus();
        enableApplyButton();
    });
    connect(ui->yOffsetLineEdit, &QLineEdit::textEdited, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->xAxisTickLabelsCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->yAxisTickLabelsCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);

    //Grids
    ui->xGridLSWidget->setBoxName("Grid");
    ui->yGridLSWidget->setBoxName("Grid");
    connectLineWidget(ui->xGridLSWidget);
    connectLineWidget(ui->yGridLSWidget);

    //Background and layout colors
    connect(ui->backgroundWidget, &ColorWidget::colorWidgetChanged, this, &PlotStyleDialog::enableApplyButton);
    connect(ui->layerWidget, &ColorWidget::colorWidgetChanged, this, &PlotStyleDialog::enableApplyButton);

    //Legend
    connect(ui->legendCheckBox, &QCheckBox::clicked, this, &PlotStyleDialog::enableApplyButton);
}

PlotStyleDialog::~PlotStyleDialog()
{
    delete ui;
}

void PlotStyleDialog::setOptions(const QVector<graphItem> &obs, const graphItem &back, const graphItem &bpoints, const graphItem &calc,
                                 const graphItem &diff, const graphItem &cdiff, const graphItem &peaks, const QVector<graphItem> &refl,
                                 const PlotSettings &psettings)
{
    changeApplied = false;

    //Observed
    obsPen.clear();
    obsScatter.clear();
    obsPen.reserve(obs.count());
    obsScatter.reserve(obs.count());
    obsVisible.reserve(obs.count());    
    for (int i = 0; i< obs.count(); i++) {
        obsPen.push_back(obs.at(i).getPen());
        obsScatter.push_back(obs.at(i).getScatter());
        obsVisible.push_back(obs.at(i).isVisible());
    }
    obsVisible0 = obsVisible;
    obsPen0 = obsPen;
    obsScatter0 = obsScatter;

    ui->observedComboBox->clear();
    for (int i = 0; i < obs.count(); i++) {
        ui->observedComboBox->addItem(obs[i].getName());        
    }

    //Background    
    backPen0 = back.getPen();
    backVisible0 = back.isVisible();

    //Background Points    
    backScatter0 = bpoints.getScatter();
    backpVisible0 = bpoints.isVisible();

    //Calculated    
    calcPen0 = calc.getPen();
    calcVisible0 = calc.isVisible();

    //Difference    
    diffPen0 = diff.getPen();
    diffVisible0 = diff.isVisible();

    //Cumulative difference    
    cDiffPen0 = cdiff.getPen();
    cDiffVisible0 = cdiff.isVisible();

    //Peaks    
    peaksPen0 = peaks.getPen();
    peaksVisible0 = peaks.isVisible();

    //Reflections
    reflPen.clear();
    if (refl.count() == 0)  {
        ui->reflComboBox->setDisabled(true);
        ui->reflCheckBox->setDisabled(true);
        ui->reflectionsLineWidget->setDisabled(true);
    } else {
        reflPen.reserve(refl.count());        
        reflVisible.resize(refl.count());        
        for (int i = 0; i< refl.count(); i++) {
            reflPen.push_back(refl[i].getPen());
            reflVisible[i] = refl[i].isVisible();
        }
        reflVisible0 = reflVisible;

        ui->reflComboBox->clear();
        for (int i = 0; i < refl.count(); i++) {
            ui->reflComboBox->addItem(refl[i].getName());
        }
    }
    reflPen0 = reflPen;

    settings0 = psettings;    

    setWidgets(true, backPen0, backScatter0, calcPen0, diffPen0, cDiffPen0, peaksPen0, psettings,
               backVisible0, backpVisible0, calcVisible0, diffVisible0, cDiffVisible0, peaksVisible0);

    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
}

void PlotStyleDialog::setWidgets(bool init, const QPen &backPen, const QCPScatterStyle &backScatter, const QPen &calcPen, const QPen &diffPen,
                                 const QPen &cDiffPen, const QPen &peaksPen, const PlotSettings &psettings,
                                 bool backVisible, bool backpVisible, bool calcVisible, bool diffVisible, bool cDiffVisible, bool peaksVisible)
{
    //Background
    ui->backgroundLineWidget->setLineStyle(backPen);
    ui->backCheckBox->setChecked(backVisible);

    //Background Points
    ui->backgroundPointWidget->setMarkerStyle(backScatter);
    ui->backpCheckBox->setChecked(backpVisible);

    //Calculated
    ui->calculatedLineWidget->setLineStyle(calcPen);
    ui->calcCheckBox->setChecked(calcVisible);

    //Difference
    ui->differenceLineWidget->setLineStyle(diffPen);
    ui->diffCheckBox->setChecked(diffVisible);

    //Cumulative difference
    ui->cumulativeLineWidget->setLineStyle(cDiffPen);
    ui->cDiffCheckBox->setChecked(cDiffVisible);

    //Peaks
    ui->peaksLineWidget->setLineStyle(peaksPen);
    ui->peaksCheckBox->setChecked(peaksVisible);

    //Layout colors
    ui->backgroundWidget->setColorWidget(psettings.backColorType, psettings.backColor, psettings.backColor1, psettings.backColor2);
    ui->layerWidget->setColorWidget(psettings.layerColorType, psettings.layerColor, psettings.layerColor1, psettings.layerColor2);

    //Axis
    mAbscissa = psettings.getAbscissa();
    if (mAbscissa == xpdutils::DVALUE) {
        ui->dRadioButton->setChecked(true);
    } else if (mAbscissa == xpdutils::TTHETA) {
        ui->tthetaRadioButton->setChecked(true);
    } else if (mAbscissa == xpdutils::ONE_OVER_DVALUE) {
        ui->oneOverDRadioButton->setChecked(true);
    } else if (mAbscissa == xpdutils::ONE_OVER_DVALUE2) {
        ui->oneOverD2RadioButton->setChecked(true);
    }
    mYScaleType = psettings.yScaleType;
    if (mYScaleType == QCPAxis::stLinear) {
        ui->linearRadioButton->setChecked(true);
    } else if (mYScaleType == QCPAxis::stLinear) {
        ui->logarithmicRadioButton->setChecked(true);
    }
    ui->xAxisLSWidget->setLineStyle(psettings.xAxisPen);
    ui->yAxisLSWidget->setLineStyle(psettings.yAxisPen);
    ui->xAxis2LSWidget->setLineStyle(psettings.xAxis2Pen);
    ui->yAxis2LSWidget->setLineStyle(psettings.yAxis2Pen);
    ui->yOffsetLineEdit->setText(QString::number(psettings.yOffset));
    ui->yOffsetLineEdit->setEnabled(psettings.hasOffset);
    ui->yOffsetCheckBox->setChecked(psettings.hasOffset);
    ui->xAxisTickLabelsCheckBox->setChecked(psettings.xTickLabels);
    ui->yAxisTickLabelsCheckBox->setChecked(psettings.yTickLabels);

    //Grid
    ui->xGridLSWidget->setLineStyle(psettings.xGridPen);
    ui->yGridLSWidget->setLineStyle(psettings.yGridPen);

    //Legend
    ui->legendCheckBox->setChecked(psettings.legendVisible);

    //Observed and reflections
    if (init) {
        ui->observedComboBox->setCurrentIndex(0);
        if (reflPen.count() > 0) ui->reflComboBox->setCurrentIndex(0);
    } else {
        setObserved(ui->observedComboBox->currentIndex());
        if (reflPen.count() > 0) setReflections(ui->reflComboBox->currentIndex());
    }
}

void PlotStyleDialog::apply(CustomPlotZoom *plot, QVector<graphItem> &obs, graphItem &back, graphItem &bpoints, graphItem &calc,
                            graphItem &diff, graphItem &cdiff, graphItem &peaks, QVector<graphItem> &refl, PlotSettings &psettings,
                            const QVector<double> waves, bool update)
{
    //update = true:  1) widget to psettings, obs, ... 2) replot according to psettings
    //update = false: only replot according to psettings, obs, ... (for Cancel)

    bool plotChanged = false;    // only style, colors have been changed
    bool redrawRequired = false; // visibility has been changed

    for (int i = 0; i < obs.count(); i++) {
        QPen pen = obs[i].getPen();
        if (pen != obsPen[i]) {
            if (update) obs[i].setPen(obsPen[i]);
            int index = obs[i].getGraphIndex();
            plot->graph(index)->setPen(obs[i].getPen());
            plotChanged = true;
        }

        QCPScatterStyle scatter = obs[i].getScatter();        
        if (scatter.shape() != obsScatter[i].shape() || scatter.size() != obsScatter[i].size() || scatter.pen() != obsScatter[i].pen()) {
            if (update) obs[i].setScatter(obsScatter[i]);
            int index = obs[i].getGraphIndex();
            plot->graph(index)->setScatterStyle(obs[i].getScatter());
            plotChanged = true;
        }

        if (obsVisible.at(i) != obs.at(i).isVisible()) {
            if (update) obs[i].setVisible(obsVisible.at(i));
            int index = obs[i].getGraphIndex();
            plot->graph(index)->setVisible(obs.at(i).isVisible());
            if (obs.at(i).isVisible())
                plot->graph(index)->addToLegend();
            else
                plot->graph(index)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->backCheckBox->isChecked() != back.isVisible()) {
        if (update) back.setVisible(ui->backCheckBox->isChecked());
        int indexB = back.getGraphIndex();
        if (indexB != -1) {
            plot->graph(indexB)->setVisible(back.isVisible());
            if (back.isVisible())
                plot->graph(indexB)->addToLegend();
            else
                plot->graph(indexB)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->backgroundLineWidget->isDifferent(back.getPen())) {
        if (update) {
            back.setLineStyle(ui->backgroundLineWidget->getStyle());
            back.setLineColor(ui->backgroundLineWidget->getColor());
            back.setLineWidth(ui->backgroundLineWidget->getWidth());
        }
        int indexB = back.getGraphIndex();
        if (indexB != -1) {
            plot->graph(indexB)->setPen(back.getPen());
            plotChanged = true;
        }
    }    

    if (ui->backpCheckBox->isChecked() != bpoints.isVisible()) {
        if (update) bpoints.setVisible(ui->backpCheckBox->isChecked());
        int indexBP = bpoints.getGraphIndex();
        if (indexBP != -1) {
            plot->graph(indexBP)->setVisible(bpoints.isVisible());
            if (bpoints.isVisible())
                plot->graph(indexBP)->addToLegend();
            else
                plot->graph(indexBP)->removeFromLegend();
            redrawRequired = true;
        }
    }

    QCPScatterStyle scatter = bpoints.getScatter();
    if (ui->backgroundPointWidget->isDifferent(scatter)) {
        if (update) {
            scatter.setShape(ui->backgroundPointWidget->getShape());
            QPen pen = scatter.pen();
            pen.setColor(ui->backgroundPointWidget->getColor());
            scatter.setPen(pen);
            scatter.setSize(ui->backgroundPointWidget->getSize());
            bpoints.setScatter(scatter);
        }
        int indexBP = bpoints.getGraphIndex();
        if (indexBP != -1) {
            plot->graph(indexBP)->setScatterStyle(bpoints.getScatter());
            plotChanged = true;
        }
    }    

    if (ui->calcCheckBox->isChecked() != calc.isVisible()) {
        if (update) calc.setVisible(ui->calcCheckBox->isChecked());
        int indexC = calc.getGraphIndex();
        if (indexC != -1) {
            plot->graph(indexC)->setVisible(calc.isVisible());
            if (calc.isVisible())
                plot->graph(indexC)->addToLegend();
            else
                plot->graph(indexC)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->calculatedLineWidget->isDifferent(calc.getPen())) {
        if (update) {
            calc.setLineStyle(ui->calculatedLineWidget->getStyle());
            calc.setLineColor(ui->calculatedLineWidget->getColor());
            calc.setLineWidth(ui->calculatedLineWidget->getWidth());
        }
        int indexC = calc.getGraphIndex();
        if (indexC != -1) {
            plot->graph(indexC)->setPen(calc.getPen());
            plotChanged = true;
        }
    }

    if (ui->diffCheckBox->isChecked() != diff.isVisible()) {
        if (update) diff.setVisible(ui->diffCheckBox->isChecked());
        int indexD = diff.getGraphIndex();
        if (indexD != -1) {
            plot->graph(indexD)->setVisible(diff.isVisible());
            if (diff.isVisible())
                plot->graph(indexD)->addToLegend();
            else
                plot->graph(indexD)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->differenceLineWidget->isDifferent(diff.getPen())) {
        if (update) {
            diff.setLineStyle(ui->differenceLineWidget->getStyle());
            diff.setLineColor(ui->differenceLineWidget->getColor());
            diff.setLineWidth(ui->differenceLineWidget->getWidth());
        }
        int indexD = diff.getGraphIndex();
        if (indexD != -1) {
            plot->graph(indexD)->setPen(diff.getPen());
            plotChanged = true;
        }
    }

    if (ui->cDiffCheckBox->isChecked() != cdiff.isVisible()) {
        if (update) cdiff.setVisible(ui->cDiffCheckBox->isChecked());
        int indexCD = cdiff.getGraphIndex();
        if (indexCD != -1) {
            plot->graph(indexCD)->setVisible(cdiff.isVisible());
            if (diff.isVisible())
                plot->graph(indexCD)->addToLegend();
            else
                plot->graph(indexCD)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->cumulativeLineWidget->isDifferent(cdiff.getPen())) {
        if (update) {
            cdiff.setLineStyle(ui->cumulativeLineWidget->getStyle());
            cdiff.setLineColor(ui->cumulativeLineWidget->getColor());
            cdiff.setLineWidth(ui->cumulativeLineWidget->getWidth());
        }
        int indexCD = diff.getGraphIndex();
        if (indexCD != -1) {
            plot->graph(indexCD)->setPen(cdiff.getPen());
            plotChanged = true;
        }
    }

    for (int i = 0; i < refl.count(); i++) {
        if (reflVisible.at(i) != refl.at(i).isVisible()) {
            if (update) refl[i].setVisible(reflVisible.at(i));
            if (refl[i].xSize() > 0) {
                for (int ind = refl[i].itemIndexStart; ind <= refl[i].itemIndexEnd; ind++) {
                    QCPItemLine *line = dynamic_cast<QCPItemLine *> (plot->item(ind));
                    line->setVisible(refl.at(i).isVisible());
                }
                int index = refl[i].getGraphIndex();                
                plot->graph(index)->setVisible(refl.at(i).isVisible());
                if (refl.at(i).isVisible()) {
                    plot->graph(index)->setVisible(true);
                    plot->graph(index)->addToLegend();
                } else {
                    plot->graph(index)->setVisible(false);
                    plot->graph(index)->removeFromLegend();
                }
                redrawRequired = true;
            }
        }
    }

    for (int i = 0; i < refl.count(); i++) {
        QPen pen = refl[i].getPen();
        if (pen != reflPen[i]) {
            if (update) refl[i].setPen(reflPen[i]);
            int index = refl[i].getGraphIndex();
            QPen penref = refl[i].getPen();
            plot->graph(index)->setPen(penref);
            if (refl[i].xSize() > 0) {
                for (int ind = refl[i].itemIndexStart; ind <= refl[i].itemIndexEnd; ind++) {
                    QCPItemLine *line = dynamic_cast<QCPItemLine *> (plot->item(ind));
                    line->setPen(penref);
                }
                plotChanged = true;
            }
        }
    }

    if (ui->peaksCheckBox->isChecked() != peaks.isVisible()) {
        if (update) peaks.setVisible(ui->peaksCheckBox->isChecked());
        int indexP = peaks.getGraphIndex();
        if (indexP != -1) {
            plot->graph(indexP)->setVisible(peaks.isVisible());
            if (peaks.isVisible())
                plot->graph(indexP)->addToLegend();
            else
                plot->graph(indexP)->removeFromLegend();
            redrawRequired = true;
        }
    }

    if (ui->peaksLineWidget->isDifferent(peaks.getPen())) {
        if (update) {
            peaks.setLineStyle(ui->peaksLineWidget->getStyle());
            peaks.setLineColor(ui->peaksLineWidget->getColor());
            peaks.setLineWidth(ui->peaksLineWidget->getWidth());
        }
        int indexP = peaks.getGraphIndex();
        if (indexP != -1) {
            plot->graph(indexP)->setPen(peaks.getPen());
            plotChanged = true;
        }
    }

    if (ui->backgroundWidget->isDifferent(psettings.backColorType,psettings.backColor,psettings.backColor1,psettings.backColor2)) {
        if (update) {
            ui->backgroundWidget->getColorWidget(psettings.backColorType,psettings.backColor,psettings.backColor1,psettings.backColor2);
        }
        psettings.applyToBackground(plot);
        plotChanged = true;
    }

    if (ui->layerWidget->isDifferent(psettings.layerColorType,psettings.layerColor,psettings.layerColor1,psettings.layerColor2)) {
        if (update) {
            ui->layerWidget->getColorWidget(psettings.layerColorType,psettings.layerColor,psettings.layerColor1,psettings.layerColor2);
        }
        psettings.applyToLayer(plot);
        plotChanged = true;
    }

    if (ui->xAxisLSWidget->isDifferent(psettings.xAxisPen)) {
        if (update) {
            psettings.xAxisPen.setColor(ui->xAxisLSWidget->getColor());
            psettings.xAxisPen.setStyle(ui->xAxisLSWidget->getStyle());
            psettings.xAxisPen.setWidth(ui->xAxisLSWidget->getWidth());
        }
        plot->xAxis->setBasePen(psettings.xAxisPen);
        plot->xAxis->setTickPen(psettings.xAxisPen);
        plot->xAxis->setSubTickPen(psettings.xAxisPen);
        plotChanged = true;
    }

    if (ui->yAxisLSWidget->isDifferent(psettings.yAxisPen)) {
        if (update) {
            psettings.yAxisPen.setColor(ui->yAxisLSWidget->getColor());
            psettings.yAxisPen.setStyle(ui->yAxisLSWidget->getStyle());
            psettings.yAxisPen.setWidth(ui->yAxisLSWidget->getWidth());
        }
        plot->yAxis->setBasePen(psettings.yAxisPen);
        plot->yAxis->setTickPen(psettings.yAxisPen);
        plot->yAxis->setSubTickPen(psettings.yAxisPen);
        plotChanged = true;
    }

    if (ui->xAxis2LSWidget->isDifferent(psettings.xAxis2Pen)) {
        if (update) {
            psettings.xAxis2Pen.setColor(ui->xAxis2LSWidget->getColor());
            psettings.xAxis2Pen.setStyle(ui->xAxis2LSWidget->getStyle());
            psettings.xAxis2Pen.setWidth(ui->xAxis2LSWidget->getWidth());
        }
        plot->xAxis2->setBasePen(psettings.xAxis2Pen);
        plot->xAxis2->setTickPen(psettings.xAxis2Pen);
        plot->xAxis2->setSubTickPen(psettings.xAxis2Pen);
        plotChanged = true;
    }

    if (ui->yAxis2LSWidget->isDifferent(psettings.yAxis2Pen)) {
        if (update) {
            psettings.yAxis2Pen.setColor(ui->yAxis2LSWidget->getColor());
            psettings.yAxis2Pen.setStyle(ui->yAxis2LSWidget->getStyle());
            psettings.yAxis2Pen.setWidth(ui->yAxis2LSWidget->getWidth());
        }
        plot->yAxis2->setBasePen(psettings.yAxis2Pen);
        plot->yAxis2->setTickPen(psettings.yAxis2Pen);
        plot->yAxis2->setSubTickPen(psettings.yAxis2Pen);
        plotChanged = true;
    }

    if (ui->yGridLSWidget->isDifferent(psettings.yGridPen)) {
        if (update) {
            psettings.yGridPen.setColor(ui->yGridLSWidget->getColor());
            psettings.yGridPen.setStyle(ui->yGridLSWidget->getStyle());
            psettings.yGridPen.setWidth(ui->yGridLSWidget->getWidth());
        }
        plot->yAxis->grid()->setPen(psettings.yGridPen);
        plotChanged = true;
    }

    if (ui->xGridLSWidget->isDifferent(psettings.xGridPen)) {
        if (update) {
            psettings.xGridPen.setColor(ui->xGridLSWidget->getColor());
            psettings.xGridPen.setStyle(ui->xGridLSWidget->getStyle());
            psettings.xGridPen.setWidth(ui->xGridLSWidget->getWidth());
        }
        plot->xAxis->grid()->setPen(psettings.xGridPen);
        plotChanged = true;
    }

    if (ui->legendCheckBox->isChecked() != psettings.legendVisible) {
        if (update) psettings.legendVisible = ui->legendCheckBox->isChecked();
        plot->legend->setVisible(psettings.legendVisible);
        plotChanged = true;
    }

    if (getAbscissaState() != psettings.getAbscissa()) {        
        if (update) {
            xpdutils::convertAbscissa(plot,waves,refl,psettings.getAbscissa(),getAbscissaState());
            psettings.setAbscissa(getAbscissaState());
        } else {
            xpdutils::convertAbscissa(plot,waves,refl,getAbscissaState(),psettings.getAbscissa());
        }
        plot->xAxis->setRangeReversed(psettings.getAbscissa() == DVALUE);

        plot->updateMinMax();
        plot->xAxis->rescale();
        plotChanged = true;        
    }

    if (mYScaleType != psettings.yScaleType) {
        if (update) psettings.yScaleType = mYScaleType;
        plot->yAxis->setScaleType(psettings.yScaleType);
        plot->yAxis->setLabel(psettings.getYLabel());
        plotChanged = true;
    }

    if (ui->xAxisTickLabelsCheckBox->isChecked() != psettings.xTickLabels) {
        if (update) psettings.xTickLabels = ui->xAxisTickLabelsCheckBox->isChecked();
        plot->xAxis->setTickLabels(psettings.xTickLabels);
        plotChanged = true;
    }

    if (ui->yAxisTickLabelsCheckBox->isChecked() != psettings.yTickLabels) {
        if (update) psettings.yTickLabels = ui->yAxisTickLabelsCheckBox->isChecked();
        plot->yAxis->setTickLabels(psettings.yTickLabels);
        plotChanged = true;
    }

    if (ui->yOffsetCheckBox->isChecked() != psettings.hasOffset) {
        if (update) {
            psettings.hasOffset = ui->yOffsetCheckBox->isChecked();
            psettings.yOffset = ui->yOffsetLineEdit->text().toDouble();
        }
        MainWindow *w = qobject_cast<MainWindow *>(parent());
        if (psettings.hasOffset) {
            w->applyOffset(psettings.yOffset);
        } else {
            w->applyOffset(); // equivalent to unapply the last offset
        }
        plotChanged = true;        
    } else {
        if (ui->yOffsetCheckBox->isChecked()) {
            double offSet = ui->yOffsetLineEdit->text().toDouble();
            if (offSet != psettings.yOffset) {
                if (update) {
                    psettings.yOffset = ui->yOffsetLineEdit->text().toDouble();
                }
                MainWindow *w = qobject_cast<MainWindow *>(parent());
                w->applyOffset(psettings.yOffset);
                plotChanged = true;
            }
        }
    }

    if (redrawRequired) {
        MainWindow *w = qobject_cast<MainWindow *>(parent());
        w->redrawPlot();
        changeApplied = true;
        plotChanged = false; // redrawPlot executes replot
    }

    if (plotChanged) {
        plot->replot();
        changeApplied = true;
    }
}

void PlotStyleDialog::cancel(CustomPlotZoom *plot, QVector<graphItem> &obs, graphItem &back, graphItem &bpoints, graphItem &calc,
                             graphItem &diff, graphItem &cdiff, graphItem &peaks, QVector<graphItem> &refl, PlotSettings &psettings,
                             const QVector<double> waves)
{
    if (anyChangeApplied()) {
        for (int i = 0; i < obs.count(); i++) {
            obs[i].setPen(obsPen0[i]);
            obs[i].setScatter(obsScatter0[i]);
            obs[i].setVisible(obsVisible0.at(i));
        }
        psettings = settings0;
        back.setPen(backPen0);
        back.setVisible(backVisible0);
        bpoints.setScatter(backScatter0);
        bpoints.setVisible(backpVisible0);
        calc.setPen(calcPen0);
        calc.setVisible(calcVisible0);
        diff.setPen(diffPen0);
        diff.setVisible(diffVisible0);
        cdiff.setPen(cDiffPen0);
        cdiff.setVisible(cDiffVisible0);
        peaks.setPen(peaksPen0);
        peaks.setVisible(peaksVisible0);
        for (int i = 0; i < refl.count(); i++) {
            refl[i].setPen(reflPen0.at(i));
            refl[i].setVisible(reflVisible0.at(i));
        }
        apply(plot, obs, back, bpoints, calc, diff, cdiff, peaks, refl, psettings, waves, false);
    }
}

bool PlotStyleDialog::anyChangeToApply() const
{    
    return ui->buttonBox->button(QDialogButtonBox::Apply)->isEnabled();
}

bool PlotStyleDialog::anyChangeApplied() const
{    
    return changeApplied;
}

void PlotStyleDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Apply) {
        emit dialogClosed(stdButton);
        //setApplied();
        ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
    } else if (stdButton == QDialogButtonBox::Ok) {
        emit dialogClosed(stdButton);

    } else if (stdButton == QDialogButtonBox::RestoreDefaults) {
        emit dialogClosed(stdButton);
        enableApplyButton();
    }
}

void PlotStyleDialog::reject()
{
    emit dialogClosed(QDialogButtonBox::Cancel);
    QDialog::reject();
}

void PlotStyleDialog::on_observedComboBox_currentIndexChanged(int index)
{
    if (index < 0) return; // signal emitted for clear combo
    setObserved(index);
}

void PlotStyleDialog::on_reflComboBox_currentIndexChanged(int index)
{
    if (index < 0) return; // signal emitted for clear combo
    setReflections(index);
}

void PlotStyleDialog::enableApplyButton()
{
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
}

void PlotStyleDialog::connectLineWidget(LineStyleWidget *lsWidget)
{
    connect(lsWidget, &LineStyleWidget::lineColorChanged, this, &PlotStyleDialog::enableApplyButton);

    connect(lsWidget, &LineStyleWidget::lineWidthChanged, this, &PlotStyleDialog::enableApplyButton);

    connect(lsWidget, &LineStyleWidget::lineStyleChanged, this, &PlotStyleDialog::enableApplyButton);
}

void PlotStyleDialog::setObserved(int index)
{
    ui->observedLineWidget->setLineStyle(obsPen.at(index));
    ui->observedMarkerWidget->setMarkerStyle(obsScatter.at(index));
    ui->obsCheckBox->setChecked(obsVisible.at(index));
}

void PlotStyleDialog::setReflections(int index)
{
    ui->reflCheckBox->setChecked(reflVisible.at(index));
    ui->reflectionsLineWidget->setLineStyle(reflPen.at(index));
}

xpdutils::xAbscissaType PlotStyleDialog::getAbscissaState() const
{
    if (ui->tthetaRadioButton->isChecked())
        return xpdutils::TTHETA;
    else if (ui->dRadioButton->isChecked())
        return xpdutils::DVALUE;
    else if (ui->oneOverDRadioButton->isChecked())
        return xpdutils::ONE_OVER_DVALUE;
    else
        return xpdutils::ONE_OVER_DVALUE2;
}
