#ifndef POLYHEDRADIALOG_H
#define POLYHEDRADIALOG_H

#include <QDialog>
#include <QStringListModel>
#include <QAbstractButton>
#include <QComboBox>
#include "table.h"

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif

namespace Ui {
class PolyhedraDialog;
}

class PolyhedraDialog : public QDialog
{
    Q_OBJECT

public:
    explicit PolyhedraDialog(QWidget *parent = nullptr);
    ~PolyhedraDialog();
    void set_polyhedron_type(int geom, bool add);
    enum geoms {TETRA, OCTA, SQUARE,
                 CUBE, TRIG, APRISM_TETRA,
                 PRISM_TRIG, ICOSA, ISOLATED};

private slots:
    void buttonBoxAccepted();
    void buttonBoxRejected();
    void radioRandomClicked();
    void radioCenteredClicked();
    void polyhedraComboActivated(int index);
    void originComboActivated(int index);
    void verticesComboActivated(int index);

private:    
    bool add;
    int nspec;
    int lastOrigin;
    int lastVertex;
    QStringListModel *modelspec;
    QStringListModel *modelatom;
    TableWindow *mTableWindow;
    Ui::PolyhedraDialog *ui;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif
    void set_distance();
    void set_visible_widgets(int geom, bool add);
    int add_element_to_modelspec(QString newel);
    bool id_specie_from_combo(const QComboBox *combo, int &id);

};

#endif // POLYHEDRADIALOG_H
