#include "powderpattern.h"
#include "ui_powderpattern.h"
#include "javkeysettings.h"
#include "savedialog.h"
#include "qt_utils.h"
#include <QResizeEvent>
#include <QStatusBar>
#include <QMessageBox>

extern "C" void jav_export_powder_pattern_(char *f1, int *tipo, size_t lung);

PowderPattern *PdpDialog = nullptr;

typedef struct {
                  float v;
                  int p;
               } vord;

QString PowderPattern::MessForSB(QPointF p)
{
    return PdpDialog->StatusMessage(p);
}

bool sort_vect ( vord elem1, vord elem2 )
{
   float v1 = elem1.v;
   float v2 = elem2.v;
   return (v1 < v2);
}

int PowderPattern::Myminloc(float *vect, float val)
{
    vector<vord> v1;
    if(nref <= 0)
       return -1;
    for(int i=0; i<nref; i++)
    {
        vord elem;
        elem.v = abs(vect[i]-val);
        elem.p = i;
        v1.push_back(elem);
    }
    sort(v1.begin(), v1.end(), sort_vect);
    int result = v1.at(0).p;
    v1.clear();
    return result;
}

QString PowderPattern::StatusMessage(QPointF p)
{
    QString stringa;
    float due_theta = p.x(), inte = p.y();

    int k = Myminloc(tthd, due_theta);
    if ((k >= 0) && (k < nref))
        stringa = QString("2-Theta: %1      I: %2      h k l :%3, %4, %5")
                              .arg(due_theta).arg(inte).arg(hkl[k].h)
                .arg(hkl[k].k).arg(hkl[k].l);
    else
        stringa = QString("2-Theta: %.1      I: %1").arg(due_theta).arg(inte);
    return stringa;
}

PowderPattern::PowderPattern(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PowderPattern),
    m_xpd({1, {1.54056f, 1.5444274f}, {1, 0.5f}, 5, 70, 0.02f, 0.1f, 1000.0f}),
    m_Scene(nullptr),
    tthd(nullptr),
    hkl(nullptr),
    nref(0),
    mPdPsett(nullptr)
{
    ui->setupUi(this);
    QStatusBar *bar = new QStatusBar(this);
    bar->setSizeGripEnabled(false);
    ui->verticalLayout->insertWidget(1,bar);
    ui->resetButton->setEnabled(false);
    PdpDialog = this;
    //ui->graphicsView->setRenderHints(QPainter::Antialiasing);
    if(m_Scene != nullptr)
       delete m_Scene;
    m_Scene = new Grafico(bar, ui->resetButton, this);
    m_Scene->SetFunc(PowderPattern::MessForSB);
}

void PowderPattern::resizeEvent(QResizeEvent *)
{
    QRectF rect1(ui->graphicsView->rect());
    m_Scene->sizeChanged(rect1.width(), rect1.height());
}

PowderPattern::~PowderPattern()
{
    delete ui;
}

void PowderPattern::FillPowderPatternDlg(int whichModel, QString filename)
{
    m_Kmodel = whichModel;
    m_FileName = filename;
    jav_powder_pattern_(&whichModel, &m_xpd);
}

void PowderPattern::DoPowderPattern(float x[], float y[], int num,
                        reflection_type *refjav, int num1)
{
using namespace GraphicSpace;
    m_Scene->ClearScene();
    m_Scene->SetArea(ui->graphicsView->width()-10, ui->graphicsView->height()-10);
    m_Scene->setSceneRect(ui->graphicsView->rect());
    m_Scene->SetLabels("2 theta", "Intensity");
    m_Scene->AddPlot(x, y, num, m_xpd.ttmin, m_xpd.ttmax);
    if(num1)
    {
       if(tthd != nullptr)
          delete [] tthd;
       if(hkl != nullptr)
          delete [] hkl;
       tthd = new float [num1]();
       hkl  = new _HKL [num1]();
       for(int i=0; i<num1; i++)
       {
           tthd[i] = refjav[i].tthd[0];
           hkl[i].h = refjav[i].hkl[0];
           hkl[i].k = refjav[i].hkl[1];
           hkl[i].l = refjav[i].hkl[2];
       }
       m_Scene->AddPlot(tthd,NULL, num1, m_xpd.ttmin, m_xpd.ttmax);
       nref = num1;
    }

    m_Scene->SetArea(ui->graphicsView->width(), ui->graphicsView->height());
    m_Scene->Aggiorna();
    ui->graphicsView->setScene(m_Scene);
}

void PowderPattern::on_exportButton_clicked()
{
    QSettings settings;
    QString selectedFilter = "ASCII powder pattern(*.xy)";
    QString fileName = SaveDialog::run(this, tr("Export pattern as"),
                                    settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(m_FileName).baseName(),
                                       "ASCII powder pattern(*.xy);;"
                                       "Reflection list(*.hkl)",
                                       "xy", selectedFilter);
    if (!fileName.isEmpty()) {
         QFile file(fileName);
         if (!file.open(QFile::WriteOnly | QFile::Text)) {
             QMessageBox::warning(this, "Warning", "Cannot save file: " + file.errorString());
             return;
         }
         file.close();
         int tipo;
         if(selectedFilter.contains("powder"))
            tipo = 1;
         else
            tipo = 2;
         char *f1 = ToCharacter(QFileInfo(fileName).absoluteFilePath());
         jav_export_powder_pattern_(f1, &tipo, strlen(f1));
    }

}

void PowderPattern::on_settingsButton_clicked()
{
    if (!mPdPsett)
        mPdPsett = new PowderPatternSettings(this);
    mPdPsett->FillForm(m_Kmodel, &m_xpd);
    mPdPsett->show();
}

void PowderPattern::on_resetButton_clicked()
{
    m_Scene->ResetZoom();
    ui->resetButton->setEnabled(false);
}

void PowderPattern::on_closeButton_clicked()
{
   reject();
}

extern "C" void do_powder_pattern_(float x[], float y[], int *num,
                        reflection_type *refjav, int *num1)
{
    PdpDialog->DoPowderPattern(x, y, *num, refjav, *num1);
}
