MODULE exploreset

   USE progtype
   USE model_util, only: model_type
   USE timer
   USE iso_c_binding

   implicit none
   
   type(model_type), dimension(:), allocatable :: modelset
   integer, parameter :: MODEL_SETNR = 1
   integer, parameter :: MODEL_RF = 2
   integer, parameter :: MODEL_DONE = 5
   integer, parameter :: MODEL_CFOM = 6
   
   !
   !  variabili per gestione grafica
   integer :: kprorest   ! il tipo di procedura (RBM,Fourier,E-map)
   integer :: lastsel    ! l'ultimo modello selezionato dalla lista
   logical :: more_trials = .false.
   
   type(timer_type), dimension(4), private :: timerst
   integer, parameter, private             :: TIMERST_FOUR=1, TIMERST_RBM=2, TIMERST_TRIAL=3, TIMERST_TOT=4
   character(len=*), private, parameter    :: STRTAG='@@'
   
   integer, parameter :: AUTO_PROC=0, RBM_PROC=1, FOUR_PROC=2, EMAP_PROC=3, COVM_PROC=4, EDM_PROC=5, EDME_PROC=6

   type, bind(C) :: trial_info_type 
      integer(c_int) :: set
      real(c_float)  :: cfom
      integer(c_int) :: done
      real(c_float)  :: rf
   end type trial_info_type 

CONTAINS

   subroutine init_restart()
   USE model_util
!
   call clear_models(modelset)
   kprorest = AUTO_PROC
   lastsel = 0
!
   end subroutine init_restart

!-------------------------------------------------------------------------------------------------

   subroutine explore_trials(kprocedure,ivtrial,ncyclet,rord)
   USE variables, only:atm
   USE Atomi
   USE atom_type_util
   USE variables, only:ref,lconn,cryst
   USE General, only:knw,fname,OUT_FILE,cell,spacegr, &
                     celem,ERR_WINDOW,lo,neutro,is_anomalous
   USE Molcom, only:ModelToCompare,jscreen
   USE compare_struct
   USE TestModel, only:vett_test
   USE atom_type_util
   USE molpnew
   USE drawatoms
   USE messagemod
   USE model_util
   USE unit_cell
   USE reflection_type_util
   USE strutil
   USE sinterface, only: update_model
   USE enable_amb   !, only: STOP_EVENT
   integer, intent(in)               :: kprocedure
   integer, dimension(:), intent(in) :: ivtrial ! =1 per i trials da explorare
   integer, intent(in)               :: ncyclet ! numero di cicli di fourier
   logical, intent(in)               :: rord
   integer                           :: i
   integer                           :: ntrial
   logical                           :: print_tb
   real                              :: timerun
   character(len=1)                  :: ap = '"'
   character(len=2)                  :: apv = '",'
   real, dimension(5)                :: infomodel
   real                              :: rf
!corr   integer                           :: ipos
   real                              :: mtrov
   integer                           :: kevent
   integer, dimension(size(ivtrial)) :: veterr  
   integer                           :: nerrt
   character(len=:), allocatable     :: sterr
   integer                           :: nExitCode
   real(DP)                          :: timetot
!
   call abilita_tasti(SaveAction)
   call abilita_tasti(DialogOpenAction)
   call abilita_tasti(RunSkipAction,1)
!
   call timerst(TIMERST_FOUR)%set_type('Time for Fourier/RBM')
   call timerst(TIMERST_RBM)%set_type('Time for RBM')
   call timerst(TIMERST_TRIAL)%set_type('Time for trial')
   call timerst(TIMERST_TOT)%set_type('Total time')
   more_trials = .true.
   call timerst(TIMERST_TOT)%tic()
!
#if defined(DISTRIB)
   print_tb = .false.
#else
   print_tb = .true.
#endif
   ntrial = size(ivtrial)
   veterr(:) = 0
!
   if(print_tb)write(6,'(a)')STRTAG//'            NTRIAL      RF           found      dist'
   do i=1,ntrial
      if (ivtrial(i) == 1) then
          call timerst(TIMERST_TRIAL)%tic()
          call timerst(TIMERST_FOUR)%tic()
          call explore_single_trial(kprocedure,i,ncyclet,kevent,veterr(i))
          call timerst(TIMERST_FOUR)%toc()
          if (veterr(i) /= 0) cycle
!
          call trial_to_listset(i)  ! ,ipos)
!
!         Tempo per ogni run
          call timerst(TIMERST_TRIAL)%toc()
          timerun = timerst(TIMERST_TRIAL)%get_interval()
!
!         Stampa informazioni
          !rf = modelset(ipos)%info(2)
          !modelset(ipos)%info(MODEL_DONE) = 1
          rf = modelset(i)%info(2)
          modelset(i)%info(MODEL_DONE) = 1
          if (knw == 1 .or. ModelToCompare /= 0) then
              write(0,10)i,rf,secs_to_string(timerun),int(vett_test(3)),int(vett_test(1)),vett_test(4)
              write(6,10)i,rf,secs_to_string(timerun),int(vett_test(3)),int(vett_test(1)),vett_test(4)
          else
              write(0,20)i,rf,secs_to_string(timerun)
              write(6,20)i,rf,secs_to_string(timerun)
          endif
10        format('Set:',i0,'  RF=',f0.4,'   Time:',a,'  found:',i3,'/',i0,'  dist:',f0.4)
20        format('Set:',i0,'  RF=',f0.4,'   Time:',a)
!
          !ivtrial(i) = 0  ! azzera ivtrial per i trial gia' sviluppati
          if (kevent == STOP_EVENT) exit   ! evento di stop dalla grafica
!
      endif
   enddo
!
!  Stampa i tempi
   call timerst(TIMERST_TOT)%toc()
   call print_time(timerst,lo)
!
!  Ordina e stampa la lista dei modelli
   if (rord) call model_sort(modelset,keycode=2,keyadd=MODEL_DONE)
   if (print_tb) then
       call model_print_info(modelset,4,iveti=(/1,3/),mark=STRTAG)
   else
       call model_print_info(modelset,4,iveti=(/1,3/))
   endif
!
!  Seleziona il modello migliore 
!qt   call model_get(modelset(1),atm,fom=infomodel)
   call model_get_best(modelset,2,atm,infomodel,keyadd=MODEL_DONE)
   call create_connectivity(atm,lconn,set_cell_type(cell),spacegr,check=.true.)
   call update_model(atm,lconn,1)
!
!  Stampa info sul modello migliore
   vett_test(3) = infomodel(3)
   vett_test(4) = infomodel(4)
   if (print_tb) then
       mtrov = sum(modelset(:)%info(3)) / nummodels(modelset)
       timetot = timerst(TIMERST_TOT)%get_time()
       write(6,"(a,'best ','found:',i0,'/',i0,' dist:',f0.4,' RF:',f0.6,'  time:',a,' media trov:',f0.2)")    &
                        STRTAG,int(vett_test(3)),int(vett_test(1)),vett_test(4), &
                         infomodel(2),secs_to_string(timetot),mtrov
       write(6,'(a,a,a,a,a,i0,a,a,f0.4,a,a,i0,a,i0,a,a,f0.3,a,a,a,a)')'TB2',     &
                       ap,trim(fname(OUT_FILE)(scan(fname(OUT_FILE),'/\',back=.true.)+1:)),apv,  &
                       ap,int(infomodel(1)),apv,                                     & 
                       ap,infomodel(2),apv,                                          & 
                       ap,int(vett_test(3)),'/',int(vett_test(1)),apv,               &
                       ap,vett_test(4),apv,                                          &
                       ap,secs_to_string(timetot),ap
              
   endif
   write(6,'(/a,i0,1x,a)')' The best model was found on trial n.',int(infomodel(1))
   call print_atoms(atm,title='Structure solution',write_occ=.true.)
!
!  Disegna il modello migliore
   if (jscreen > 0) then
       call draw_atoms(atm,lconn)
!
!      Check for error in restart
       nerrt = count(veterr /= 0) 
       if (nerrt > 0) then
           sterr = 'Error at trial'
           if (nerrt > 1) sterr = sterr//'s'
           do i=1,size(veterr)
              if (veterr(i) /= 0) then
                  sterr = sterr//' '//trim(i_to_s(i))
              endif
           enddo
           call MsgWin2(sterr,ERR_WINDOW,nExitCode,5000)
       endif
   endif
!
   if (knw == 1 .or. ModelToCompare /= 0) then
       call fcalcang(ref,atm,spacegr,celem,neutro,is_anomalous)
       call to_coord_sir(0,atm,ref,size(ref),set_cell_type(cell),spacegr,lo,cryst(1)%atpub)
       call write_message_found(4,1)
   endif
   more_trials = .false.
!
   call abilita_tasti(RunSkipAction,0)
   call abilita_tasti(RestoreAction)
   call abilita_tasti(PatternAndStructureAction)
!
   end subroutine explore_trials

! -----------------------------------------------------------------

   subroutine explore_single_trial(kprocedure,ntrial,ncyclefou,kevent,ier)
   USE General, only: lo,celem
   !USE comandiexpo
   USE variables
   USE atomi
   USE atom_type_util
   USE sinterface, only: fillatom
   implicit none
   common /graphic_restart/kgraph_restart,ncyc_restart,nset_restart
   integer, intent(in)  :: kprocedure
   integer, intent(in)  :: ntrial,ncyclefou
   integer, intent(out) :: kevent
   integer, intent(out) :: ier
   integer              :: kgraph_restart,ncyc_restart,nset_restart
!
   write(lo,10) ntrial,ncyclefou
10 format(///,' ',78('+'),//,                         &
              ' Fourier for trial no.',i5,            &
              ' with ',i5,' cycles',//)
!
   ier=0
   kgraph_restart = 1
   ncyc_restart = ncyclefou
   nset_restart = ntrial
   !icomat = icomat + 1
   call fourie(ier,kprocedure,kevent)
   if (kprocedure == EMAP_PROC) then
!
!      rimuovi ghost solo per kprocedure=3 (E-MAP)
       call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
       call delete_ghosts(atm) 
       nats = size(atm)
       call fillatom(atm)
       call frm15(nats)
   endif
   kgraph_restart = 0
   !icomat = icomat - 1
   if (ier == 0)  then
       call lastplot()  
   else
      ! if (jscreen > 0) then
      !     call MsgWin2('Error in restart',ERR_WINDOW,nExitCode,5000)
      ! endif
       write(lo,'(1x,62("*"),/," *",22x,"Error in restart",22x,"*",/,1x,62("*"))')
   endif
!
   end subroutine explore_single_trial

!-------------------------------------------------------------------------------------------------

   subroutine trial_to_listset(nums)   !,ipos)
   USE Atomi
   USE stat
   USE atom_type_util
   USE variables
   USE TestModel, only:vett_test
   USE compare_struct
   USE General, only:knw,spacegr,celem,cell,lo,neutro,is_anomalous
   USE Dialogo, only:ModelToCompare
   USE model_util
   USE reflection_type_util
   USE unit_cell
   integer, intent(in)            :: nums   ! numero del set da aggiungere
!corr   integer, optional, intent(out) :: ipos   ! in che posizione e' stato aggiunto
   real, dimension(5)             :: info
!corr   integer                        :: pos
!
!  Calcola Fc e to_coord
   call xyz_to_atom(atm,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,inte=xyzs(4,:),elem=celem)
   call fcalcang(ref,atm,spacegr,celem,neutro,is_anomalous)
   if (knw == 1 .or. ModelToCompare /= 0) then
       call to_coord_sir(0,atm,ref,size(ref),set_cell_type(cell),spacegr,lo,cryst(1)%atpub)
   endif
!
!  Salva il modello nella lista
   info(1) = nums                   ! numero del set
   info(2) = Rvalue(ref%fo,ref%fc)  ! RF
   info(3) = vett_test(3)           ! trovati
   info(4) = vett_test(4)           ! distanza
!   pos = trial_to_model(nums)
!   if (pos == 0) pos = nums
!   call model_set(modelset(pos),atm,fom=info(1:4))
   call model_set(modelset(nums),atm,fom=info(1:4))
!   if (present(ipos)) ipos = pos
!
   end subroutine trial_to_listset

!-------------------------------------------------------------------------------------------------

   subroutine explore_all_trials()
   USE PhaseModule, only:nset,imove
   USE reflection_type_util
   USE strutil
   USE commandsmod
   USE exportfiles
   integer, dimension(:), allocatable :: ivtrial
   integer                            :: ncy,i
   integer                            :: ier
   integer                            :: posc
   integer                            :: kproc
!
   call restart_prelim(ier)
   if (ier == 0) then
       ncy = 20 
       kproc = 0
       posc = command_position(commands,'alltrials')
       if (posc > 0) then
           do i=1,commands(posc)%ndir  ! alltrials can accept more than one directive 
               select case (upper(commands(posc)%strdir(i)(1:3)))
                  case ('EMA','DMA')
                    kproc = EMAP_PROC
                    kprorest = 1
                    ncy = 0
                  case ('FOU')
                    kproc = FOUR_PROC
                  case ('RBM')
                    kproc = RBM_PROC
                  case ('COV')     ! covmap
                    kproc = COVM_PROC
                  case ('SHI')     ! shift
                    imove = 1
               end select
           enddo
           if (is_directive(commands(posc),'edmo') > 0) then
               if (kproc == EMAP_PROC) then
                   kproc = EDME_PROC  ! EDM+EMAP
               else
                   kproc = EDM_PROC   ! EDM
               endif
           endif
       endif
       allocate(ivtrial(nset))
       ivtrial(:) = 1
       !call explore_trials(kproc,ivtrial,ncy,rord=.true.)
       call explore_trials(kproc,ivtrial,ncy,rord=.false.)
       call export_files()             ! aggiorna file esterno
   endif
!
   end subroutine explore_all_trials

! -----------------------------------------------------------------
!corr
!corr   integer function trial_to_model(itrial) result(kmodel)
!corr   USE model_util
!corr   integer, intent(in) :: itrial
!corr   !integer             :: i
!corr!
!corr   !kmodel = 0
!corr   !do i=1,nummodels(modelset)
!corr   !   if (modelset(i)%info(1) == itrial) then
!corr   !       kmodel = i; exit
!corr   !   endif
!corr   !enddo
!corr   kmodel = find_model(modelset,nummodels(modelset),1,itrial)
!corr!
!corr   end function trial_to_model
!corr
! ----------------------------------------------------------------------

   elemental function cast_to_trial_info(model) result(c_trial)
   use model_util
   type(model_type), intent(in) :: model
   type(trial_info_type)        :: c_trial

   c_trial%set = int(model%info(MODEL_SETNR),c_int)
   c_trial%cfom = real(model%info(MODEL_CFOM),c_float)
   c_trial%done = int(model%info(MODEL_DONE),c_int)
   c_trial%rf = real(model%info(MODEL_RF),c_float)

   end function cast_to_trial_info

! ----------------------------------------------------------------------

   subroutine restart_dm() bind(C,name="restart_dm")
   USE PhaseModule, only: imove, nset
   interface
      subroutine open_restart_dlg(kproc,saf,nset,selset,trialinfo) bind(C,name="open_restart_dlg")
      use iso_c_binding
      type, bind(C) :: trial_info_type 
         integer(c_int) :: set
         real(c_float)  :: cfom
         integer(c_int) :: done
         real(c_float)  :: rf
      end type trial_info_type 
      integer(c_int), intent(in), value   :: kproc
      integer(c_int), intent(in), value   :: saf
      integer(c_int), intent(in), value   :: nset
      integer(c_int), intent(in), value   :: selset
      type(trial_info_type), dimension(*) :: trialinfo
      end subroutine open_restart_dlg
   end interface
!
   integer :: ier
   integer :: kproc
   integer :: ksaf
   integer :: selset
!
   call restart_prelim(ier)
   if (ier == 0) then
       kproc = kprorest
       ksaf = imove
       selset = minloc(modelset(:)%info(MODEL_RF),dim=1,mask=modelset(:)%info(MODEL_DONE)==1)
       call open_restart_dlg(kproc,ksaf,nset,selset,cast_to_trial_info(modelset))
   endif
!
   end subroutine restart_dm

! -----------------------------------------------------------------

   subroutine restart_prelim(ier)
   USE Molcom
   USE General
   USE PhaseModule
   USE variables, only:ref
   USE proginterface
   USE Atomi, only:nats
   USE model_util
   integer, intent(out) :: ier
   real, dimension(302)               :: cfom
   integer, dimension(302)            :: ipos
   real, dimension(11)                :: fom
   real, dimension(2000)              :: ep
   integer, dimension(nrefl)          :: iphb
   integer                            :: i,j
   integer                            :: lsn,lfw,llw,lpb,ll16
   integer                            :: nlastset
   !integer :: kfind_spg
!
   call srlin (16,lsn,lfw,llw,lpb,ll16)
   if (lfw.gt.0) then
       ier = 0
!
       call fillref(ref)
!
       call ysfl16(cfom,ipos,nset)  ! leggi il numero di set
!
!      Individua quali set sono stati eseguiti
       if (.not.allocated(modelset)) then
!
!          se la lista e vuota riempi infoset dalla lista 16
           call resize_models(modelset,nset)
           do j=1,nset
              i = ipos(j)
              call snr16(i,fom,ep,iphb)
              modelset(j)%info(MODEL_DONE) = 0
              modelset(j)%info(1) = j
              modelset(j)%info(MODEL_CFOM) = fom(5)
              modelset(j)%info(2) = 0
           enddo
!           
!          aggiungi l'ultimo modello se esiste, cioe' se ho fatto la direttiva fourier
           if (nats > 0 .and. numset > 0) then
               call trial_to_listset(numset)
               modelset(numset)%info(MODEL_DONE) = 1
           endif
       else
          if (lastsel > 0) then
!              
!             aggiorna l'ultimo modello selezionato perche' potrebbe essere stato modificato
              nlastset = int(modelset(lastsel)%info(1))   ! prendi il set dell'ultima selezione
              call trial_to_listset(nlastset)                   ! aggiorna il set
          endif
       endif
!
   else
       call OpenWarning('Cannot restart. No trials stored')
       nset = nummodels(modelset)  ! > 0 in case of models from binary file .expo
       if (nset == 0) ier = -1     ! Set error only in case of no models stored
         !kfind_spg=0; call cell_spg_dialog(kfind_spg)
         !call restart_command()
   endif
!
   end subroutine restart_prelim

! -----------------------------------------------------------------

   subroutine TrialSelected(selrow) bind(C,name="TrialSelected")
   !USE exploreset
   USE Atomi
   USE molpnew
   USE variables
   USE atom_type_util
   USE drawatoms
   USE messagemod
   USE general, only:knw, cell, spacegr
   USE TestModel, only:vett_test
   USE strutil
   USE Molcom, only:ModelToCompare
   USE model_util
   USE exportfiles
   USE connect_mod
   USE Phasemodule, only: numset
   USE unit_cell
   USE sinterface, only: update_model
   integer(c_int), value, intent(in) :: selrow
   real, dimension(5)                :: infomodel
   integer, dimension(1)             :: best_model
!
   if (modelset(selrow)%info(MODEL_DONE) == 1) then
       call model_get(modelset(selrow),atm,fom=infomodel)
!
!      Crea connettivita' e disegna
       call create_connectivity(atm,lconn,set_cell_type(cell),spacegr,check=.true.,kpr=0)
       call update_model(atm,lconn,1)
       call draw_atoms(atm,lconn)
       call bond_to_iconn(lconn,iconn) ! aggiorna iconn
       call export_files()             ! aggiorna file esterno
!
       best_model = minloc(modelset(:)%info(MODEL_RF),dim=1,mask=modelset(:)%info(MODEL_DONE)==1)
       call clear_messages(ipos=2)
       if (selrow == best_model(1)) then
           call write_message('The best structure was selected. Trial n:'//trim(i_to_s(int(infomodel(1)))),pos=2,kproc=0)
       else
           call write_message('The structure of trial n:'//trim(i_to_s(int(infomodel(1)))),pos=2,kproc=0)
       endif
       call write_message('RF:',rnum=infomodel(2),pos=3,kproc=0)
       lastsel = selrow       ! memorizza l'ultimo modello selezionato
       numset = nint(infomodel(1))  ! aggiorna il puntatore al set corrente
       if (knw > 0 .or. ModelToCompare /= 0) then
           vett_test(3) = infomodel(3)           ! trovati
           vett_test(4) = infomodel(4)           ! distanza
           call write_message_found(4,1)
       endif
   endif
!
   end subroutine TrialSelected

! -----------------------------------------------------------------

   subroutine c_finish_restart(kproc,ksaf,ivtrial,c_trial,selset) bind(C, name="c_finish_restart")
   use iso_c_binding
   use PhaseModule, only: nset
   integer(c_int), intent(in), value   :: kproc
   integer(c_int), intent(in), value   :: ksaf
   integer(c_int), dimension(*)        :: ivtrial
   type(trial_info_type), dimension(*) :: c_trial
   integer(c_int)                      :: selset
   integer, dimension(nset) :: ivset,ivdone
   real, dimension(nset)    :: rfvalue,cfomvalue
!
   call finishrestart(ivtrial, kproc, ivset, cfomvalue, ivdone, rfvalue, selset, ksaf)
   c_trial(:nset) = cast_to_trial_info(modelset)
!
   end subroutine c_finish_restart
   
! -----------------------------------------------------------------

   subroutine finishrestart(ivtrial, kproc, ivset, cfomvalue, ivdone, rfvalue, selset, ksaf)
   USE PhaseModule, only:nset,imove
   !USE exploreset
   USE commandsmod
   !implicit none
   integer, intent(in)                    :: kproc            
   integer, intent(in)                    :: ksaf             ! Shift_and_fix check
   integer,  dimension(nset), intent(out) :: ivtrial          ! righe con i trial da sviluppare
   integer,  dimension(nset), intent(out) :: ivset
   real, dimension(nset), intent(out)     :: cfomvalue
   integer, dimension(nset), intent(out)  :: ivdone           ! quali trial gia' sviluppati
   real, dimension(nset)                  :: rfvalue          ! RF per trial già sviluppati
   integer                                :: ncyclC       
   !integer                                :: iset
   integer                                :: i
   integer                                :: selset
   !integer                                :: ndevset
!
   select case (kproc)
       !case (1,2,4)    ! RBM,Fourier,RBM-CONN
       case (RBM_PROC,FOUR_PROC,COVM_PROC)    ! RBM,Fourier,RBM-CONN
          ncyclC = 20
       case (EMAP_PROC)        ! E-map
          ncyclC = 0
   end select 
   imove = ksaf
!
   !ivtrial(int(modelset(:)%info(1))) = ivtrial(:)     ! perche' ivtrial punta alle righe della gtklist
!
!corr   if (is_directive_cmd(commands,'dmap')) then   ! non ordinare i modelli se attiva la direttiva dmap
!corr       call explore_trials(kproc,ivtrial,ncyclC,rord=.false.)
!corr   else
!corr       call explore_trials(kproc,ivtrial,ncyclC,rord=.true.)
!corr   endif
       call explore_trials(kproc,ivtrial,ncyclC,rord=.false.)
!
   do i=1,nset
      ivset(i) = nint(modelset(i)%info(1))
      ivdone(i) = nint(modelset(i)%info(MODEL_DONE))
      rfvalue(i) = modelset(i)%info(MODEL_RF)
      cfomvalue(i) = modelset(i)%info(MODEL_CFOM)
   enddo
!
!  Quale set selezionare sulla gtklist
!corr   ndevset = count(ivtrial(:) == 1)      ! num di set. sviluppati
!corr   if (ndevset == 1) then  ! un solo set sviluppato
!corr       selset = 0
!corr       do i=1,nset        ! cerca il set sviluppato
!corr          if (ivtrial(i) == 1) then
!corr              selset = i
!corr              exit
!corr          endif
!corr       enddo
!corr       !do i=1,nset        ! seleziona il set sviluppato
!corr       !   if (ivset(i) == iset) then
!corr       !       selset = i
!corr       !   endif
!corr       !enddo
!corr       !write(0,*)'iset=',iset
!corr   else                   ! piu' set sviluppati
!corr       selset = 1         ! selezione sul set migliore
!corr   endif
!
!  Select the best trial
   selset = minloc(modelset(:)%info(MODEL_RF),dim=1,mask=modelset(:)%info(MODEL_DONE)==1)
   ivtrial(:) = 0
!
   end subroutine finishrestart

! -----------------------------------------------------------------

   subroutine quit_restart(kproc,ksaf) bind(C,name="quit_restart")
!
!  Button 'Close' in the dialog 'Explore trials'
!
   USE PhaseModule, only: imove
   USE iso_c_binding
   implicit none
   integer(c_int), intent(in), value :: kproc,ksaf
!
   imove = ksaf
   kprorest = kproc
!
   end subroutine quit_restart

! -----------------------------------------------------------------

   subroutine finish_dm(c_cell_cont, len_cont, c_space, len_space, c_cell, alltrials) bind(C,name="finish_dm")
   use iso_c_binding, only: c_char, c_int
   use strutil
   use commandsmod
   use General, only: POW_FILE, card
   use comandiexpo
   use datamod, only: update_content, update_cell, gener
   use errormod
   use variables, only: dataset
   character(c_char), intent(in)     :: c_cell_cont(*)
   integer(c_int), intent(in), value :: len_cont
   character(c_char), intent(in)     :: c_space(*)
   integer(c_int), intent(in), value :: len_space
   real(c_double), dimension(*)      :: c_cell
   integer(c_int), intent(in), value :: alltrials
   character(len=:), allocatable     :: cell_cont, space
   integer                           :: ip, ier, iEnd
   type(error_type)                  :: err
   logical                           :: spg_updated,cell_updated
   integer                           :: iflagref
!
!  Check space group
   space = toFortranString(c_space,len_space)
   call update_spaceg(space,spg_updated,err)
   if (err%signal) go to 10
!
!  Check cell parameters
   call update_cell(real(c_cell(1:6)),cell_updated)
   if (spg_updated .or. cell_updated) then
       call dataset_to_thetaint(dataset(1))   ! extra change data: restore from dataset
       call gener(dataset(1)%wave,iflagref,.false.)
       call data3(POW_FILE,ier)
   endif
!
!  Check content
   cell_cont = toFortranString(c_cell_cont,len_cont)
   call update_content(cell_cont,ier)
   if (ier /= 0) return
!
   ip = command_position(commands,'data',back=.true.)
   call add_continue(commands,ip)
   if (alltrials == 1) then
       call replace_command(commands,FOUR_CMD,ALLTRIALS_CMD,last=.true.)
   endif
   call print_commands(commands,6)
!
   call create_expo_commands(commands,ipcom,icomq,card)
   icomat = ip
   ipdir = sum(commands(:ip)%ndir)
!
   call gescom(POW_FILE,iEnd,ier)
   return

10 if (err%signal) call err%print()
!
   end subroutine finish_dm

! -----------------------------------------------------------------
!corr
!corr   subroutine init_dm(spg_index, c_space, c_cell) bind(C,name="init_dm")
!corr   use iso_c_binding
!corr   use variables, only: cryst
!corr   use strutil
!corr   use spginfom
!corr   use crystal_phase
!corr   integer(c_int), intent(out)                  :: spg_index
!corr   character(c_char), dimension(*), intent(out) :: c_space
!corr   real(c_double), dimension(6)                 :: c_cell
!corr   integer                                      :: lens
!corr   
!corr   spg_index = cryst(1)%spg%get_id()
!corr   if (spg_index < 0) then
!corr       c_space(:1) = c_null_char
!corr   else
!corr       lens = len_trim(cryst(1)%spg%symbol_xhm)
!corr       c_space(:lens+1) = toCString(cryst(1)%spg%symbol_xhm)
!corr   endif
!corr   if (cryst(1)%is_cell()) then
!corr       c_cell = real(cryst(1)%get_cellpar(),c_double)
!corr   else
!corr       c_cell = -1  ! these force empty entries for cell
!corr   endif
!corr   
!corr   end subroutine init_dm
!corr
! -----------------------------------------------------------------

   function get_cell_content_len() bind(C,name="get_cell_content_len")
   use variables, only: cryst
   integer(c_int) :: get_cell_content_len
   get_cell_content_len = len_trim(cryst(1)%get_strelem())
   end function get_cell_content_len

! -----------------------------------------------------------------
   subroutine get_cell_content(c_content) bind(C,name="get_cell_content")
   use iso_c_binding
   use strutil
   use variables, only: cryst
   character(c_char), dimension(*), intent(out) :: c_content
   integer                                      :: lenc

   lenc = len_trim(cryst(1)%get_strelem())
   c_content(:lenc+1) = toCString(cryst(1)%get_strelem())

   end subroutine get_cell_content

END MODULE exploreset


! -----------------------------------------------------------------

   subroutine open_dm(cell_cont)
   use variables, only: cryst
   character(len=:), allocatable, intent(out) :: cell_cont
   cell_cont = cryst(1)%get_strelem()
   end subroutine open_dm

! -----------------------------------------------------------------

   subroutine open_dsm(cell, contentC)
   use variables, only: cryst
   use unit_cell
   character(len=*) :: contentC
   real, dimension(6)                         :: cell
   contentC = cryst(1)%get_strelem()//char(0)
   cell = get_par_v(cryst(1)%cell)
   end subroutine open_dsm


! -----------------------------------------------------------------

   subroutine recycle() bind(C,name="recycle")
   USE General, only: is_anomalous,lo,card,spacegr,neutro,POW_FILE, &
                      celem,nrefl
   USE Molcom
   USE PatternRef
   USE atomi, only:nats
   USE PhaseModule, only: setmin, erm1min, isumalmin, jrest
   USE commandsmod
   USE variables, only: atm,ref
   USE reflection_type_util
   USE Riflessi, only: fcvet
   implicit none
   common /priori/iprioro,ipriorp,ipriors,ipriopa,ipriofr,ksetfo
   integer :: iprioro,ipriorp,ipriors,ipriopa,ipriofr,ksetfo
   common /step_extra/ipriort,iprio,ipriosta,sogran,thetsav1,thetsav2 &
   ,ncounsav1,ncounsav2,fracout,ioptim
   integer :: ipriort,iprio,ipriosta,ncounsav1,ncounsav2,ioptim
   real :: sogran,thetsav1,thetsav2,fracout
   common /comdir/ icomq(200,2),maxcom,ipcom,ipdir,icomat
   integer :: icomq,maxcom,ipcom,ipdir,icomat
   integer :: ier, iEnd
   integer :: ip
!
!FIXME-corrado-prova per attivare il riciclo da 'OLD PROJECT'
   if (nats == 0 .or. nrefl == 0) return
!
   if (icomat == 0) then
       icomat = 2
       icomq(1,1) = 11
       icomq(2,1) = 12
   endif
!corrado-prova
   jrest=0
!corr      icomat=icomat-1
!corr      call UseFragmn(ier)
!corr      if(ier.lt.0)return
!
!     add directive fsir (iext=3) to read fc from the model (see reflin)
   ip = command_position(commands,'extraction',back=.true.)
   if (ip == 0) return
   !call add_directive(commands(ip),['fsir','fast'])
   call commands(ip)%add(['fsir','fast'])
   call create_expo_commands(commands,ipcom,icomq,card)
   icomat = ip - 1
   ipdir = sum(commands(:icomat)%ndir)
   !call print_commands(commands,0)
!  ifc=0
!
   ipriofr = ipriofr+1
!  call calcfc(ifc,ier)
   !call calcfc(ier)
   !if(ier.lt.0)return
   write(lo,10)
!
!  Calculate s.f. e write on the reflection list
     !write(0,*)'SIZE=',size(ref), size(fcvet),nrefl,size(IndHKL(:,1))
   call sfcalc(ref,atm,spacegr,celem,neutro,is_anomalous)
   fcvet(:nrefl) = ref%fc*ref%fc
   call CreaListaRiflessi
!!!!!prova
!   ipriofr = ipriofr+1
!   call prepal6
   istep_extra=1
   ipriort=ipriorp+ipriors+ipriopa+ipriofr
!corr-evita che 'Lowest Phase Error' sia sbagliato in phase
!init of these variables is in phase.f
!corr      setmin = 0
!corr      erm1min  = 9999
!corr      isumalmin=0
!corr
   call gescom(POW_FILE,iEnd,ier)
!
10 format(/,15x,'***   Fragment information will be used as   ***', &
           /,15x,'***        prior information in Extra        ***')
!
   end subroutine recycle
