#ifndef RIETVELDDIALOG_H
#define RIETVELDDIALOG_H

#include "dataset.h"

#include <QDialog>
#include <QAbstractButton>

class MainWindow;

typedef struct {
     int              algo;          // tipo di algoritmo di l.sq.
     int              auto_coefb;    // selezione automatica del numero di coeff. di backgr.?
     int              auto_prof;     // affinamento automatico profilo
     int              auto_stru;     // affinamento automatico struttura
     int              ncauto;        // num. di cicli di affinamento automatico
     int              typewei;       // schema di pesaggio
     float            eps;           // criterio di convergenza
     int              maxcy;         // numero massimo di cicli di affinamento
     int              raction;       // 0=Rietveld 1=Le Bail 2=Pawley
     int              printOptions[5];     // print options
                } refine_condition_type;

namespace Ui {
class RietveldDialog;
}

class RietveldDialog : public QDialog
{
    Q_OBJECT

public:
    explicit RietveldDialog(MainWindow *mw);
    ~RietveldDialog();
    void waitSignalIsEmitted();
    void setDialog(const refine_condition_type &rcond);

private slots:
    void onButtonBoxClicked(QAbstractButton *button);
    void autoProfileClicked(bool state);
    void nCycleSpinEnabled();

private:
    Ui::RietveldDialog *ui;
    MainWindow *mWindow;
    refine_condition_type rietCond;
    dataset_t dataset;
    BackgroundSettings backriet;
    bool wSchemeChanged;
    QVector<bool> poChanged, cellChanged;
    int numPhase;
    enum PrintOptions {StartEndCycle, EachCycle, CorrelMatrix, LSQMatrix, CreateCIF};

    void setGeneralTab();
    void getGeneralTab();
    void setPhaseTab();
    void getPhaseTab();
    void getPowderData();
    void addPhaseTab(const QString &phaseName = "Crystal");
    void removePhaseTab();
    bool hasParameterToRefine();
    bool checkParameterChanged();
    void applyChanges();
    void setLeBailRefineEnabled(bool enabled);
};

#endif // RIETVELDDIALOG_H
