#include <iostream>
#include "libcomune.h"
#include "libmatrix.h"
#include "models.h"
#define BREAKDIST   2.5f
#define ERR         -1
#define CUTOFF      6.0f
#define RmGRID      3.0
#define EmGRID      -2.8
#define CGRID       -3.0*EmGRID*pow(RmGRID,8)
#define DGRID       -4.0*EmGRID*pow(RmGRID,6)
#define K1GRID      0.9/pow(cos(DEGTORAD(110.0)),6)
#define K2GRID      pow(cos(DEGTORAD(110.0)),2)
#define MINACCANG_SP2   90.0
#define MAXACCANG_SP2   180.0
#define MINDONANG_SP2   90.0
#define MAXDONANG_SP2   180.0
#define ACCDONANG       60.0
#define DONACCANG       90.0
#define DEFNUMPIXEL     18
#define MAXRESDNR                 6
#define MAXRESACC                 6
#define DIST_N_H                  1.0f
#define MAXHYDRBOND               50000
#define Eps                       0.000001
#define Treshold_H1     -230.0
#define Treshold_H3      0.12
#define Treshold_H4      0.06
#define Treshold_E1     -240.0
#define Treshold_E2     -310.0
#define DENTRO(x, target, range)      ( (x >= (target - range)) && (x <= (target + range)) )

typedef struct {
                 float Phi, Psi;
                 int PhiZn, PsiZn;
                 char Asn, PdbAsn;
               } ResidueProperty;

typedef struct {
                 int HBondDnr[MAXRESDNR];
                 int HBondAcc[MAXRESACC];
                 int NBondDnr, NBondAcc;
                 bool InterchainHBonds;
               } Involved;

typedef struct {
                 int Chain;
                 int D_Res, DD_Res, DDI_Res;
                 int D_At, DD_At, DDI_At, H;
                 float HB_Radius;
               } Donor;

typedef struct {
                 int Chain;
                 int A_Res, AA_Res, AA2_Res;
                 int A_At, AA_At, AA2_At;
                 float HB_Radius;
               } Acceptor;

typedef struct {
                 Donor *Dnr;
                 Acceptor *Acc;
                 bool ExistPolarInter, ExistHydrBondRose, ExistHydrBondBaker;
                 float Energy, Er, Et, Ep, ti, to, p;
                 double AccDonDist, OHDist, AngNHO, AngCOH;
                 double AccAng, DonAng, AccDonAng, DonAccAng;
               } HBOND;

typedef struct PAT {
                 HBOND *Hb1, *Hb2;
                 struct PAT *Nei1, *Nei2;
                 bool ExistPattern;
                 //char Type[256];
		 std::string Type;
               } MYPATTERN;

const float PhiPsiStep = 20.f;
const int   nPixel = DEFNUMPIXEL;
ResidueProperty *properties;  // per ogni residuo info geometriche 
Involved *involved;           // per ogni residuo n. res. coinv. in leg h

bool DefineDnr(Model *md, int ch, Donor **Dnr, int *dc, int Res)
{
  Dnr[*dc] =  new Donor ();
  
  Dnr[*dc]->Chain = ch;
  Dnr[*dc]->D_Res = Res;
  Dnr[*dc]->DD_Res = Res-1;
  Dnr[*dc]->DDI_Res = Res;
  Dnr[*dc]->HB_Radius = 1.90f;
  
  if( Res != 0 )
  {
      Dnr[*dc]->D_At  = md->Chains[ch].AminoRes[Res].n;
      Dnr[*dc]->DD_At = md->Chains[ch].AminoRes[Res-1].c;
  }
  else 
  {
      Dnr[*dc]->D_At  = ERR;
      Dnr[*dc]->DD_At = ERR;
  }
  Dnr[*dc]->DDI_At = md->Chains[ch].AminoRes[Res].ca;
  Dnr[*dc]->H      = md->Chains[ch].AminoRes[Res].h;
  
  if( Dnr[*dc]->H == ERR || Dnr[*dc]->D_At   == ERR || Dnr[*dc]->DD_At  == ERR ||
      (Dnr[*dc]->DDI_At == ERR  ) )
  {
    delete [](Dnr[*dc]); 
    return(false);
  }
  else 
    (*dc)++;
  return(true);
}

int get_atom(Model *md, int abnum, int Res, int chain)
{
    RESIDUE rr = md->Chains[chain].AminoRes[Res];
    for (int i=0; i<(int)rr.atoms.size(); i++)
         if(abnum == rr.atoms[i])
            return i;
    return -1;
}

int FindDnr(Model *md, Donor **Dnr, int *NDnr)
{

  int dc = *NDnr;

  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=0; Res< md->Chains[ch].nAmino; Res++)
          if(Is_Amino(md->Chains[ch].AminoRes[Res].code))
             DefineDnr(md, ch,Dnr,&dc, Res);
  }
  *NDnr = dc;
  return(dc);
}

bool DefineAcceptor(Model *md, int nch, Acceptor **Acc, int *ac, int Res)
{

  Acc[*ac] = new Acceptor ();

  Acc[*ac]->Chain     = nch;
  Acc[*ac]->A_Res     = Res;
  Acc[*ac]->AA_Res    = Res;
  Acc[*ac]->AA2_Res   = Res;
  Acc[*ac]->HB_Radius = 1.60f;
  
    if( Res != (md->Chains[nch].nAmino-1))
    {
        Acc[*ac]->A_At   = md->Chains[nch].AminoRes[Res].o;
        Acc[*ac]->AA_At  = md->Chains[nch].AminoRes[Res].c;
    }
    else 
    {
        Acc[*ac]->A_At  = ERR;
        Acc[*ac]->AA_At = ERR;
    }
    Acc[*ac]->AA2_At = md->Chains[nch].AminoRes[Res].ca;
  
  if( Acc[*ac]->A_At   == ERR || Acc[*ac]->AA_At  == ERR ||
      (Acc[*ac]->AA2_At == ERR  ) ) 
  {
      delete [](Acc[*ac]); 
      return(false); 
  }
  else 
      (*ac)++;
  return(true);
}

int FindAcc(Model *md, Acceptor **Acc, int *NAcc)
{

  int ac;
  
  ac = *NAcc;

  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=0; Res< md->Chains[ch].nAmino; Res++)
          if(Is_Amino(md->Chains[ch].AminoRes[Res].code))
             DefineAcceptor(md, ch, Acc, &ac, Res);
  }
  *NAcc = ac;

  return(ac);
}

vet3 NEWProject4_123(vet3 Coord1, vet3 Coord2, vet3 Coord3, vet3 Coord4)
{

/*
                          Atom4
   Atom3                  .
   \                     .
    \                   .    
     \                 .  .Proj4_123
      \               .. 
      Atom2-------Atom1

*/
  vet3 Vector21 = MRVectorSub(Coord1, Coord2);
  vet3 Vector23 = MRVectorSub(Coord3, Coord2);
  vet3 Vector14 = MRVectorSub(Coord4, Coord1);
  double Length_14 = MRVectorLength(Vector14);
  
  vet3 VectorNormal_123 = MRVectorCrossProduct(Vector21,Vector23);
  double NormalLength = MRVectorLength(VectorNormal_123);

  VectorNormal_123 = MRVectorNormalize(VectorNormal_123);

  double COS_Norm_14 = MRVectorDotProduct(VectorNormal_123, Vector14);

  COS_Norm_14 /= (Length_14*NormalLength);

  if( COS_Norm_14 < 0.0 ) {
    COS_Norm_14 = fabs(COS_Norm_14);
    VectorNormal_123 = MRVectorNegate(VectorNormal_123);
  }

  double Proj_14_Norm = Length_14*COS_Norm_14;

  VectorNormal_123 = MRVectorMultiplyScalar(VectorNormal_123, Proj_14_Norm);
  vet3 Coord_Proj4_123  = MRVectorAdd(MRVectorSub(Vector14, VectorNormal_123),
                           Coord1);
  return Coord_Proj4_123;
}

double NewAngolo (vet3 p1, vet3 p2, vet3 p3)
{
   vet3 a = MRVectorSub(p1, p2);
   vet3 b = MRVectorSub(p3, p2);
   return MRVectorAngle(a, b);
}

void NEWHBOND_Energy(vet3 CA2, vet3 C, vet3 O, vet3 H, vet3 N, HBOND *HBond)
{

  vet3 ProjH;

 /***** Distance dependence ( 8-6 potential ) ****/

  if( HBond->AccDonDist < RmGRID )
    HBond->AccDonDist = RmGRID;
  HBond->Er = (float) (CGRID/pow(HBond->AccDonDist,8) - DGRID/pow(HBond->AccDonDist,6));

 /************** Angular dependance ****************/

 /* Find projection of the hydrogen on the O-C-CA plane */
  ProjH = NEWProject4_123(O, C, CA2, H);


 /* Three angles determining the direction of the hydrogen bond */
  HBond->ti = (float) fabs(180.0 - NewAngolo(ProjH, O, C));
  HBond->to = (float) NewAngolo(H, O, ProjH);
  HBond->p  = (float) NewAngolo(N, H, O);

 /* Calculate both angle-dependent HB energy components Et and Ep */
  if( HBond->ti >= 0.0 && HBond->ti < 90.0 )
    HBond->Et = (float) (cos(DEGTORAD(HBond->to))*(0.9+0.1*sin(DEGTORAD(2*HBond->ti))));
  else
  if( HBond->ti >= 90.0 && HBond->ti < 110.0 )
    HBond->Et = (float) (K1GRID*cos(DEGTORAD(HBond->to))*
      (pow((K2GRID-pow(cos(DEGTORAD(HBond->ti)),2)),3)));
  else
    HBond->Et = 0.0f;

  if( HBond->p > 90.0 && HBond->p < 270.0 )
    HBond->Ep = (float) pow(cos(DEGTORAD(HBond->p)),2);
  else
    HBond->Ep = 0.0;

    /******** Full hydrogen bond energy *********************/
  HBond->Energy = 1000.f*HBond->Er*HBond->Et*HBond->Ep;
}

int FindHydrogenBonds(Model *md, HBOND **HBond, std::string &msg)
{
    Donor **Dnr;
    Acceptor **Acc;
    bool *BondedDonor, *BondedAcceptor;
    int NDnr=0, NAcc=0;
    int dc, ac, hc=0, i;

    Dnr = new Donor* [md->nResTot]();
    Acc = new Acceptor *[md->nResTot]();
  
    FindDnr(md, Dnr,&NDnr);
    FindAcc(md, Acc,&NAcc);

    BondedDonor    = new bool [NDnr]();
    BondedAcceptor = new bool [NAcc]();
    for( i=0; i<NDnr; i++ )
      BondedDonor[i] = false;
    for( i=0; i<NAcc; i++ )
      BondedAcceptor[i] = false;

    for( dc=0; dc<NDnr; dc++ ) 
    {
         for( ac=0; ac<NAcc; ac++ ) 
         {
              if( (abs(Acc[ac]->A_Res - Dnr[dc]->D_Res) < 2) && 
                  (Acc[ac]->Chain == Dnr[dc]->Chain) ) 
                   continue;
              if( hc == MAXHYDRBOND ) 
              {
                  msg = "Number of hydrogen bonds exceeds current limit \n";
                  return 0;
              }
              HBond[hc] = new HBOND();
              HBond[hc]->ExistHydrBondRose = false;
              HBond[hc]->ExistHydrBondBaker = false;
              HBond[hc]->ExistPolarInter = false;
      
              vet3 n_d = {{md->atm[Dnr[dc]->D_At].xyz[0], 
                           md->atm[Dnr[dc]->D_At].xyz[1], 
                           md->atm[Dnr[dc]->D_At].xyz[2]}}; 
              vet3 h_d;
              for(int jj=0; jj<3; jj++)
              {
                  if(Dnr[dc]->H >= 0)
                     h_d.v[jj] = md->atm[Dnr[dc]->H].xyz[jj];
                  else
                     h_d.v[jj] = md->c_alpha[-(1+Dnr[dc]->H)].Hxyz[jj];
              }
              vet3 c_d = {{md->atm[Dnr[dc]->DD_At].xyz[0], 
                           md->atm[Dnr[dc]->DD_At].xyz[1], 
                           md->atm[Dnr[dc]->DD_At].xyz[2]}}; 
              vet3 ca_d = {{md->atm[Dnr[dc]->DDI_At].xyz[0], 
                            md->atm[Dnr[dc]->DDI_At].xyz[1], 
                            md->atm[Dnr[dc]->DDI_At].xyz[2]}}; 
              vet3 o_a = {{md->atm[Acc[ac]->A_At].xyz[0], 
                           md->atm[Acc[ac]->A_At].xyz[1], 
                           md->atm[Acc[ac]->A_At].xyz[2]}}; 
              vet3 c_a = {{md->atm[Acc[ac]->AA_At].xyz[0], 
                           md->atm[Acc[ac]->AA_At].xyz[1], 
                           md->atm[Acc[ac]->AA_At].xyz[2]}}; 
              vet3 ca_a = {{md->atm[Acc[ac]->AA2_At].xyz[0], 
                            md->atm[Acc[ac]->AA2_At].xyz[1], 
                            md->atm[Acc[ac]->AA2_At].xyz[2]}}; 
              if( (HBond[hc]->AccDonDist = MRVectorDistance(n_d, o_a) ) <= CUTOFF ) 
              {
                 if( Dnr[dc]->H != ERR) 
                 {
                     NEWHBOND_Energy(ca_a, c_a, o_a, h_d, n_d, HBond[hc]); 
                     if( (HBond[hc]->Energy < -10.0) ) 
                         HBond[hc]->ExistPolarInter = true;
                 }
                 HBond[hc]->OHDist = MRVectorDistance(h_d, o_a);
                 HBond[hc]->AngNHO = NewAngolo(n_d, h_d, o_a);
                 HBond[hc]->AngCOH = NewAngolo(c_a, o_a, h_d);
                 if( (HBond[hc]->OHDist <= 2.5) &&
                     (HBond[hc]->AngNHO  >= 90.0) &&
                     (HBond[hc]->AngNHO <= 180.0) &&
                     (HBond[hc]->AngCOH >= 90.0) &&
                     (HBond[hc]->AngCOH <= 180.0 ))
                          HBond[hc]->ExistHydrBondBaker = true;
                 if( HBond[hc]->AccDonDist <= Dnr[dc]->HB_Radius+Acc[ac]->HB_Radius ) 
                 {
                     HBond[hc]->AccAng = NewAngolo(n_d, o_a, c_a);
                     if( ( HBond[hc]->AccAng >= MINACCANG_SP2 ) && 
                         ( HBond[hc]->AccAng <= MAXACCANG_SP2 ) ) 
                     {
                         HBond[hc]->DonAng = NewAngolo(o_a, n_d, c_d);
                         if( ( HBond[hc]->DonAng >= MINDONANG_SP2 && 
                             HBond[hc]->DonAng <= MAXDONANG_SP2 ) ) 
                         {
                             HBond[hc]->AccDonAng = abs(MRVectorDihedral(ca_d, n_d, c_d, o_a));
                             if( HBond[hc]->AccDonAng > 90.0 && HBond[hc]->AccDonAng < 270.0 )
                                 HBond[hc]->AccDonAng = abs(180.0 - HBond[hc]->AccDonAng);
                             HBond[hc]->DonAccAng = abs(MRVectorDihedral(n_d, o_a, c_a, ca_a));
                             if(HBond[hc]->DonAccAng > 90.0 && HBond[hc]->DonAccAng < 270.0)
                                HBond[hc]->DonAccAng = abs(180.0 - HBond[hc]->DonAccAng);
                             if( (HBond[hc]->AccDonAng <= ACCDONANG) &&
                                 (HBond[hc]->DonAccAng <= DONACCANG ) )
                                  HBond[hc]->ExistHydrBondRose = true;
                          }
                      }
                 }
              }
              if( (HBond[hc]->ExistPolarInter && HBond[hc]->Energy < 0.0) 
                || HBond[hc]->ExistHydrBondRose || HBond[hc]->ExistHydrBondBaker ) 
              {
                 HBond[hc]->Dnr = Dnr[dc];
                 HBond[hc]->Acc = Acc[ac];
                 BondedDonor[dc] = true;
                 BondedAcceptor[ac] = true;
                 int ddc = md->GetNumResiduo(Dnr[dc]->Chain, Dnr[dc]->D_Res);
                 if(involved[ddc].NBondDnr < MAXRESDNR)
                    involved[ddc].HBondDnr[involved[ddc].NBondDnr++] = hc;
                 else
                 {
                     msg = "Residue involved in more hydrogen bonds than allowed";
//                     return 0;
                 }
                 int aac = md->GetNumResiduo(Acc[ac]->Chain, Acc[ac]->A_Res);
                 if(involved[aac].NBondAcc < MAXRESACC)
                    involved[aac].HBondAcc[involved[aac].NBondAcc++] = hc;
                 else
                 {
                     msg = "Residue involved in more hydrogen bonds than allowed";
//                     return 0;
                 }
                 int ccd = Dnr[dc]->Chain;
                 int cca = Acc[ac]->Chain;
                 if( ccd != cca)
                 {
                     involved[ddc].InterchainHBonds = true;
                     involved[aac].InterchainHBonds = true;
                 }
                 hc++;
              }
              else
                 delete [](HBond[hc]);
         }
    }

    for( i=0; i<NDnr; i++ )
      if( !BondedDonor[i] )
        delete [](Dnr[i]);
    for( i=0; i<NAcc; i++ )
      if( !BondedAcceptor[i] )
        delete [](Acc[i]);
    if( NDnr )
      delete [](BondedDonor);
    if( NAcc )
      delete [](BondedAcceptor);
    return(hc);
}

int FindPolInt(HBOND **HBond, int Res1, int Res2)
{

  int i, j, hb;
  Involved *p1, *p2;

  p1 = &involved[Res1];
  p2 = &involved[Res2];

  if( p1->NBondDnr && p2->NBondAcc ) 
  {
      for( i=0; i<p1->NBondDnr; i++ ) 
      {
           hb = p1->HBondDnr[i];
           for( j=0; j<p2->NBondAcc; j++ )
                if( hb == p2->HBondAcc[j] && HBond[hb]->ExistPolarInter ) 
                    return(hb);
      }
  }
  return(ERR);
}

int FindBnd(HBOND **HBond, int Res1, int Res2)
{

  int i, j, hb;
  Involved *p1, *p2;

  p1 = &involved[Res1];
  p2 = &involved[Res2];

  if( p1->NBondDnr && p2->NBondAcc ) 
  {
      for( i=0; i<p1->NBondDnr; i++ ) 
      {
            hb = p1->HBondDnr[i];
            for( j=0; j<p2->NBondAcc; j++ )
                  if( hb == p2->HBondAcc[j] && HBond[hb]->ExistHydrBondRose ) 
                      return(hb);
      }
  }
  return(ERR);
}

int NoDoubleHBond(HBOND **HBond, int NHBond)
{

  int i, j, NExcl=0;

  for( i=0; i<NHBond-1; i++ )
  {
       for( j=i+1; j<NHBond; j++ ) 
       {
           if( HBond[i]->Dnr->D_Res == HBond[j]->Dnr->D_Res &&
               HBond[i]->Dnr->Chain == HBond[j]->Dnr->Chain &&
               HBond[i]->ExistPolarInter && HBond[j]->ExistPolarInter ) 
           {
              if( HBond[i]->Energy < 5.0*HBond[j]->Energy ) 
              {
                  HBond[j]->ExistPolarInter = false;
                  NExcl++;
              }
              else if( HBond[j]->Energy < 5.0*HBond[i]->Energy ) 
              {
                   HBond[i]->ExistPolarInter = false;
                   NExcl++;
              }
           }
       }
  }
  return(NExcl);
}

float **DefaultHelixMap()
{
  int i;

  float **Map;
  static float Data[DEFNUMPIXEL][DEFNUMPIXEL] = {

      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0009014423f, 0.0041898815f,
      0.0085105160f, 0.0133839026f, 0.0245425366f, 0.0407802090f, 0.0464176536f, 0.0330946408f,
      0.0134803243f, 0.0024038462f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0007370283f, 0.0077203326f, 0.0269849468f,
      0.0492307022f, 0.0621860325f, 0.0747849122f, 0.0919913873f, 0.0918549150f, 0.0617070347f,
      0.0241584498f, 0.0041428790f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0041416897f, 0.0287234355f, 0.0835687742f,
      0.1384727061f, 0.1562444866f, 0.1470608264f, 0.1360232681f, 0.1159155145f, 0.0742164999f,
      0.0290896539f, 0.0050673936f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0009375000f, 0.0156580955f, 0.0757770315f, 0.1856354773f,
      0.2785892785f, 0.2880102694f, 0.2332847565f, 0.1741978228f, 0.1281246394f, 0.0793832615f,
      0.0320557840f, 0.0058840578f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0048893229f, 0.0437000208f, 0.1617751122f, 0.3399706185f,
      0.4626395404f, 0.4418565035f, 0.3235570788f, 0.2100441158f, 0.1358627081f, 0.0776144490f,
      0.0297011137f, 0.0052390974f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0136979166f, 0.0917820632f, 0.2773087323f, 0.5047551394f,
      0.6214492917f, 0.5485223532f, 0.3655386865f, 0.2054343373f, 0.1121114418f, 0.0548815951f,
      0.0178668182f, 0.0025975490f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0246484373f, 0.1396044195f, 0.3594934344f, 0.5710113049f,
      0.6337110400f, 0.5133636594f, 0.3054708838f, 0.1402616948f, 0.0584463216f, 0.0228670351f,
      0.0058531328f, 0.0005151099f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0265885405f, 0.1365883052f, 0.3163702190f, 0.4545661211f,
      0.4628692269f, 0.3425511420f, 0.1761947423f, 0.0607788190f, 0.0158569515f, 0.0042061093f,
      0.0008107311f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0152018229f, 0.0738445148f, 0.1630392224f, 0.2269553691f,
      0.2237145752f, 0.1528334022f, 0.0652616471f, 0.0150429625f, 0.0014589608f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0035156249f, 0.0165251363f, 0.0379281938f, 0.0584417619f,
      0.0619409233f, 0.0404052660f, 0.0136552500f, 0.0016678370f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0011718750f, 0.0046875002f,
      0.0070312503f, 0.0046875002f, 0.0011718750f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0006944445f, 0.0036063762f, 0.0080820229f, 0.0101532144f, 0.0076146079f,
      0.0032324446f, 0.0006009616f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f }
  };

  Map = new float * [DEFNUMPIXEL]();

  for( i=0; i<DEFNUMPIXEL; i++ )
    Map[i] = &(Data[i][0]);

  return(Map);
}

float **DefaultSheetMap()
{

  int i;
  float **Map;
  static float Data[DEFNUMPIXEL][DEFNUMPIXEL] = {

      {0.2769023776f, 0.1408346891f, 0.0464910716f, 0.0073784725f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0047086575f, 0.0218229108f, 0.0569166169f,
      0.1254088134f, 0.2340224832f, 0.3511219919f, 0.4355685711f, 0.4584180117f, 0.4007356465f},
      {0.4067636132f, 0.2329865396f, 0.0927943364f, 0.0237838365f, 0.0055147060f, 0.0013786765f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0088186050f, 0.0420726910f, 0.1043856740f,
      0.2086037844f, 0.3677131534f, 0.5367187858f, 0.6412357688f, 0.6458424330f, 0.5580080152f},
      {0.4286311865f, 0.2678007782f, 0.1282834113f, 0.0529448465f, 0.0220588241f, 0.0055147060f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0086062262f, 0.0445192643f, 0.1197573245f,
      0.2487278134f, 0.4369854629f, 0.6241853237f, 0.7160459757f, 0.6829043031f, 0.5716546178f},
      {0.3639202416f, 0.2397334576f, 0.1305907220f, 0.0683420748f, 0.0330882370f, 0.0082720593f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0053559211f, 0.0328565054f, 0.1048930883f,
      0.2402425259f, 0.4295993447f, 0.6026929021f, 0.6669865251f, 0.6039550304f, 0.4841639400f},
      {0.2637948096f, 0.1723874062f, 0.0920098722f, 0.0464194641f, 0.0220588241f, 0.0055147060f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0030202419f, 0.0224239044f, 0.0804052502f,
      0.1923188865f, 0.3456886411f, 0.4811576009f, 0.5223571062f, 0.4586051404f, 0.3565762639f},
      {0.1628032923f, 0.0930610597f, 0.0400134660f, 0.0143100554f, 0.0055147060f, 0.0013786765f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0015453297f, 0.0132468110f, 0.0489843786f,
      0.1174781919f, 0.2150468081f, 0.3082944453f, 0.3439011276f, 0.3080393970f, 0.2371628135f},
      {0.0825822726f, 0.0338854715f, 0.0092895878f, 0.0012122844f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0003863324f, 0.0046614520f, 0.0186656341f,
      0.0477515720f, 0.0961741805f, 0.1546680480f, 0.1961039603f, 0.1944279373f, 0.1469529718f},
      {0.0326442868f, 0.0073916214f, 0.0008854167f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0006347656f, 0.0031504754f,
      0.0104655549f, 0.0272454955f, 0.0570511036f, 0.0941907763f, 0.1088592261f, 0.0785619915f},
      {0.0090501504f, 0.0007651417f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0007207961f, 0.0035958111f, 0.0131648667f, 0.0318824202f, 0.0425693691f, 0.0292618107f},
      {0.0013020834f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0013020834f, 0.0052083335f, 0.0078125000f, 0.0052083335f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0210939310f, 0.0078523019f, 0.0013020834f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0014204546f,
      0.0071634995f, 0.0169352461f, 0.0272206441f, 0.0357281528f, 0.0395361669f, 0.0343801714f},
      {0.1146211401f, 0.0503530800f, 0.0130920913f, 0.0015190972f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0010016026f, 0.0046167620f, 0.0157516468f,
      0.0453012958f, 0.0937970504f, 0.1454590708f, 0.1861637682f, 0.2019522935f, 0.1764564067f}
  };

 
  Map = new float * [DEFNUMPIXEL]();

  for( i=0; i<DEFNUMPIXEL; i++ )
    Map[i] = &(Data[i][0]);

  return(Map); 
}

int PlaceHydrogens(Model *md, std::vector<C_Alpha> &c_alpha)
{

  int i, PlacedCnt=0;
  float Length_N_C, Length_N_CA, Length_N_H;
  int hyd_pres = 0;

  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=1; Res<md->Chains[ch].nAmino; Res++)
      {
          if(!Is_Amino(md->Chains[ch].AminoRes[Res].code) || 
             !Is_Amino(md->Chains[ch].AminoRes[Res-1].code))
                 continue;
          int ca = md->GetNumResiduo(ch, Res);
          if(md->Chains[ch].AminoRes[Res].name.compare("PRO") == 0)
             continue;
          if( (c_alpha[ca].h == -1) && (c_alpha[ca].n != -1) && 
              (c_alpha[ca-1].c != -1))
          {
              Length_N_C   = Distanza(md->atm[c_alpha[ca].n].xyz,md->atm[c_alpha[ca-1].c].xyz);
              Length_N_CA  = Distanza(md->atm[c_alpha[ca].n].xyz,md->atm[c_alpha[ca].atom].xyz);
              for( i=0; i<3; i++ )
                 c_alpha[ca].Hxyz[i] = md->atm[c_alpha[ca].n].xyz[i] -
                   ( (md->atm[c_alpha[ca-1].c].xyz[i] -  md->atm[c_alpha[ca].n].xyz[i])/Length_N_C +
                     (md->atm[c_alpha[ca].atom].xyz[i] -  md->atm[c_alpha[ca].n].xyz[i])/Length_N_CA );
              Length_N_H = Distanza(md->atm[c_alpha[ca].n].xyz, c_alpha[ca].Hxyz);
              for( i=0; i<3; i++ )
                    c_alpha[ca].Hxyz[i] = md->atm[c_alpha[ca].n].xyz[i] +
                    DIST_N_H*(c_alpha[ca].Hxyz[i]-md->atm[c_alpha[ca].n].xyz[i])/Length_N_H;
              md->Chains[ch].AminoRes[Res].h = -(ca+1);
              PlacedCnt++;
         }
         else if(c_alpha[ca-1].c != -1)
          hyd_pres = 1;
      }
  }
  return(PlacedCnt+hyd_pres);
}

void DiscrPhiPsi(Model *md)
{
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           int curr = md->GetNumResiduo(ch, j);
           properties[curr].PhiZn = -1;
           properties[curr].PsiZn = -1;
           if( j != 0 )
           {
              for( int i=0; i<nPixel; i++ )
              {
                   if( properties[curr].Phi  >  -180+(float)(i)*PhiPsiStep && 
                       properties[curr].Phi <=  -180+(float)(i+1)*PhiPsiStep ) 
                   {
                       properties[curr].PhiZn = i;
                       break;
                   }
              }
           }
           if( j != (md->Chains[ch].nAmino-1) )
           {
               for( int i=0; i<nPixel; i++ )
               {
                    if( properties[curr].Psi  >  -180+(float)(i)*PhiPsiStep && 
                        properties[curr].Psi <=  -180+(float)(i+1)*PhiPsiStep ) 
                    {
                        properties[curr].PsiZn = i;
                        break;
                    }
               }
          }
       }
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           int curr = md->GetNumResiduo(ch, j);
           if( (j != 0) && (properties[curr].PsiZn == -1) )
               properties[curr].PsiZn = properties[curr-1].PsiZn;
           if( (j != (md->Chains[ch].nAmino-1)) && (properties[curr].PhiZn == -1) )
               properties[curr].PhiZn = properties[curr+1].PhiZn;
       }
   }
}

void Helix(Model *md,  int Cn, HBOND **HBond, float **PhiPsiMap)
{
  int BondNumb;
  float *Proba;
  ResidueProperty *rp;

  Proba = new float [md->Chains[Cn].nAmino]();

  for( int i=0; i<md->Chains[Cn].nAmino; i++ )
    Proba[i] = 0.0; 

  for( int i=0; i<md->Chains[Cn].nAmino-5; i++ )
  {
    int Res = md->GetNumResiduo(Cn, i);    
    rp = &properties[Res];

    if( rp[0].PhiZn != ERR && rp[0].PsiZn != ERR &&
        rp[1].PhiZn != ERR && rp[1].PsiZn != ERR &&
        rp[2].PhiZn != ERR && rp[2].PsiZn != ERR &&
        rp[3].PhiZn != ERR && rp[3].PsiZn != ERR &&
        rp[4].PhiZn != ERR && rp[4].PsiZn != ERR )
    {
        if( (BondNumb = FindPolInt(HBond,Res+4,Res)) != ERR ) 
        {
             Proba[i] = HBond[BondNumb]->Energy*(0.5f*
                 (PhiPsiMap[rp[0].PhiZn][rp[0].PsiZn]+
                 PhiPsiMap[rp[4].PhiZn][rp[4].PsiZn]));
        }
    }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-5; i++ )
  {
    if( (Proba[i] < Treshold_H1) && (Proba[i+1] < Treshold_H1) )
    {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      rp[1].Asn = 'H'; 
      rp[2].Asn = 'H'; 
      rp[3].Asn = 'H'; 
      if( rp[0].PhiZn!= ERR && rp[0].PsiZn != ERR &&
          PhiPsiMap[rp[0].PhiZn][rp[0].PsiZn] > Treshold_H3 )
          rp[0].Asn = 'H';
      if( rp[4].PhiZn != ERR && rp[4].PsiZn != ERR &&
          PhiPsiMap[rp[4].PhiZn][rp[4].PsiZn] > Treshold_H4 )
          rp[4].Asn = 'H';
    }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-4; i++ )
  {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      if(  FindBnd(HBond,i+3,i) != ERR && FindBnd(HBond,i+4,i+1) != ERR &&
         ( (rp[1].Asn != 'H' && rp[2].Asn != 'H') ||
          (rp[2].Asn != 'H' && rp[3].Asn != 'H') ) ) 
      {
          rp[1].Asn = 'G'; 
          rp[2].Asn = 'G'; 
          rp[3].Asn = 'G'; 
      }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-6; i++ )
  {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      if( FindBnd(HBond,i+5,i) != ERR && FindBnd(HBond,i+6,i+1) != ERR &&
        rp[1].Asn == 'C' && rp[2].Asn == 'C' && 
        rp[3].Asn == 'C' && rp[4].Asn == 'C' && 
        rp[5].Asn == 'C' ) 
      {
         rp[1].Asn = 'I'; 
         rp[2].Asn = 'I';
         rp[3].Asn = 'I'; 
         rp[4].Asn = 'I'; 
         rp[5].Asn = 'I'; 
      }
  }

  delete [](Proba);
}

int Link(HBOND **HBond, int Res1_1,
         int Res1_2, int Res2_2, int Res2_1, int CRes1, 
         int CRes2, float **PhiPsiMap, MYPATTERN **Pattern, int *NumPat, 
         const char *Text, float Treshold, int Test)
{

  int BondNumber1, BondNumber2, Flag = 0;
  float Prob1, Prob2, Conf, Coeff;
  float C1_E = -0.2f;
  float C2_E =  0.2f;

  if( (BondNumber1 = FindPolInt(HBond,Res1_1,Res1_2)) == ERR ) 
    return(0);

  if( (BondNumber2 = FindPolInt(HBond,Res2_2,Res2_1)) == ERR ) 
    return(0);

  if( CRes1 == -1 ) 
  {
    if( properties[CRes2].PhiZn == ERR || properties[CRes2].PsiZn == ERR )
      return(0);
    Conf = PhiPsiMap[properties[CRes2].PhiZn][properties[CRes2].PsiZn];
  }
  else if( CRes2 == -1 ) 
  {
    if( properties[CRes1].PhiZn == ERR || properties[CRes1].PsiZn == ERR )
      return(0);
    Conf = PhiPsiMap[properties[CRes1].PhiZn][properties[CRes1].PsiZn];
  }
  else 
  {
    if( properties[CRes2].PhiZn == ERR || properties[CRes2].PsiZn == ERR ||
        properties[CRes1].PhiZn == ERR || properties[CRes1].PsiZn == ERR )
      return(0);
    Conf = 
      0.5f*(PhiPsiMap[properties[CRes1].PhiZn][properties[CRes1].PsiZn]+
         PhiPsiMap[properties[CRes2].PhiZn][properties[CRes2].PsiZn]);
  }
  Coeff = 1+C1_E+C2_E*Conf;
  Prob1 = HBond[BondNumber1]->Energy*Coeff;
  Prob2 = HBond[BondNumber2]->Energy*Coeff;

  if( Prob1 < Treshold && Prob2 < Treshold ) {

    if( !Test ) {
      Pattern[*NumPat] = new MYPATTERN();
      Pattern[*NumPat]->ExistPattern = true;
      Pattern[*NumPat]->Hb1 = HBond[BondNumber1];
      Pattern[*NumPat]->Hb2 = HBond[BondNumber2];
      Pattern[*NumPat]->Nei1 = NULL;
      Pattern[*NumPat]->Nei2 = NULL;
      Pattern[*NumPat]->Type = Text;
      (*NumPat)++;
    }
    Flag = 1;
  }
  
  return(Flag);
  
}

bool MakeEnds(Model *md, int *Beg1, int ResBeg1, int NeiBeg1, char *Beg1Cn, char ResBeg1Cn, int *End1, 
               int ResEnd1, int NeiEnd1, char ResEnd1Cn, int *Beg2, int ResBeg2, int NeiBeg2, 
               char *Beg2Cn, char ResBeg2Cn, int *End2, int ResEnd2, int NeiEnd2, 
               char ResEnd2Cn, MYPATTERN **Pat, int NPat)
{

  int i;
  int Flag1 = 0, Flag2 = 0;


  if( ResBeg1 <= NeiBeg1 && NeiBeg1 <= NeiEnd1 && NeiEnd1 <= ResEnd1 &&
      ResBeg2 <= NeiBeg2 && NeiBeg2 <= NeiEnd2 && NeiEnd2 <= ResEnd2 &&
      ResBeg1Cn == ResEnd1Cn && ResBeg2Cn == ResEnd2Cn ) {

    *Beg1 = ResBeg1;
    *End1 = ResEnd1;
    *Beg2 = ResBeg2;
    *End2 = ResEnd2;
    *Beg1Cn = ResBeg1Cn;
    *Beg2Cn = ResBeg2Cn;
    
    for( i=0; i<NPat && (Flag1 == 0 || Flag2 == 0); i++ ) {
      if( ( (Pat[i]->Hb1->Dnr->D_Res == (*Beg1) 
             && Pat[i]->Hb1->Acc->A_Res == (*End2)
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg1Cn)   
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg2Cn) )
             ||
             (Pat[i]->Hb1->Acc->A_Res == (*Beg1) 
             && Pat[i]->Hb1->Dnr->D_Res == (*End2) 
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg1Cn) 
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg2Cn) ) ) 
             && Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res 
             && Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             Flag1 = 1; 
      if( ( (Pat[i]->Hb1->Dnr->D_Res == (*Beg2) 
             && Pat[i]->Hb1->Acc->A_Res == (*End1) 
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg2Cn)    
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg1Cn) ) 
             ||
             (Pat[i]->Hb1->Acc->A_Res == (*Beg2) 
             && Pat[i]->Hb1->Dnr->D_Res == (*End1) 
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg2Cn)
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg1Cn) ) ) 
             && Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res 
             && Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             Flag2 = 1; 
    }
    
    if( !Flag1 ) {
      if( *Beg1 != NeiBeg1 ) (*Beg1)++;
      if( *End2 != NeiEnd2 ) (*End2)--;
    }
    
    if( !Flag2 ) {
      if( *End1 != NeiEnd1 ) (*End1)--;
      if( *Beg2 != NeiBeg2 ) (*Beg2)++;
    }
    return(true);
  }
  
  return(false);
}
  
void Alias(Model *md, int *D1,int *A1,int *D2,int *A2,char *D1Cn,char *A1Cn,char *D2Cn,char *A2Cn,
          MYPATTERN *Pat)
{
    *D1 = Pat->Hb1->Dnr->D_Res;
    *A1 = Pat->Hb1->Acc->A_Res;
    *D2 = Pat->Hb2->Dnr->D_Res;
    *A2 = Pat->Hb2->Acc->A_Res;
    *D1Cn = md->Chains[Pat->Hb1->Dnr->Chain].id[0]; 
    *A1Cn = md->Chains[Pat->Hb1->Acc->Chain].id[0];
    *D2Cn = md->Chains[Pat->Hb2->Dnr->Chain].id[0];
    *A2Cn = md->Chains[Pat->Hb2->Acc->Chain].id[0];
}

void FillAsnAntiPar(char *Asn1, char *Asn2, Model *md, int Cn1, int Cn2, 
             MYPATTERN **Pat, int NPat)
{
  int i, j;
  int Beg1, Beg2, End1, End2;
  int B1D, B1A, B2D, B2A, E1D, E1A, E2D, E2A;
  char B1DCn, B1ACn, B2DCn, B2ACn, E1DCn, E1ACn, E2DCn, E2ACn, Beg1Cn, Beg2Cn; 
  MYPATTERN *CurrPat, *PrevPat;;

  for( i=0; i<NPat; i++ ) {

    if( Pat[i]->Nei1 != NULL && Pat[i]->Nei2 == NULL )
      CurrPat = Pat[i]->Nei1;
    else
    if( Pat[i]->Nei2 != NULL && Pat[i]->Nei1 == NULL ) 
      CurrPat = Pat[i]->Nei2;
    else 
      continue;
    
    PrevPat = Pat[i];
    while( CurrPat->Nei1 != NULL && CurrPat->Nei2 != NULL ) {
      
      if( (CurrPat->Nei1->Nei1 == CurrPat || CurrPat->Nei1->Nei2 == CurrPat) && 
             CurrPat->Nei1 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei1;
      }
      else 
      if( (CurrPat->Nei2->Nei1 == CurrPat || CurrPat->Nei2->Nei2 == CurrPat) && 
             CurrPat->Nei2 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei2;
      }
      else 
             break;
    }  
    
    Alias(md,&B1D,&B1A,&B2D,&B2A,&B1DCn,&B1ACn,&B2DCn,&B2ACn,Pat[i]);
    Alias(md,&E1D,&E1A,&E2D,&E2A,&E1DCn,&E1ACn,&E2DCn,&E2ACn,CurrPat);

    if( (Cn1 != Cn2 || E1D - B2A <  E2D - B2A ) &&
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2A,E1D,E2ACn,&Beg2,E2D,E1A,&Beg2Cn,E2DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1D,E2A,E1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2D - B2A <  E1D - B2A ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1A,E2D,E1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2D,E1A,E2DCn,&Beg2,E2A,E1D,&Beg2Cn,E2ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2A - E1D < B2A - E2D ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2D,E1A,E2DCn,&Beg2,E2A,E1D,&Beg2Cn,E2ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1A,E2D,E1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2A - E2D < B2A - E1D ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1D,E2A,E1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2A,E1D,E2ACn,&Beg2,E2D,E1A,&Beg2Cn,E2DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B1D - E2A <  B2D - E2A ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2A,B1D,B2ACn,&Beg2,B2D,B1A,&Beg2Cn,B2DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1D,B2A,B1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2D - E2A <  B1D - E2A ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1A,B2D,B1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2D,B1A,B2DCn,&Beg2,B2A,B1D,&Beg2Cn,B2ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2A - B1D < E2A - B2D ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2D,B1A,B2DCn,&Beg2,B2A,B1D,&Beg2Cn,B2ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1A,B2D,B1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2A - B2D < E2A - B1D ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1D,B2A,B1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2A,B1D,B2ACn,&Beg2,B2D,B1A,&Beg2Cn,B2DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
      continue;


    if( Beg1Cn == md->Chains[Cn1].id[0] ) {
      for( j=Beg1; j<=End1; j++ ) 
             Asn1[j] = 'N';
      for( j=Beg2; j<=End2; j++ ) 
             Asn2[j] = 'N';
    }
    else {
      for( j=Beg1; j<=End1; j++ ) 
             Asn2[j] = 'N';
      for( j=Beg2; j<=End2; j++ ) 
             Asn1[j] = 'N';
    }

    Pat[i]->Nei1 = NULL;
    Pat[i]->Nei2 = NULL;
    CurrPat->Nei1 = NULL;
    CurrPat->Nei2 = NULL;

  }
}
  

void FillAsnPar(char *Asn1, char *Asn2, Model *md, int Cn1, int Cn2, 
             MYPATTERN **Pat, int NPat)
{
  int i, j;
  int Beg1, Beg2, End1, End2;
  int B1D, B1A, B2D, B2A, E1D, E1A, E2D, E2A; 
  char B1DCn, B1ACn, B2DCn, B2ACn, E1DCn, E1ACn, E2DCn, E2ACn, Beg1Cn, Beg2Cn; 
  MYPATTERN *CurrPat, *PrevPat;;

  for( i=0; i<NPat; i++ ) {
    
    if( Pat[i]->Nei1 != NULL && Pat[i]->Nei2 == NULL )
      CurrPat = Pat[i]->Nei1;
    else
    if( Pat[i]->Nei2 != NULL && Pat[i]->Nei1 == NULL ) 
      CurrPat = Pat[i]->Nei2;
    else 
      continue;
    
    PrevPat = Pat[i];
    while( CurrPat->Nei1 != NULL && CurrPat->Nei2 != NULL ) {
      
      if( (CurrPat->Nei1->Nei1 == CurrPat || CurrPat->Nei1->Nei2 == CurrPat) && 
             CurrPat->Nei1 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei1;
      }
      else {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei2;
      }
    }  

    Alias(md,&B1D,&B1A,&B2D,&B2A,&B1DCn,&B1ACn,&B2DCn,&B2ACn,Pat[i]);
    Alias(md,&E1D,&E1A,&E2D,&E2A,&E1DCn,&E1ACn,&E2DCn,&E2ACn,CurrPat);

    if( ( Cn1 != Cn2 || abs(E1D-B2A) < abs(E2D-B2A) ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2A,E1D,E2ACn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E2D,E1A,E2DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1D,E2A,E1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2D-B2A) < abs(E1D-B2A) ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1A,E2D,E1ACn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2D,E1A,E2DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E2A,E1D,E2ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2A-E1D) < abs(B2A-E2D) ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2D,E1A,E2DCn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E2A,E1D,E2ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1A,E2D,E1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2A-E2D) < abs(B2A-E1D) ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1D,E2A,E1DCn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2A,E1D,E2ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E2D,E1A,E2DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B1D-E2A) < abs(B2D-E2A) ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2A,B1D,B2ACn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B2D,B1A,B2DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1D,B2A,B1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2D-E2A) < abs(B1D-E2A) ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1A,B2D,B1ACn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2D,B1A,B2DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B2A,B1D,B2ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2A-B1D) < abs(E2A-B2D) ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2D,B1A,B2DCn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B2A,B1D,B2ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1A,B2D,B1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2A-B2D) < abs(E2A-B1D) ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1D,B2A,B1DCn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2A,B1D,B2ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B2D,B1A,B2DCn,Pat,NPat) ) )
      ;
    else 
      continue;

    if( Beg1Cn == md->Chains[Cn1].id[0] ) {
      for( j=Beg1; j<=End1; j++ ) Asn1[j] = 'P';
      for( j=Beg2; j<=End2; j++ ) Asn2[j] = 'P';
    }
    else {
      for( j=Beg1; j<=End1; j++ ) Asn2[j] = 'P';
      for( j=Beg2; j<=End2; j++ ) Asn1[j] = 'P';
    }

    Pat[i]->Nei1 = NULL;
    Pat[i]->Nei2 = NULL;
    CurrPat->Nei1 = NULL;
    CurrPat->Nei2 = NULL;

  }
}
  

  
void FilterAntiPar(Model *md, MYPATTERN **Pat, int NPat)
{
 
  int i, j;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {
    
    if( !Pat[i]->ExistPattern ) continue;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);
    
    for( j=0; j<NPat; j++ ) {
      
      if( j == i || !Pat[j]->ExistPattern ) continue;
      
      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( J1D == J2A && J2D == J1A && I1D != I2A && I2D != I1A &&
             ( (J1D == I1D && J1A == I1A) ||  (J1D == I1A && J1A == I1D) || 
             (J1D == I2A && J1A == I2D) ||  (J1D == I2D && J1A == I2A) ) ) continue;

      if( ( ( I1D < I2A || I2D < I1A ) && 
             ( (J1A <= I2A && J1A >= I1D && J2D <= I2A && J2D >= I1D && J2DCn == I1DCn &&
             J2A <= I1A && J2A >= I2D && J1D <= I1A && J1D >= I2D && J1DCn == I2DCn) ||
             (J2A <= I2A && J2A >= I1D && J1D <= I2A && J1D >= I1D && J1DCn == I1DCn &&
             J1A <= I1A && J1A >= I2D && J2D <= I1A && J2D >= I2D && J2DCn == I2DCn) ) ) || 
             ( ( I1D > I2A || I2D > I1A ) && 
             ( (J1A >= I2A && J1A <= I1D && J2D >= I2A && J2D <= I1D && J2DCn == I1DCn &&
             J2A >= I1A && J2A <= I2D && J1D >= I1A && J1D <= I2D && J1DCn == I2DCn) ||
             (J2A >= I2A && J2A <= I1D && J1D >= I2A && J1D <= I1D && J1DCn == I1DCn &&
             J1A >= I1A && J1A <= I2D && J2D >= I1A && J2D <= I2D && J2DCn == I2DCn) ) ) ) {
             Pat[j]->ExistPattern = false;
      }
    }
  }
}

void FilterPar(Model *md, MYPATTERN **Pat, int NPat)
{
 
  int i, j;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {
    
    if( !Pat[i]->ExistPattern ) continue;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);
    
    for( j=0; j<NPat; j++ ) {
      
      if( j == i || !Pat[j]->ExistPattern ) continue;
      
      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( ( ( I1A >= I2D && I1D >= I2A ) && 
             ( (J1A >= I2A && J1A <= I1D && J2D >= I2A && J2D <= I1D && J2DCn == I1DCn &&
             J2A <= I1A && J2A >= I2D && J1D <= I1A && J1D >= I2D && J1DCn == I2DCn) ||
             (J2A >= I2A && J2A <= I1D && J1D >= I2A && J1D <= I1D && J1DCn == I1DCn &&
             J1A <= I1A && J1A >= I2D && J2D <= I1A && J2D >= I2D && J2DCn == I2DCn) ) ) || 

             ( I2A >= I1D && I2D >= I1A  && 
             ( (J1A <= I2A && J1A >= I1D && J2D <= I2A && J2D >= I1D && J2DCn == I1DCn &&
             J2A >= I1A && J2A <= I2D && J1D >= I1A && J1D <= I2D && J1DCn == I2DCn) ||

             (J2A <= I2A && J2A >= I1D && J1D <= I2A && J1D >= I1D && J1DCn == I1DCn &&
             J1A >= I1A && J1A <= I2D && J2D >= I1A && J2D <= I2D && J2DCn == I2DCn) ) ) ) {
             Pat[j]->ExistPattern = false;
      }
    }
  }
}
      
void Bridge(char *Asn1, char *Asn2, int Cn1, int Cn2, MYPATTERN **Pat, int NPat)
{

  int i;
  int B_Res;
  
  for( i=0; i<NPat; i++ ) {
    if( Pat[i]->Nei1 != NULL || Pat[i]->Nei2 != NULL ) continue;

    if( !Pat[i]->Type.compare("1331") && 
       ( Cn1 != Cn2 || abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) >= 3 ) ) {
      
      if( Pat[i]->Hb1->Dnr->Chain == Cn1)
      {
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             if( Asn2[Pat[i]->Hb1->Acc->A_Res] == 'C' )
             Asn2[Pat[i]->Hb1->Acc->A_Res] = 'B';
      }
      else 
      {
             if( Asn2[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             if( Asn1[Pat[i]->Hb1->Acc->A_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Acc->A_Res] = 'B';
      }

    }
    else
      if( !Pat[i]->Type.compare("3124") && 
             ( Cn1 != Cn2 || 
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) >= 2 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) >= 2 ) ) ) {
      
             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;

             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';

             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;

             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;

             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';

             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;

             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';
             }
      }
      else
             if( ( ( !Pat[i]->Type.compare("3123") || !Pat[i]->Type.compare("1341") ) &&
             ( Cn1 != Cn2 ||
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) > 3 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) > 3 ) ) ) ) { 

             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res ) {
             
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;
             
             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             
             if( Asn2[Pat[i]->Hb2->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb2->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;
             
             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';
             }
             }
             }
             else
             if( ( !Pat[i]->Type.compare("13B1") || !Pat[i]->Type.compare("133A") ) &&
             ( Cn1 != Cn2 ||
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) > 4 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) > 4 ) ) ) { 

             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res ) {
             
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;
             
             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             
             if( Asn2[Pat[i]->Hb2->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb2->Dnr->D_Res] = 'b';
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;
             
             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'b';
             }
             }
             }
    
  }
}

bool RightSide2(int L_A1, int L_D1, int LnkD, int LnkA, int I1A, int I1D, int I2A, int I2D)
{

  if( ( I2A < I1D && LnkA <= I1D && LnkA <= I2A ) || 
      ( I2A > I1D && LnkA >= I1D && LnkA >= I2A ) ||
      ( I2D < I1A && LnkD <= I1A && LnkA <= I2D ) || 
      ( I2D > I1A && LnkD >= I1A && LnkD >= I2D )  )
    return(true);
  else
    if( I2A == I1D && I2D == I1A ) {
      if( L_A1 != ERR && 
             ( ( LnkD <= I2D && L_A1 <= I2D && LnkA >= I2A && L_D1 >= I2A ) || 
             ( LnkD >= I2D && L_A1 >= I2D && LnkA <= I2A && L_D1 <= I2A ) ) )
             return(false);
      else
             return(true);
    }

  return(false);
}

bool RightSide(int LnkA, int LnkD, int I1A, int I1D, int I2A, int I2D )
{

  if( ( I1A == I2D && I1D == I2A ) ||
      ( I1A < I2D && LnkA <= I2D && LnkA <= I1A ) || 
      ( I1A > I2D && LnkA >= I2D && LnkA >= I1A ) ||
      ( I1D < I2A && LnkD <= I2A && LnkD <= I1D ) ||
      ( I1D > I2A && LnkD >= I2A && LnkD >= I1D ) )
    return(true);
  
  return(false);
}

bool RightSidePar(int LnkA, int LnkD, int I1A, int I1D, int I2A, int I2D )
{

  if( ( I1A == I2D && I1D == I2A ) ||
      ( I1A < I2D && LnkA < I2D && LnkA <= I1A && I1D <= I2A && LnkD <= I2A && LnkD <= I1D ) || 
      ( I1A > I2D && LnkA > I2D && LnkA >= I1A && I1D >= I2A && LnkD >= I2A && LnkD >= I1D ) ||
      ( I1D < I2A && LnkD < I2A && LnkD <= I1D && I1A <= I2D && LnkA <= I2D && LnkA <= I1A ) ||
      ( I1D > I2A && LnkD > I2A && LnkD >= I1D && I1A >= I2D && LnkA >= I2D && LnkA >= I1A) )
    return(true);
  
  return(false);
}

void JoinNeighbours(int *Lnk1A, int Res1, int *Lnk1D, int Res2, MYPATTERN **Nei, 
             MYPATTERN *Pat, int *MinDB1, int DB, int *MinDW1, int DW, int *Min, int j)
{
  *Lnk1A = Res1;
  *Lnk1D = Res2;
  (*Nei) = Pat;
  *MinDB1 = DB;
  *MinDW1 = DW;
  *Min = j;
}

void JoinNeighb(MYPATTERN **Nei, MYPATTERN *Pat, int *MinDB2, int DB, int *MinDW2, int DW)
{
  (*Nei) = Pat;
  *MinDB2 = DB;
  *MinDW2 = DW;
}

bool NearPar(int Res1, int Res2, int Res3, int Res4, int Res5, int Res6, int Res7, int Res8,
             char Cn1, char Cn2, char Cn3, char Cn4, int *DistBest, int *DistWorst)
{

/*
   Res5 Res2 Res1
   Res6 Res3 Res4
*/

  int a, b, c1, d1, c, d, Nei1, Nei2;

  if( Cn1 != Cn2 || Cn3 != Cn4 ) return(false);

  if( Res1 >= Res2 && Res2 >= Res5 && Res7 >= Res1 && 
      Res4 >= Res3 && Res4 >= Res6 && Res8 >= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2-1;

    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1+1;

    a = Nei2-Nei1;
    c1 = Nei2-Res5;

    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3-1;

    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4+1;

    b = Nei2-Nei1;
    d1 = Nei2-Res6;

  }
  else
  if( Res1 <= Res2 && Res2 <= Res5 && Res7 <= Res1 && 
      Res4 <= Res3 && Res4 <= Res6 && Res8 <= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2+1;

    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1-1;

    a = Nei1-Nei2;
    c1 = Res1-Res7;

    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3+1;

    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4-1;

    b = Nei1-Nei2;
    d1 = Nei1-Res8;


  }
  else
   return(false);

  c = max(c1,a);
  d = max(d1,b);

  if( a >= 0 && b >= 0 && c >= 0 && d >= 0 && 
      ( (a <= 2 && b <= 5) || (a <= 5 && b <= 2) ) ) {
    *DistBest  = min(a,b);
    *DistWorst = max(c,d);
    if( *DistBest <= *DistWorst ) 
      return(true);
    else
      return(false);
  }
  
  return(false);
}

bool Near(int Res1, int Res2, int Res3, int Res4, int Res5, int Res6, int Res7, int Res8,
             char Cn1, char Cn2, char Cn3, char Cn4, int *DistBest, int *DistWorst)
{

/*
   Res5 Res2 Res1
   Res6 Res3 Res4
*/

  int a, b, c1, d1, c, d, Nei1, Nei2;


  if( Cn1 != Cn2 || Cn3 != Cn4 ) return(false);


  if( Res1 >= Res2 && Res2 >= Res5 && Res7 >= Res1 && 
      Res4 <= Res3 && Res4 <= Res6 && Res8 <= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2-1;
    
    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1+1;
    
    a = Nei2-Nei1;
    c1 = Nei2-Res5;
    
    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3+1;
    
    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4-1;
    
    b = Nei1-Nei2;
    d1 = Res6-Nei2;
  }
  else
    return(false);
  
  c = max(c1,a);
  d = max(d1,b);
  
  if( a >= 0 && b >= 0 && c >= 0 && d >= 0 && 
      ( (a <= 2 && b <= 5) || (a <= 5 && b <= 2) ) ) {
    *DistBest  = min(a,b);
    *DistWorst = max(c,d);
    if( *DistBest <= *DistWorst ) 
      return(true);
    else
      return(false);
  }
  
  return(false);
}

void MergePatternsAntiPar(Model *md, MYPATTERN **Pat, int NPat)
{
  int i, j;
  int DB, DW, MinDB1, MinDB2, MinDW1, MinDW2, Min, Lnk1A, Lnk1D;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {

    if( !Pat[i]->ExistPattern ) continue;

    MinDB1 = MinDB2 = MinDW1 = MinDW2 = 1000;
    Min = ERR;
    Lnk1D = Lnk1A = ERR;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);

    for( j=0; j<NPat; j++ ) {

      if( i == j || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( Near(I1D,J1D,J1A,I1A,J2A,J2D,I2A,I2D,I1DCn,J1DCn,J1ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J1A,J1D,I1A,J2D,J2A,I2A,I2D,I1DCn,J1ACn,J1DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J2D,J2A,I1A,J1A,J1D,I2A,I2D,I1DCn,J2DCn,J2ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J1A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J2A,J2D,I1A,J1D,J1A,I2A,I2D,I1DCn,J2ACn,J2DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J1D,J1A,I1D,J2A,J2D,I2D,I2A,I1ACn,J1DCn,J1ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J1A,J1D,I1D,J2D,J2A,I2D,I2A,I1ACn,J1ACn,J1DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J2D,J2A,I1D,J1A,J1D,I2D,I2A,I1ACn,J2DCn,J2ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J2A,J2D,I1D,J1D,J1A,I2D,I2A,I1ACn,J2ACn,J2DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

    }

    for( j=0; j<NPat; j++ ) {

      if( j == Min || j == i || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);

      if( Near(I2D,J1D,J1A,I2A,J2A,J2D,I1A,I1D,I2DCn,J1DCn,J1ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J1A,J1D,I2A,J2D,J2A,I1A,I1D,I2DCn,J1ACn,J1DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J2D,J2A,I2A,J1A,J1D,I1A,I1D,I2DCn,J2DCn,J2ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) ) 
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J2A,J2D,I2A,J1D,J1A,I1A,I1D,I2DCn,J2ACn,J2DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J1D,J1A,I2D,J2A,J2D,I1D,I1A,I2ACn,J1DCn,J1ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J1A,J1D,I2D,J2D,J2A,I1D,I1A,I2ACn,J1ACn,J1DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J2D,J2A,I2D,J1A,J1D,I1D,I1A,I2ACn,J2DCn,J2ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J2A,J2D,I2D,J1D,J1A,I1D,I1A,I2ACn,J2ACn,J2DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      }
  }
}

void MergePatternsPar(Model *md, MYPATTERN **Pat, int NPat)
{
  int i, j;
  int DB, DW, MinDB1, MinDB2, MinDW1, MinDW2, Min, Lnk1A, Lnk1D;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {

    if( !Pat[i]->ExistPattern ) continue;

    MinDB1 = MinDB2 = MinDW1 = MinDW2 = 1000;
    Min = ERR;
    Lnk1D = Lnk1A = ERR;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);

    for( j=0; j<NPat; j++ ) {

      if( i == j || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( NearPar(I1D,J1D,J1A,I1A,J2A,J2D,I2A,I2D,I1DCn,J1DCn,J1ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J1A,J1D,I1A,J2D,J2A,I2A,I2D,I1DCn,J1ACn,J1DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J2D,J2A,I1A,J1A,J1D,I2A,I2D,I1DCn,J2DCn,J2ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J1A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J2A,J2D,I1A,J1D,J1A,I2A,I2D,I1DCn,J2ACn,J2DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J1D,J1A,I1D,J2A,J2D,I2D,I2A,I1ACn,J1DCn,J1ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J1A,J1D,I1D,J2D,J2A,I2D,I2A,I1ACn,J1ACn,J1DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J2D,J2A,I1D,J1A,J1D,I2D,I2A,I1ACn,J2DCn,J2ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J2A,J2D,I1D,J1D,J1A,I2D,I2A,I1ACn,J2ACn,J2DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

    }

    for( j=0; j<NPat; j++ ) {

      if( j == Min || j == i || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);

      if( NearPar(I2D,J1D,J1A,I2A,J2A,J2D,I1A,I1D,I2DCn,J1DCn,J1ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J1A,J1D,I2A,J2D,J2A,I1A,I1D,I2DCn,J1ACn,J1DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J2D,J2A,I2A,J1A,J1D,I1A,I1D,I2DCn,J2DCn,J2ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) ) 
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J2A,J2D,I2A,J1D,J1A,I1A,I1D,I2DCn,J2ACn,J2DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J1D,J1A,I2D,J2A,J2D,I1D,I1A,I2ACn,J1DCn,J1ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J1A,J1D,I2D,J2D,J2A,I1D,I1A,I2ACn,J1ACn,J1DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J2D,J2A,I2D,J1A,J1D,I1D,I1A,I2ACn,J2DCn,J2ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J2A,J2D,I2D,J1D,J1A,I1D,I1A,I2ACn,J2ACn,J2DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      }
  }
}

void Sheet(Model *md, int Cn1, int Cn2, HBOND **HBond, float **PhiPsiMap)
{
  MYPATTERN **PatN, **PatP;
  int  Res1, Res3, Res2, Res4, ResA, ResB, Res1m1, Res3p1;
  int R1, R3, R2, R4, RA, RB, PatCntN = 0, PatCntP = 0, Beg;
  char *AntiPar1, *Par1, *AntiPar2, *Par2;

  PatN = new MYPATTERN * [MAXHYDRBOND]();
  PatP = new MYPATTERN * [MAXHYDRBOND]();
  AntiPar1  = new char [md->Chains[Cn1].nAmino]();       /* Antiparallel strands */
  Par1      = new char [md->Chains[Cn1].nAmino]();       /* Parallel strands */
  AntiPar2  = new char [md->Chains[Cn2].nAmino]();       /* Antiparallel strands */
  Par2      = new char [md->Chains[Cn2].nAmino]();       /* Parallel strands */

  for( int i=0; i<md->Chains[Cn1].nAmino; i++ ) 
  { 
       AntiPar1[i] = 'C'; 
       Par1[i] = 'C'; 
  }

  for( int i=0; i<md->Chains[Cn2].nAmino; i++ ) 
  { 
       AntiPar2[i] = 'C'; 
       Par2[i] = 'C'; 
  }

  for( R1=0; R1<md->Chains[Cn1].nAmino; R1++ ) 
  {
        Res1   = md->GetNumResiduo(Cn1, R1);
        if((!involved[Res1].NBondDnr && !involved[Res1].NBondAcc) || 
            ((Cn1 != Cn2) && !involved[Res1].InterchainHBonds) )
           continue;
        RA     = R1+1;
        R2     = R1+2;
        Res1m1 = md->GetNumResiduo(Cn1, R1-1);
        ResA   = md->GetNumResiduo(Cn1, RA);
        Res2   = md->GetNumResiduo(Cn1, R2);

        if( R2 >= md->Chains[Cn1].nAmino || RA >= md->Chains[Cn1].nAmino || 
            properties[Res1].PhiZn == ERR || properties[Res1].PsiZn == ERR ||
            properties[Res2].PhiZn == ERR || properties[Res2].PsiZn == ERR ||
            properties[ResA].PhiZn == ERR || properties[ResA].PsiZn == ERR ) 
          continue;

        if( Cn1 != Cn2 ) 
          Beg = 0;
        else
          Beg = R1+1;

        for( R3=Beg; R3<md->Chains[Cn2].nAmino; R3++ )
        {
// Process anti-parallel strands
             Res3   = md->GetNumResiduo(Cn2, R3);
             if( (!involved[Res3].NBondAcc && !involved[Res3].NBondDnr ) || 
             ((Cn1 != Cn2) && !involved[Res3].InterchainHBonds) ) 
             continue;
             RB     = R3-1;
             R4     = R3-2;
             Res3p1 = md->GetNumResiduo(Cn2, R3+1);
             ResB   = md->GetNumResiduo(Cn2, RB);
             Res4   = md->GetNumResiduo(Cn2, R4);
      
             if( Cn1 != Cn2 || R3 - R1 >= 3  )
             Link(HBond,Res1,Res3,Res3,Res1,Res1,Res3,
             PhiPsiMap,PatN,&PatCntN,"1331",Treshold_E1,0);
      
             if( R2 < md->Chains[Cn1].nAmino && ((Cn1 != Cn2 && R4 >= 0) || R4-R2 >=2 ) )
             Link(HBond,Res3,Res1,Res2,Res4,ResB,ResA,
             PhiPsiMap,PatN,&PatCntN,"3124",Treshold_E1,0);
      
             if( ((Cn1 != Cn2 && RB >= 0 ) || RB-R1 > 4) && 
             ( RA >= md->Chains[Cn1].nAmino || (Cn1 == Cn2 && R3-RA <= 4 ) ||
             !Link(HBond,Res1,Res3,Res3,ResA,-1,Res3,
             PhiPsiMap,PatN,&PatCntN,"133A",Treshold_E1,1))
             &&
             ( R1-1 < 0 ||
             !Link(HBond,Res1m1,ResB,ResB,Res1,-1,ResB,
             PhiPsiMap,PatN,&PatCntN,"1-BB1",Treshold_E1,1)))
             Link(HBond,Res1,Res3,ResB,Res1,Res1,-1,
             PhiPsiMap,PatN,&PatCntN,"13B1",Treshold_E1,0);
      
             if( (RA < md->Chains[Cn1].nAmino && (Cn1 != Cn2 || R3-RA > 4)) &&
             ( (Cn1 == Cn2 && RB-R1 <= 4 ) || (Cn1 != Cn2 && RB < 0 ) || 
             !Link(HBond,Res1,Res3,ResB,Res1,Res1,-1,
             PhiPsiMap,PatN,&PatCntN,"13B1",Treshold_E1,1))
             &&
             ( R3+1 >= md->Chains[Cn2].nAmino ||
             !Link(HBond,ResA,Res3p1,Res3,ResA,ResA,-1,
             PhiPsiMap,PatN,&PatCntN,"A3+3A",Treshold_E1,1)))
             Link(HBond,Res1,Res3,Res3,ResA,-1,Res3,
             PhiPsiMap,PatN,&PatCntN,"133A",Treshold_E1,0);
      
// Process parallel strands */

                R4 = R3+2; 
                RB = R3+1;
                ResB   = md->GetNumResiduo(Cn2, RB);
                Res4   = md->GetNumResiduo(Cn2, R4);

                if( (Cn1 == Cn2 && abs(R3-R1) <= 3) || R4 >= md->Chains[Cn2].nAmino )
                     continue;
      
      if( R2 < md->Chains[Cn1].nAmino && (Cn1 != Cn2 || abs(R2-R3) > 3) )
             Link(HBond,Res3,Res1,Res2,Res3,Res3,ResA,
             PhiPsiMap,PatP,&PatCntP,"3123",Treshold_E2,0);
      
      if( R4 < md->Chains[Cn2].nAmino && (Cn1 != Cn2 || abs(R4-R1) > 3) )
             Link(HBond,Res1,Res3,Res4,Res1,Res1,ResB,
             PhiPsiMap,PatP,&PatCntP,"1341",Treshold_E2,0);
    }
  }
    
  FilterAntiPar(md,PatN,PatCntN);
  FilterPar(md,PatP,PatCntP);

  MergePatternsAntiPar(md,PatN,PatCntN);
  MergePatternsPar(md,PatP,PatCntP);

  FillAsnAntiPar(AntiPar1,AntiPar2,md,Cn1,Cn2,PatN,PatCntN);
  FillAsnPar(Par1,Par2,md,Cn1,Cn2,PatP,PatCntP);

  Bridge(AntiPar1,AntiPar2,Cn1,Cn2,PatN,PatCntN);
  Bridge(Par1,Par2,Cn1,Cn2,PatP,PatCntP);

  for( int i=0; i<md->Chains[Cn1].nAmino; i++ )
  {
    int ir = md->GetNumResiduo(Cn1, i);
    if( AntiPar1[i] == 'N' || Par1[i] == 'P' ) 
      properties[ir].Asn = 'E';
    else if( AntiPar1[i] == 'B' || Par1[i] == 'B' )
      properties[ir].Asn = 'B';
    else if( AntiPar1[i] == 'b' || Par1[i] == 'b' )
      properties[ir].Asn = 'b';
  }
  for( int i=0; i<md->Chains[Cn2].nAmino; i++ )
  {
    int ir = md->GetNumResiduo(Cn2, i);
    if( properties[ir].Asn == 'E' )
      continue;
    else if( AntiPar2[i] == 'N' || Par2[i] == 'P' ) 
      properties[ir].Asn = 'E';
    else if( AntiPar2[i] == 'B' || Par2[i] == 'B' ) 
      properties[ir].Asn = 'B';
    else if( AntiPar2[i] == 'b' || Par2[i] == 'b' ) 
      properties[ir].Asn = 'b';
   }
  delete [](PatN);
  delete [](PatP);
  delete [](AntiPar1);
  delete [](Par1);
  delete [](AntiPar2);
  delete [](Par2);
}

bool TurnCondition(float Phi2,float Phi2S,float Psi2,float Psi2S,
                  float Phi3,float Phi3S,float Psi3,float Psi3S,
                  float Range1,float Range2)
{
  if((DENTRO(Phi2,Phi2S,Range2) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range2) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range2) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range2))
     )
    return(true);

  return(false);
}

void BetaTurn(Model *md, int Cn)
{

  RESIDUE r, r4;
  int CA1, CA4;
  float dist14;

  for(int i=0; i<md->Chains[Cn].nAmino-4; i++ ) 
  {
    r  = md->Chains[Cn].AminoRes[i];
    r4 = md->Chains[Cn].AminoRes[i+3];
    int Res  = md->GetNumResiduo(Cn, i);
    int Res1 = md->GetNumResiduo(Cn, i+1);
    int Res2 = md->GetNumResiduo(Cn, i+2);
    int Res3 = md->GetNumResiduo(Cn, i+3);

    char asn  = properties[Res ].Asn;
    char asn1 = properties[Res1].Asn;
    char asn2 = properties[Res2].Asn;
    char asn3 = properties[Res3].Asn;
    
    CA1 = md->FindAtom("CA",r, 6);
    CA4 = md->FindAtom("CA",r4, 6);
    if( (CA1 != -1) && (CA4 != -1) )
        dist14 = Distanza(md->atm[CA1].xyz,md->atm[CA4].xyz);
    else
        dist14 = 10;
    if( asn1 == 'H' || asn2 == 'H' || 
        asn1 == 'G' || asn2 == 'G' || 
        asn1 == 'I' || asn2 == 'G' || 
        (CA1 == -1)  || (CA4 == -1)  ||
        dist14 > 7.0 )
      continue;
    
// Informazioni dettagliate
/*
    float Phi2, Phi3, Psi2, Psi3, Range1 = 30.0, Range2 = 45.0;
    char TurnType;
    Phi2 = properties[Res1].Phi;
    Psi2 = properties[Res1].Psi;
    Phi3 = properties[Res2].Phi;
    Psi3 = properties[Res2].Psi;
    
    if( TurnCondition(Phi2,-60.0,Psi2,-30,Phi3,-90.0,Psi3,0,Range1,Range2) )
      TurnType = '1';
    else
    if( TurnCondition(Phi2,60.0,Psi2,30,Phi3,90.0,Psi3,0,Range1,Range2) )
      TurnType = '2';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,120,Phi3,80.0,Psi3,0,Range1,Range2) )
      TurnType = '3';
    else
    if( TurnCondition(Phi2,60.0,Psi2,-120,Phi3,-80.0,Psi3,0,Range1,Range2) )
      TurnType = '4';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,120,Phi3,-90.0,Psi3,0,Range1,Range2) )
      TurnType = '5';
    else
    if( TurnCondition(Phi2,-120.0,Psi2,120,Phi3,-60.0,Psi3,0,Range1,Range2) )
      TurnType = '6';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,-30,Phi3,-120.0,Psi3,120,Range1,Range2) )
      TurnType = '7';
    else
      TurnType = '8';
*/
// Fine informazioni dettagliate
  
    if( asn == 'C' ) 
      properties[Res ].Asn= 'T';
    
    if( asn1 == 'C' )
      properties[Res1].Asn = 'T';
    
    if( asn2 == 'C' )
      properties[Res2].Asn = 'T';
    
    if( asn3 == 'C' )
      properties[Res3].Asn = 'T';
  }
}

void GammaTurn(Model *md, int Cn, HBOND **HBond)
{
  ResidueProperty rp, rp1, rp2, rp3, rp4;
  int Res=0, Res1, Res2, Res3, Res4=0;
  float Phi2, Psi2;
  char Asn;

  for( int i=0; i<md->Chains[Cn].nAmino-2; i++ ) 
  {
    if(i > 0)
    {
       Res = md->GetNumResiduo(Cn, i-1);
       rp = properties[Res];
    }
    Res1 = md->GetNumResiduo(Cn, i);
    rp1 = properties[Res1];
    Res2 = md->GetNumResiduo(Cn, i+1);
    rp2 = properties[Res2];
    Res3 = md->GetNumResiduo(Cn, i+2);
    rp3 = properties[Res3];
    if(i != md->Chains[Cn].nAmino-3)
    {
       Res4 = md->GetNumResiduo(Cn, i+3);
       rp4 = properties[Res4];
    }
    Asn = rp2.Asn;

    if( Asn == 'H' || Asn == 'T' || Asn == 'G' || Asn == 'I' ||
        FindBnd(HBond,i+2,i) == ERR ||
        (i > 0 && FindBnd(HBond,Res3,Res) != ERR) || 
        (i < md->Chains[Cn].nAmino-3 && FindBnd(HBond,Res4,Res1) != ERR) )
      continue;
    
    Phi2 = rp2.Phi;
    Psi2 = rp2.Psi;
    
/*
    char TurnType;
    if( Phi2 > 0.0 && Psi2 < 0.0 )
      TurnType = '@';
    else
    if( Phi2 < 0.0 && Psi2 > 0.0 )
      TurnType = '&';
    else 
      continue;
*/
    if( (Phi2*Psi2) >= 0)
      continue;

    if( rp1.Asn == 'C' )
      rp1.Asn = 'T';
    
    if( rp2.Asn == 'C' )
      rp2.Asn = 'T';
    
    if( rp3.Asn == 'C' )
      rp3.Asn = 'T';
  }
}

bool DetectSecondary(Model *md, std::string &msg)
{
      properties = new ResidueProperty [md->nResTot]();
      involved = new  Involved [md->nResTot]();

//Backbone Angles
      for(int ch=0; ch<md->nChains; ch++)
      {
          for(int j=0; j< md->Chains[ch].nAmino; j++)
          {
              int curr = md->GetNumResiduo(ch, j);
              properties[curr].Asn = 'C';
              properties[curr].Phi = 360.;
              RESIDUE cur = md->Chains[ch].AminoRes[j];
              if(j != 0)
              {
                 RESIDUE prec = md->Chains[ch].AminoRes[j-1];
                 if( (prec.c != -1) && (cur.n != -1) &&
                     (cur.ca != -1) && (cur.c != -1) &&
                     (Distanza(md->atm[prec.c].xyz, md->atm[cur.n].xyz) < BREAKDIST) )
                        properties[curr].Phi = Torsione(md->atm[prec.c].xyz,
                                                     md->atm[cur.n].xyz,
                                                     md->atm[cur.ca].xyz,
                                                     md->atm[cur.c].xyz);
              }
              properties[curr].Psi = 360.;
              if(j != (md->Chains[ch].nAmino-1))
              {
                 RESIDUE next = md->Chains[ch].AminoRes[j+1];
                 if( (next.n != -1) && (cur.n != -1) &&
                     (cur.ca != -1) && (cur.c != -1) &&
                     (Distanza(md->atm[cur.c].xyz, md->atm[next.n].xyz) < BREAKDIST) )
                        properties[curr].Psi = Torsione(md->atm[cur.n].xyz,
                                                        md->atm[cur.ca].xyz,
                                                        md->atm[cur.c].xyz,
                                                        md->atm[next.n].xyz);
              }
          }
      }

      float **PhiPsiMapHelix, **PhiPsiMapSheet;
      PhiPsiMapHelix = DefaultHelixMap();
      PhiPsiMapSheet = DefaultSheetMap();
      int h_cnt = PlaceHydrogens(md, md->c_alpha);
      if(h_cnt == 0)
      {
         msg = "No hydrogen placed \n";
         return false;
      }
      int NHBond;
      HBOND **HBond = new HBOND *[50000]();
      if( (NHBond = FindHydrogenBonds(md, HBond, msg)) == 0 )
      {
          msg = "No hydrogen bonds found \n";
          return false;
      }
      NoDoubleHBond(HBond,NHBond);
   
      DiscrPhiPsi(md);
      for(int Cn=0; Cn<md->nChains; Cn++)
      {
        if(md->Chains[Cn].nAmino > 5)
        {
          Helix(md, Cn, HBond, PhiPsiMapHelix);
          for(int i=0; i<md->nChains; i++)
          {
              if(md->Chains[i].nAmino > 5)
                  Sheet(md, Cn, i, HBond, PhiPsiMapSheet);
          }
          BetaTurn(md,Cn);
          GammaTurn(md,Cn,HBond);
        }
      }
      for(int ch=0; ch<md->nChains; ch++)
      {
          for(int j=0; j< md->Chains[ch].nAmino; j++)
          {
              RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
              if(Is_Amino(cur->code))
              {
                 cur->secondary = properties[cur->num].Asn;
                 for(int na=0; na<(int) cur->atoms.size(); na++)
                 {
                     int kat = cur->atoms[na];
                     md->Secondary[kat] = cur->secondary;
                 }
              }
          }
      }
  
      if(properties)
         delete [](properties);
      if(involved)
         delete [](involved);
      for( int i=0; i<NHBond; i++ ) 
         delete [](HBond[i]);
      delete [](HBond);
      delete [](PhiPsiMapHelix);
      delete [](PhiPsiMapSheet);
      return true;
}

int TypeS(char Code)
{
  switch(Code)
  {
  case 'H':
  case 'G':
  case 'I':
        return HELIX;
  case 'E':
  case 'S':
        return STRAND;
  default:
  case 'C':
        return COIL;
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '@':
  case '&':
  case 'T':
        return TURN;
  case 'D':
        return OTHER;
  }

}

const char *Translate(char Code)
{
  const char *Dictionary[] = {
    "AlphaHelix","310Helix","PiHelix","Strand","Bridge","Coil","TurnI","TurnI'",
    "TurnII","TurnII'","TurnVIa","TurnVIb","TurnVIII","TurnIV","GammaClassic",
    "GammaInv","Turn","Nucleic Acid"," ", "Unknown" };

  switch(Code) {
  case 'H': return(Dictionary[0]);
  case 'G': return(Dictionary[1]);
  case 'I': return(Dictionary[2]);
  case 'E': return(Dictionary[3]);
  case 'S': return(Dictionary[3]);
  case 'B':
  case 'b':
            return(Dictionary[4]);
  case ' ':
  case 'C': return(Dictionary[5]);
  case '1': return(Dictionary[6]);
  case '2': return(Dictionary[7]);
  case '3': return(Dictionary[8]);
  case '4': return(Dictionary[9]);
  case '5': return(Dictionary[10]);
  case '6': return(Dictionary[11]);
  case '7': return(Dictionary[12]);
  case '8': return(Dictionary[13]);
  case '@': return(Dictionary[14]);
  case '&': return(Dictionary[15]);
  case 'T': return(Dictionary[16]);
  case 'D': return(Dictionary[17]);
  case 'L': return(Dictionary[18]);
  default:  return(Dictionary[19]);
  }
}

void FillSecondary(Model *md)
{
   for(int i=0; i<(int)(sizeof(md->numbStructs)/sizeof(md->numbStructs[0])); i++)
      md->numbStructs[i] = 0;
   int tipo = -1;
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           if( tipo != TypeS(cur->secondary) )
           {
               tipo = TypeS(cur->secondary);
               (md->numbStructs[tipo])++;
           }
       }
   }
}

void PrintSecondary(Model *md, QString &string_sec)
{
   QTextStream os(&string_sec);
   
   os << "---------------- Secondary structure summary ---------------\n";

   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           char c = (cur->secondary != 'C')?cur->secondary:' ';
           os << c;
       }
       os << "\n";
   }
   os << "\n";
   os << "----------- Detailed Secondary structure summary -----------\n";
   os << "\n|--- Residue ---| |---- Structure ----|\n";
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           os << "     " << md->Chains[ch].id.c_str() << " "
	      <<  cur->name.c_str()
              << " " << cur->PDBnum.c_str() << "\t      "
              << cur->secondary << " " << Translate(cur->secondary)
              << "\n";
       }
       os << "\n";
   }
   os.flush();
}
