#include "spacegroupwidget.h"
#include "ui_spacegroupwidget.h"

#include <QDebug>

SpaceGroupWidget::SpaceGroupWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SpaceGroupWidget)
{
    ui->setupUi(this);

    sgModel = new QStringListModel(this);
    ui->spacegComboBox->setModel(sgModel);
    sgListIndex = 0;

    connect(ui->spacegComboBox, &QComboBox::currentTextChanged, this, [=](const QString &space){
        int pos = space.lastIndexOf('(');
        if (pos < 0) {
            emit spaceGroupChanged(space);
        } else {
            QString spg = space.left(pos).simplified();
            emit spaceGroupChanged(spg);
        }
    });
}

SpaceGroupWidget::~SpaceGroupWidget()
{
    delete ui;
}

void SpaceGroupWidget::setSpaceGroup(int index, const QString &spg)
{
    using namespace sgdat;

    if (index >= 0 && index < sizeSgTable) {
        int csys = systemFromSgNumber(TabSg[index].SgNumber);        
        ui->systemComboBox->setCurrentIndex(csys);
        ui->spacegComboBox->setCurrentIndex(index-sgListIndex);
    } else {
        ui->spacegComboBox->setCurrentText(spg);
    }
}

int SpaceGroupWidget::getSpaceGroup(QString &spg) const
{
    int index = ui->spacegComboBox->currentIndex() + sgListIndex;
    int pos = ui->spacegComboBox->currentText().lastIndexOf('(');

    spg = sgdat::TabSg[index].name;

    //Check for string edited
    QString spgText = ui->spacegComboBox->currentText().left(pos).simplified();
    if (spg.compare(spgText)) {
        spg = spgText;
        index = -1;
    }

    return index;
}

bool SpaceGroupWidget::isValidSpaceGroup() const
{
    // This function doesn't check if the edited space group is a valid string
    if (ui->systemComboBox->currentIndex() < 0) return false;
    if (ui->spacegComboBox->currentText().isEmpty()) return false;
    return true;
}

void SpaceGroupWidget::on_systemComboBox_currentIndexChanged(int index)
{
    fillCombo((sgdat::CrystalSystem) index);
    ui->spacegComboBox->setCurrentIndex(0);
}



void SpaceGroupWidget::fillCombo(sgdat::CrystalSystem csys)
{
    using namespace sgdat;    

    QStringList sgList;
    sgListIndex = 0;
    for (int i = 0; i < sizeSgTable; i++) {
        if (systemFromSgNumber(TabSg[i].SgNumber) == csys) {
            if (sgListIndex == 0) sgListIndex = i;
            sgList << QString("%1 (%2)").arg(TabSg[i].name,QString::number(TabSg[i].SgNumber));
        }
    }
    sgModel->setStringList(sgList);
}

//void SpaceGroupWidget::on_spacegComboBox_currentIndexChanged(int index)
//{
//    int indexTab = index + sgListIndex;
//    //emit spaceGroupChanged(sgdat::TabSg[indexTab].name);
//}
